// Acquisition.cpp : implementation file
//

#include "stdafx.h"
#include "Acquisition.h"
#include "fli.h"
#include "neuron.h"
#include "beam.h"
#include "DataPointDoc.h"
#include "Microscope.h"
#include <afx.h>
#include <stdlib.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAquisitionParams

IMPLEMENT_DYNCREATE(CAcquisitionParams, CObject)

CAcquisitionParams::CAcquisitionParams()
{
	ShutterDelay=500;
	CellExposure=100;
	CellImageLaserOff=true;
	CellBinning=1;//10;
	CycleTime=5;//30;
	Height=1060;//1000;//420;//2056;//420;  
	LaserBinning=2;
	LaserExposure=10;
	SeparateLaserImage=true;
	Width=1590;//1500;//626;//3088;//626;   
	XOrigin=1000;
	YOrigin=300;
	StoreCycle=0;
	Calibration.x=277;
	Calibration.y=277;
	CellImage8Bit=false;
	LaserImage8Bit=true;
	LaserPixelShift=1;
	CellPixelShift=1;
	CellAutoContrast=false;
	LaserAutoContrast=false;
	CellContrastLevel=1000;
	LaserContrastLevel=10;
	m_LightDefault = false;
}

CAcquisitionParams::~CAcquisitionParams()
{
}

// Define assignment operator.
CAcquisitionParams &CAcquisitionParams::operator=( CAcquisitionParams &PRHS )
{
	Calibration=PRHS.Calibration;
	CellBinning=PRHS.CellBinning;
	CellExposure=PRHS.CellExposure;
	CellImageLaserOff=PRHS.CellImageLaserOff;	
	CycleTime=PRHS.CycleTime;
	Height=PRHS.Height;
	LaserBinning=PRHS.LaserBinning;
	LaserExposure=PRHS.LaserExposure;
	SeparateLaserImage=PRHS.SeparateLaserImage;
	StoreCycle=PRHS.StoreCycle;
	Width=PRHS.Width;
	XOrigin=PRHS.XOrigin;
	YOrigin=PRHS.YOrigin;    
	CellPixelShift=PRHS.CellPixelShift;
	LaserPixelShift=PRHS.LaserPixelShift;
	CellImage8Bit=PRHS.CellImage8Bit;
	LaserImage8Bit=PRHS.LaserImage8Bit;
	CellAutoContrast=PRHS.CellAutoContrast;
	LaserAutoContrast=PRHS.LaserAutoContrast;
	CellContrastLevel=PRHS.CellContrastLevel;
	LaserContrastLevel=PRHS.LaserContrastLevel;
	ShutterDelay=PRHS.ShutterDelay;
	m_LightDefault = PRHS.m_LightDefault;
    return *this;  // Assignment operator returns left side.
}

/////////////////////////////////////////////////////////////////////////////
// CAcquisitionParams serialization

void CAcquisitionParams::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar<<LaserImage8Bit;
		ar<<CellImage8Bit;
		ar<<Calibration;
		ar<<CellExposure;
		ar<<CellImageLaserOff;
		ar<<CellBinning;
		ar<<CycleTime;
		ar<<Height;
		ar<<LaserBinning;
		ar<<LaserExposure;
		ar<<SeparateLaserImage;
		ar<<Width;
		ar<<XOrigin;
		ar<<YOrigin;
	    ar<<StoreCycle;
		ar<<CellPixelShift;
		ar<<LaserPixelShift;		
		ar<<CellAutoContrast;
		ar<<LaserAutoContrast;
		ar<<CellContrastLevel;
		ar<<LaserContrastLevel;
		ar<<ShutterDelay;
	}
	else
	{
		ar>>LaserImage8Bit;
		ar>>CellImage8Bit;
		ar>>Calibration;
		ar>>CellExposure;
		ar>>CellImageLaserOff;
		ar>>CellBinning;
		ar>>CycleTime;
		ar>>Height;
		ar>>LaserBinning;
		ar>>LaserExposure;
		ar>>SeparateLaserImage;
		ar>>Width;
		ar>>XOrigin;
		ar>>YOrigin;
	    ar>>StoreCycle;
		ar>>CellPixelShift;
		ar>>LaserPixelShift;		
		ar>>CellAutoContrast;
		ar>>LaserAutoContrast;
		ar>>CellContrastLevel;
		ar>>LaserContrastLevel;
		ar>>ShutterDelay;		
	}	
}

/////////////////////////////////////////////////////////////////////////////
// CAcquisition
//c:\neuron\vc\apogee\ini\pci\ap2e_pci.ini
CAcquisition::CAcquisition(CNeuronApp *aparent, CMicroscope *aMicroscope)
{
	CameraExists=-1;
	HardwareAccess=false;
	TestPictureName="c:\\Neuron\\Data\\test1.bmp";
	IniFileName="c:\\Neuron\\ap9e_pci.ini";
	Microscope=aMicroscope;
	parent=aparent;	
	DataPoint=NULL;
	AcquisitionDlg=NULL;
	ParametersChanged=true;	
	Playing=false;
	NextAcquisitionTime=CTime::GetCurrentTime();		
	ForceTestPicture=false;
	FakeRunDir = parent->RootDirectory + parent->DataDirectory + "fakerun\\";
	FakeRunNr = 1;

}


void CAcquisition::InitialUpdate(bool aHardwareAccess)
{
	HardwareAccess=aHardwareAccess;
	InitCamera(IniFileName);
}

void CAcquisition::SetTestPictureName(CString aTestPictureName) {
	TestPictureName=aTestPictureName;
}

void CAcquisition::SetIniFileName(CString aIniFileName) {
	IniFileName=aIniFileName;
}

CAcquisition::~CAcquisition()
{
  CloseCamera();
}

/////////////////////////////////////////////////////////////////////////////
// CAcquisition message handlers

void CAcquisition::InitCamera(CString IniFileName)
{
	if (!HardwareAccess) return;
	//FLISetDebugLevel("192.168.1.255", 9);
	char **list;
	int i;	
	FLIList(FLIDOMAIN_USB | FLIDEVICE_CAMERA, &list);
	CameraExists=list[0] != NULL;
	if (CameraExists) {	  
  	  char camera[MAX_PATH];
	  strcpy(camera, list[0]);
	  for(i=0; i< (int) strlen(camera); i++) {
	    if(camera[i] == ';') camera[i] = '\0';
	  }
	  bool error=0;
	  error|=(0!=FLIOpen(&device, camera, FLIDEVICE_CAMERA | FLIDOMAIN_USB));
   	  error|=(0!=FLIGetVisibleArea(device, &ul_x, &ul_y, &lr_x, &lr_y));	  
	  error|=(0!=FLISetImageArea(device, ul_x, ul_y, lr_x, lr_y));
      error|=(0!=FLISetNFlushes(device, 10));      //Clear the CCD sensor x times before taking image
	  error|=(0!=FLISetTemperature(device, 15));   //in degC
	  error|=(0!=FLISetExposureTime(device, 50));  //in ms
      error|=(0!=FLISetFrameType(device, FLI_FRAME_TYPE_NORMAL));
	  error|=(0!=FLISetBitDepth(device, FLI_MODE_16BIT));
	  error|=(0!=FLISetHBin(device, 1));
	  error|=(0!=FLISetVBin(device, 1));
	  if (error) {
		  AfxMessageBox("CAcquisition : InitCamera : FLILib Error");
		  CameraExists=-1;
	  } else CameraExists=1;
	} //else AfxMessageBox( "CAcquisition : Camera not found");
    FLIFreeList(list);    

	if (CameraExists<=0) {
		HRESULT hr=cam.CreateInstance(__uuidof(Camera));
		if (!FAILED(hr)) {
			_bstr_t inifile(IniFileName);
			hr=cam->Init(inifile,-1,-1);
			if (!FAILED(hr)) {
				short present;				
				cam->get_Present(&present);
				if (present>=0) {
					cam->put_ForceShutterOpen(false);
					cam->put_LongCable(false);
					cam->put_FastReadout(false);
					cam->put_UseTrigger(false);
					//cam->put_TDI(false);//?
					cam->put_Timeout(10);//which units?
					cam->put_CoolerSetPoint(-15);//in degree Celsius					
					cam->put_CoolerMode(Camera_CoolerMode_On);  
					cam->put_BinX(1);
					cam->put_BinY(1);
					cam->put_StartX(0); //in unbinned pixel
					cam->put_StartY(0);					
					short col;
					cam->get_ImgColumns(&col);
					short row;
					cam->get_ImgRows(&row);					
					cam->put_NumX(col);
					cam->put_NumY(row);
					cam->put_ConvertShortToLong(false);//use integer array (2 bytes per pixel)
					cam->put_OptionBase(false);//start array with element 0
					cam->put_HighPriority(false);//true);//during image download
					cam->Reset();
					Camera_Status status;
					cam->get_Status(&status);
					if (status==Camera_Status_Idle) CameraExists=2; 
					else CameraExists=-1;
				} else CameraExists=-1;
			} else CameraExists=-1; 
		} else CameraExists=-1; 		
	} else cam=NULL;	
	if (CameraExists<=0) cam=NULL;
}

void CAcquisition::CloseCamera()
{
	if (HardwareAccess) {
		if (CameraExists==1) FLIClose(device);
		if (cam) cam=NULL;//this automatically releases the ActiveX object
	}
}

void CAcquisition::ConfigDlg()
{
  CAcquisitionDlg dlg;
// Initialize dialog data
	dlg.m_nCellExposure=Param.CellExposure;
	dlg.m_nCellImageLaserOff=Param.CellImageLaserOff;
	dlg.m_nCellBinning=Param.CellBinning;
	dlg.m_nCycleTime=Param.CycleTime;
	dlg.m_nHeight=Param.Height;
	dlg.m_nLaserBinning=Param.LaserBinning;
	dlg.m_nLaserExposure=Param.LaserExposure;
	dlg.m_nSeparateLaserImage=Param.SeparateLaserImage;
	dlg.m_nWidth=Param.Width;
	dlg.m_nXOrigin=Param.XOrigin;
	dlg.m_nYOrigin=Param.YOrigin;
	dlg.m_CalibrationX=Param.Calibration.x;
	dlg.m_CalibrationY=Param.Calibration.y;
	dlg.m_StoreCycle=Param.StoreCycle;
	dlg.m_CellImage8Bit=Param.CellImage8Bit;
	dlg.m_LaserImage8Bit=Param.LaserImage8Bit;	
	dlg.m_CellPixelShift=Param.CellPixelShift;
	dlg.m_LaserPixelShift=Param.LaserPixelShift;
	dlg.m_CellAutoContrast=Param.CellAutoContrast;
	dlg.m_LaserAutoContrast=Param.LaserAutoContrast;
	dlg.m_CellContrastLevel=Param.CellContrastLevel;
	dlg.m_LaserContrastLevel=Param.LaserContrastLevel;
	dlg.m_ShutterDelay=Param.ShutterDelay;
	dlg.m_LightDefault = Param.m_LightDefault;

// Invoke the dialog box
	if (dlg.DoModal() == IDOK)
		{
	// retrieve the dialog data
		Param.CellExposure=dlg.m_nCellExposure;
		Param.CellImageLaserOff=dlg.m_nCellImageLaserOff;
		Param.CellBinning=dlg.m_nCellBinning;
		Param.CycleTime=dlg.m_nCycleTime;
		Param.Height=dlg.m_nHeight;
		Param.LaserBinning=dlg.m_nLaserBinning;
		Param.LaserExposure=dlg.m_nLaserExposure;
		Param.SeparateLaserImage=dlg.m_nSeparateLaserImage;
		Param.Width=dlg.m_nWidth;
		Param.XOrigin=dlg.m_nXOrigin;
		Param.YOrigin=dlg.m_nYOrigin;
		Param.StoreCycle=dlg.m_StoreCycle;
		Param.Calibration.x=dlg.m_CalibrationX;
		Param.Calibration.y=dlg.m_CalibrationY;
		Param.CellImage8Bit=dlg.m_CellImage8Bit;
	    Param.LaserImage8Bit=dlg.m_LaserImage8Bit;
		Param.CellPixelShift=dlg.m_CellPixelShift;
		Param.LaserPixelShift=dlg.m_LaserPixelShift;
		Param.CellAutoContrast=dlg.m_CellAutoContrast;
		Param.LaserAutoContrast=dlg.m_LaserAutoContrast;
		Param.CellContrastLevel=dlg.m_CellContrastLevel;
		Param.LaserContrastLevel=dlg.m_LaserContrastLevel;
		Param.ShutterDelay=dlg.m_ShutterDelay;
		Param.m_LightDefault = dlg.m_LightDefault;
		ParametersChanged=true;
	}
}
/////////////////////////////////////////////////////////////////////////////
// CAcquisitionDlg dialog


CAcquisitionDlg::CAcquisitionDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CAcquisitionDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CAcquisitionDlg)
	m_nCellExposure = 0;
	m_nCellImageLaserOff = FALSE;
	m_nCellBinning = 0;
	m_nCycleTime = 0;
	m_nHeight = 0;
	m_nLaserBinning = 0;
	m_nLaserExposure = 0;
	m_nSeparateLaserImage = FALSE;
	m_nWidth = 0;
	m_nXOrigin = 0;
	m_nYOrigin = 0;
	m_CalibrationX = 0;
	m_CalibrationY = 0;
	m_StoreCycle = 0;
	m_CellImage8Bit = FALSE;
	m_LaserImage8Bit = FALSE;
	m_CellPixelShift = 0;
	m_LaserPixelShift = 0;
	m_CellAutoContrast = FALSE;
	m_LaserAutoContrast = FALSE;
	m_CellContrastLevel = 0;
	m_LaserContrastLevel = 0;
	m_ShutterDelay = 0;
	m_LightDefault = FALSE;
	//}}AFX_DATA_INIT
}


void CAcquisitionDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAcquisitionDlg)
	DDX_Text(pDX, IDC_CELLEXPOSURE, m_nCellExposure);
	DDV_MinMaxInt(pDX, m_nCellExposure, 10, 10000);
	DDX_Check(pDX, IDC_CELLIMAGELASEROFF, m_nCellImageLaserOff);
	DDX_Text(pDX, IDC_CELLBINNING, m_nCellBinning);
	DDV_MinMaxLong(pDX, m_nCellBinning, 1, 16);
	DDX_Text(pDX, IDC_CYCLETIME, m_nCycleTime);
	DDV_MinMaxLong(pDX, m_nCycleTime, 1, 600);
	DDX_Text(pDX, IDC_HEIGHT, m_nHeight);
	DDV_MinMaxLong(pDX, m_nHeight, 1, 65535);
	DDX_Text(pDX, IDC_LASERBINNING, m_nLaserBinning);
	DDV_MinMaxLong(pDX, m_nLaserBinning, 1, 16);
	DDX_Check(pDX, IDC_SEPARATELASERIMAGE, m_nSeparateLaserImage);
	DDX_Text(pDX, IDC_WIDTH, m_nWidth);
	DDV_MinMaxLong(pDX, m_nWidth, 1, 65535);
	DDX_Text(pDX, IDC_XORIGIN, m_nXOrigin);
	DDV_MinMaxLong(pDX, m_nXOrigin, 0, 65535);
	DDX_Text(pDX, IDC_YORIGIN, m_nYOrigin);
	DDV_MinMaxLong(pDX, m_nYOrigin, 0, 65535);
	DDX_Text(pDX, IDC_CALIBRATIONX, m_CalibrationX);
	DDX_Text(pDX, IDC_CALIBRATIONY, m_CalibrationY);
	DDX_Text(pDX, IDC_STORECYCLE, m_StoreCycle);
	DDX_Check(pDX, IDC_CELLIMAGE8BIT, m_CellImage8Bit);
	DDX_Check(pDX, IDC_LASERIMAGE8BIT, m_LaserImage8Bit);
	DDX_Text(pDX, IDC_CELLPIXELSHIFT, m_CellPixelShift);
	DDV_MinMaxInt(pDX, m_CellPixelShift, 0, 15);
	DDX_Text(pDX, IDC_LASERPIXELSHIFT, m_LaserPixelShift);
	DDV_MinMaxInt(pDX, m_LaserPixelShift, 0, 15);
	DDX_Check(pDX, IDC_CELLAUTOCONTRAST, m_CellAutoContrast);
	DDX_Check(pDX, IDC_LASERAUTOCONTRAST, m_LaserAutoContrast);
	DDX_Text(pDX, IDC_CELLCONTRASTLEVEL, m_CellContrastLevel);
	DDV_MinMaxLong(pDX, m_CellContrastLevel, 0, 1000000);
	DDX_Text(pDX, IDC_LASERCONTRASTLEVEL, m_LaserContrastLevel);
	DDV_MinMaxLong(pDX, m_LaserContrastLevel, 0, 1000000);
	DDX_Text(pDX, IDC_SHUTTERDELAY, m_ShutterDelay);
	DDV_MinMaxLong(pDX, m_ShutterDelay, 0, 10000);
	DDX_Check(pDX, IDC_LIGHT, m_LightDefault);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAcquisitionDlg, CDialog)
	//{{AFX_MSG_MAP(CAcquisitionDlg)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAcquisitionDlg message handlers



bool CAcquisition::IsPlaying()
{
   return Playing;
}

void CAcquisition::Play()
{
	if (!Playing) {
		Playing=true;	
		NextAcquisitionTime=CTime::GetCurrentTime();
		OldAcquisitionTime=NextAcquisitionTime;
		CTimeSpan Delta=CTimeSpan::CTimeSpan(0,0,0,1);
		NextAcquisitionTime+=Delta;		
	} else {
		Playing=false;
	}
}

void CAcquisition::OnIdle()
{
	if (Playing) {					
			CTime Now=CTime::GetCurrentTime();		
			if (Now>NextAcquisitionTime) {	
				SetCursor(AfxGetApp()->LoadStandardCursor(IDC_WAIT));
				CTimeSpan Delta=CTimeSpan::CTimeSpan(0,0,0,Param.CycleTime);
				CTimeSpan TimeSinceLast=Now-OldAcquisitionTime;
				OldAcquisitionTime=Now;
				NextAcquisitionTime+=Delta;							
				if (CameraExists==2) {
					short col;
					short row;
					cam->get_ImgColumns(&col);						
					cam->get_ImgRows(&row);
					if (((Param.Width+Param.XOrigin)>col) || ((Param.Height+Param.YOrigin)>row)) {
						Param.Height=row;
						Param.Width=col;							
						Param.XOrigin=0;
						Param.YOrigin=0;						
					}
					int timeout=0;
					bool error;
					do {						
						timeout++;
						if ((Param.XOrigin % Param.LaserBinning)!=0) Param.XOrigin= (1+(Param.XOrigin/Param.LaserBinning))*Param.LaserBinning;
						if ((Param.YOrigin % Param.LaserBinning)!=0) Param.YOrigin= (1+(Param.YOrigin/Param.LaserBinning))*Param.LaserBinning;
						if ((Param.XOrigin % Param.CellBinning)!=0) Param.XOrigin= (1+(Param.XOrigin/Param.CellBinning))*Param.CellBinning;
						if ((Param.YOrigin % Param.CellBinning)!=0) Param.YOrigin= (1+(Param.YOrigin/Param.CellBinning))*Param.CellBinning;
						if ((Param.Width % Param.LaserBinning)!=0) Param.Width= (-1+(Param.Width/Param.LaserBinning))*Param.LaserBinning;
						if ((Param.Height % Param.LaserBinning)!=0) Param.Height= (-1+(Param.Height/Param.LaserBinning))*Param.LaserBinning;
						if ((Param.Width % Param.CellBinning)!=0) Param.Width= (-1+(Param.Width/Param.CellBinning))*Param.CellBinning;
						if ((Param.Height % Param.CellBinning)!=0) Param.Height= (-1+(Param.Height/Param.CellBinning))*Param.CellBinning;						
						error=((Param.XOrigin % Param.LaserBinning)!=0) || ((Param.YOrigin % Param.LaserBinning)!=0) ||
							  ((Param.XOrigin % Param.CellBinning)!=0) || ((Param.YOrigin % Param.CellBinning)!=0) ||
							  ((Param.Width % Param.LaserBinning)!=0) || ((Param.Height % Param.LaserBinning)!=0) ||
							  ((Param.Width % Param.CellBinning)!=0) || ((Param.Height % Param.CellBinning)!=0);
					} while (error && (timeout<10));					
					if (error) AfxMessageBox("CAquisition: Image Origin, Size and Binning not compatible");
				}
				if (DataPoint!=NULL) delete DataPoint;
				DataPoint=new CDataPointDoc(Param);
				DataPoint->Active=true;
				bool OldBeamStatus;
				if (Param.CellImageLaserOff) {
					OldBeamStatus=parent->BeamGuiding->IsLaserOn();
					parent->BeamGuiding->SwitchBeam(false);					
				}	
				double LaserExpTime;
				if (Param.LaserExposure<10) Param.LaserExposure=10;
				if (parent->BeamGuiding) {
					LaserExpTime=parent->BeamGuiding->TotalTime;															
				} else LaserExpTime=Param.LaserExposure;
				bool TakeLaserImage=Param.SeparateLaserImage && (CameraExists>0) && (LaserExpTime>=0.01) && OldBeamStatus;				
				bool CellImageTaken=false;
				if (parent->Fakerun)
				{
					CString cellimage;
					cellimage.Format("%s\\Cell%04i.TGA", FakeRunDir, FakeRunNr);
					CellImageTaken = DataPoint->CellImage->LoadFromTGA(cellimage);
					FakeRunNr++;
					//CellImageTaken=true;
				}
				else if ((CameraExists>0) && (!ForceTestPicture)) {
					Microscope->SwitchLight(true);
					Delay(200);
					bool error=false;
					if (CameraExists==1) {
						size_t *Pbytesgrabbed;
						size_t bytesgrabbed;
						Pbytesgrabbed=&bytesgrabbed;
						error|=(0!=FLISetImageArea(device, Param.XOrigin, Param.YOrigin,Param.XOrigin+Param.Width, Param.YOrigin+Param.Height));      
						error|=(0!=FLISetExposureTime(device, Param.CellExposure));  //in ms
						error|=(0!=FLISetHBin(device, Param.CellBinning));
						error|=(0!=FLISetVBin(device, Param.CellBinning));					
						error|=(0!=FLIGrabFrame(device, DataPoint->CellImage->bitmap,DataPoint->CellImage->Height*DataPoint->CellImage->Width,
							Pbytesgrabbed));
						if (error || (0!=bytesgrabbed-2*DataPoint->CellImage->Height*DataPoint->CellImage->Width)) {
							AfxMessageBox("CAcquisition : OnIdle CellImage : FLILib Error");
							CameraExists=false;
						}
					} else if (CameraExists==2) {			
						cam->Reset();
						cam->Flush(-1);
						long timeout=0;
						while ((cam->Status!=Camera_Status_Idle) && (timeout++<10000)) Delay(1);
						cam->put_BinX(Param.CellBinning);
						cam->put_BinY(Param.CellBinning);
						cam->put_StartX(Param.XOrigin); //in unbinned pixel
						cam->put_StartY(Param.YOrigin);					
						
						short col;
						short row;
						cam->get_ImgColumns(&col);						
						cam->get_ImgRows(&row);						
						short lrow=row/Param.LaserBinning;
						short lcol=col/Param.LaserBinning;
						col/=Param.CellBinning;
						row/=Param.CellBinning;
						short setcol=Param.Width;												
						short setrow=Param.Height;						 
						setcol/=Param.CellBinning;
						setrow/=Param.CellBinning;												
						cam->put_NumX(setcol);
						cam->put_NumY(setrow);					
						if (Param.CellExposure<10) Param.CellExposure=10;
						if (!FAILED(cam->Expose(0.001*Param.CellExposure,true))) {
							long timeout=0;
							while ((cam->Status!=Camera_Status_ImageReady) && (timeout++<10000)) Delay(1);
							if (TakeLaserImage) Microscope->SwitchLight(false);															
							if (Param.CellImageLaserOff) parent->BeamGuiding->SwitchBeam(OldBeamStatus);							
							if (!FAILED(cam->GetImage((long)DataPoint->CellImage->bitmap))) {
								CellImageTaken=true;
								DataPoint->CellImage->ShiftPixelBits(Param.CellPixelShift);														
								DataPoint->CellImage->FlipVertical();
							}
						}
					}
				} else {
					if ((TestPictureName.GetLength()>0) && (!DataPoint->CellImage->LoadFromBMP(TestPictureName,2,false))) { 											
						unsigned short * h=DataPoint->CellImage->bitmap;
						int x0=rand()/16;
						int y0=rand()/16;
						for (int x=x0;x<x0+DataPoint->CellImage->Height;x++)
							for (int y=y0;y<y0+DataPoint->CellImage->Width;y++) {
								unsigned int help=256*x+256*y;
								while (help>65535) help-=65536;							
								*h=help;
								h++;
							}						
					}
					CellImageTaken=true;
				}
				if (!CellImageTaken) DataPoint->CellImage->Clear();
				DataPoint->CellImage->EffectiveBitPerPixel=(Param.CellImage8Bit) ? 8 : 16;
				if (Param.CellImageLaserOff) parent->BeamGuiding->SwitchBeam(OldBeamStatus);
				bool LaserImageTaken=false;
				if (Param.SeparateLaserImage && parent->Fakerun)
				{
					CString laser;
					laser.Format("%s\\Laser%04i.BMP", FakeRunDir, FakeRunNr);
					LaserImageTaken = DataPoint->LaserImage->LoadFromBMP(laser,2,false);
					FakeRunNr++;
					//LaserImageTaken=true;
				}
				else if (Param.SeparateLaserImage && parent->BeamGuiding->IsLaserOn()) {					
					if (CameraExists) {
						if (TakeLaserImage) parent->BeamGuiding->SetBeamIntense(false);
						Microscope->SwitchLight(false);
						Delay(Param.ShutterDelay);
						if (CameraExists==1) {
							bool error=false;
							size_t *Pbytesgrabbed;
							size_t bytesgrabbed;
							Pbytesgrabbed=&bytesgrabbed;
							error|=(0!=FLISetImageArea(device, Param.XOrigin, Param.YOrigin,Param.XOrigin+Param.Width, Param.YOrigin+Param.Height));      
							error|=(0!=FLISetExposureTime(device, Param.LaserExposure));  //in ms
							error|=(0!=FLISetHBin(device, Param.LaserBinning));
							error|=(0!=FLISetVBin(device, Param.LaserBinning));					
							error|=(0!=FLIGrabFrame(device, DataPoint->LaserImage->bitmap,DataPoint->LaserImage->Height*DataPoint->LaserImage->Width,
								Pbytesgrabbed));
							if (error || (0!=bytesgrabbed-2*DataPoint->LaserImage->Height*DataPoint->LaserImage->Width)) {
								AfxMessageBox("CAcquisition : OnIdle LaserImage : FLILib Error");
								CameraExists=false;
							}
						} else if (CameraExists==2) {							
							cam->Reset();
							cam->Flush(-1);
							long timeout=0;
							while ((cam->Status!=Camera_Status_Idle) && (timeout++<10000)) Delay(1);
							cam->put_BinX(Param.LaserBinning);
							cam->put_BinY(Param.LaserBinning);
							cam->put_StartX(Param.XOrigin); //in unbinned pixel
							cam->put_StartY(Param.YOrigin);												
							short setcol=Param.Width;												
							short setrow=Param.Height;						 
							setcol/=Param.LaserBinning;
							setrow/=Param.LaserBinning;														
							cam->put_NumX(setcol);
							cam->put_NumY(setrow);
							short helpx,helpy,helpBX;
							cam->get_NumX(&helpx);
							cam->get_NumY(&helpy);
							cam->get_BinX(&helpBX);
							if (LaserExpTime>=0.01) {
								if (!FAILED(cam->Expose(0.001*LaserExpTime,true))) {
									long timeout=0;
									while ((cam->Status!=Camera_Status_ImageReady) && (timeout++<10000)) Delay(1);		
									Microscope->SwitchLight(true);
									parent->BeamGuiding->SetBeamIntense(true);
									if (!FAILED(cam->GetImage((long)DataPoint->LaserImage->bitmap))) {	
										LaserImageTaken=true;
										DataPoint->LaserImage->ShiftPixelBits(Param.LaserPixelShift);
										DataPoint->LaserImage->FlipVertical();
									}
								}
							}
						}
					} else {
						unsigned short * h=DataPoint->LaserImage->bitmap;
						int x0=rand()/16;
						int y0=rand()/16;
						for (int x=x0;x<x0+DataPoint->LaserImage->Height;x++)
						for (int y=y0;y<y0+DataPoint->LaserImage->Width;y++) {
							unsigned int help=256*x;
							while (help>65535) help-=65536;
							*h=help;
							h++;
						}
						LaserImageTaken=true;
					}
					DataPoint->LaserImage->EffectiveBitPerPixel=(Param.LaserImage8Bit) ? 8 : 16;
				}				
				parent->BeamGuiding->SetBeamIntense(true);
				Microscope->SwitchLight(Param.m_LightDefault);
				if (!LaserImageTaken && DataPoint->LaserImage) {
					delete DataPoint->LaserImage;
					DataPoint->LaserImage=NULL;
				}
				parent->NewDataPoint(DataPoint,TimeSinceLast.GetTotalSeconds());
				DataPoint=NULL;
				Now=CTime::GetCurrentTime();		
				if (Now>NextAcquisitionTime) {
					//AfxMessageBox("CAcquisition : OnIdle : Acquisition too slow for cycle time");										
					NextAcquisitionTime+=Delta;
					while (Now>NextAcquisitionTime) {
						Now=CTime::GetCurrentTime();		
						NextAcquisitionTime+=Delta;		
					}
				}
				SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
			}
		}
}

void CAcquisition::SerializeParams(CArchive &ar)
{
	if (ar.IsStoring())
	{			
		ar<<TestPictureName;
		if (ForceTestPicture) ar<<'Y'; else ar<<'N';
		Param.Serialize(ar);
	} else {		
		ar>>TestPictureName;	
		char help;
		ar>>help;
		ForceTestPicture=help=='Y';
		Param.Serialize(ar);
	}
}


