// Beam.cpp : implementation file
//

#include "stdafx.h"
#include "Neuron.h"
#include "Beam.h"
#include "DataPointDoc.h"
#include "DataPointView.h"
#include "NIDAQEX.h"
#include "spline.h"
#include "MainFrm.h"
#include "math.h"
#include "Beam.h"
#include <fstream.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



/////////////////////////////////////////////////////////////////////////////
// CBeamGuiding

const i16 iNumChans = 4;
static i16 piChanVect[iNumChans] = {0,1,2,3};
const double pi = 3.1415926535;
const SplineGraining=1.0;

i16 iDevice = 1;
i16 iPort = 0;
i16 iBeamIntensityLine = 1;

   
CBeamGuiding::CBeamGuiding(CNeuronApp *aparent)
{
	BeamPathChanged=false;
	HardwareAccess=false;
	parent=aparent;
	BeamOn=false;
	iDevice = 1;
	Voltage0=0;
	Voltage1=0;
	Voltage2=0;
	Voltage3=0;
	OffVoltage0=-5;
	OffVoltage1=0;
	OffVoltage2=-5;
	OffVoltage3=0;
	BeamPath=NULL;
	BeamPathBuffer=NULL;
	BeamPathBufferLength=0;		
	//Frequencies in Hz
	Frequency = 100000.0;	
	WobbleXFrequency=100.0;
	WobbleYFrequency=100.0;
	TotalTime=0;
	ulCount = 0;
	piBuffer = NULL;
	ScanningModeOn=false;	
	SinusSweep=false;
	AntiHysteresis=false;
	DAC0Frequency=300;
	DAC2Frequency=300;
	DAC1Frequency=60;
	DAC3Frequency=60;
	DAC0Amplitude=3;
	DAC1Amplitude=1;
	DAC2Amplitude=3;
	DAC3Amplitude=1;
	AntiHysteresisAfterHop=false;
	DAC0FrequencyAfterHop=300;
	DAC2FrequencyAfterHop=300;
	DAC1FrequencyAfterHop=60;
	DAC3FrequencyAfterHop=60;
	DAC0AmplitudeAfterHop=3;
	DAC1AmplitudeAfterHop=1;
	DAC2AmplitudeAfterHop=3;
	DAC3AmplitudeAfterHop=1;
	AntiHysteresisDecayTime=100;
	AntiHysteresisHopToCenter=false;
	AntiHysteresisHopAlsoForOneStop=false;
	AntiHysteresisHopRestingTime=100;
	FixPointVoltageDAC0=5;
	FixPointVoltageDAC1=5;
	FixPointVoltageDAC2=5;
	FixPointVoltageDAC3=5;
	InsertTime=0;
	AktHop=0;
	ParabolicHopTime=1;
	LaserCellOverlap=3000; // Laser Cell Overlap in nm
	ScanSpeed=100000; //in nm per second
}

void CBeamGuiding::InitialUpdate(bool aHardwareAccess)
{
	HardwareAccess=aHardwareAccess;
	SwitchBeam(false);
	i16 iStatus = 0;
    i16 iRetVal = 0;  
    i16 iDir = 1;
    i16 iStateON = 1;
    i16 iStateOFF = 0;
    i16 iIgnoreWarning = 0;    
	if (HardwareAccess) {
		iStatus = DIG_Line_Config(iDevice, iPort, iBeamIntensityLine, iDir);
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::InitialUpdate DIG_Line_Config", iIgnoreWarning);
	}	
	SetBeamIntense(true);
}

CBeamGuiding::~CBeamGuiding()
{
	SetBeamIntense(false);
	SwitchBeam(false);
	if (BeamPath) delete BeamPath;
	if (BeamPathBuffer) delete BeamPathBuffer;
	if (piBuffer) delete piBuffer;
}

void CBeamGuiding::SwitchScanningModeOn(u32 TransferBufferSize, i16 *TransferBuffer, bool NewBuffer) {
	if (!HardwareAccess) return;
	if (TransferBuffer==NULL) return;
	bool TransferNeeded=NewBuffer;
	if (TransferBufferSize!=ulCount) {
		if (ScanningModeOn) SwitchScanningModeOff();	
		if (piBuffer) delete piBuffer;
		ulCount=TransferBufferSize;
		piBuffer=new i16[ulCount];				
		TransferNeeded=true;
	}
	if (TransferNeeded) {
		i16 *tb=&TransferBuffer[TransferBufferSize-1];
		i16 *pi=&piBuffer[ulCount-1];
		for (u32 i=0;i<TransferBufferSize;i++) {
			*pi=*tb;
			pi--;
			tb--;
		}		
	}
	i16 iStatus = 0;
	i16 iRetVal = 0;
	i16 iIgnoreWarning = 0;
	i16 iUpdateTB = 0;
	u32 ulUpdateInt = 0;
	if (!ScanningModeOn) {		
		iStatus = Timeout_Config(iDevice, 180); //Timeout=(#Sec * 18ticks/Sec)
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SwitchScanningModeOn Timeout_Config", iIgnoreWarning);
		iStatus = WFM_DB_Config(iDevice, iNumChans, piChanVect,1,0,0);
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SwitchScanningModeOn WFM_DB_Config", iIgnoreWarning);
		iStatus = WFM_Group_Setup(iDevice, iNumChans, piChanVect, 1);
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SwitchScanningModeOn WFM_Group_Setup", iIgnoreWarning);
		iStatus = WFM_Load(iDevice, 4, piChanVect, piBuffer, ulCount, 0, 0);
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SwitchScanningModeOn WFM_Load", iIgnoreWarning);
		iStatus = WFM_Rate(Frequency, 0, &iUpdateTB, &ulUpdateInt);
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SwitchScanningModeOn WFM_Rate", iIgnoreWarning);
		iStatus = WFM_ClockRate(iDevice, 1, 0,iUpdateTB, ulUpdateInt,0);
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SwitchScanningModeOn WFM_ClockRate", iIgnoreWarning);
		iStatus = WFM_Group_Control(iDevice, 1, 1);        
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SwitchScanningModeOn WFM_Group_Control/START", iIgnoreWarning);
	}
	/*
	//Doublebuffering not used to increase copy speed	
	  if (TransferNeeded) {
		iStatus = WFM_DB_Transfer(iDevice, iNumChans,piChanVect, TransferBuffer, ulCount/2);
		iRetVal = NIDAQErrorHandler(iStatus,"CBeamGuiding::SwitchScanningModeOn WFM_DB_Transfer", iIgnoreWarning);
		iStatus = WFM_DB_Transfer(iDevice, iNumChans,piChanVect, TransferBuffer, ulCount/2);
		iRetVal = NIDAQErrorHandler(iStatus,"CBeamGuiding::SwitchScanningModeOn WFM_DB_Transfer", iIgnoreWarning);
	}*/
	ScanningModeOn=true;	
}

void CBeamGuiding::SwitchScanningModeOff() {
	SwitchScanningModeOffAndSetVoltages(OffVoltage0,OffVoltage1,OffVoltage2,OffVoltage3);	
}

void CBeamGuiding::SwitchScanningModeOffAndSetVoltages(double aVoltage0,double aVoltage1,double aVoltage2,double aVoltage3) {
	if (!HardwareAccess) return;
	if (!ScanningModeOn) return;
	ScanningModeOn=false;	
	i16 iStatus = 0;
	iStatus = WFM_Group_Control(iDevice, 1, 0);
	iStatus = WFM_DB_Config(iDevice, 4, piChanVect, 0, 0, 0);
	Voltage0=aVoltage0;
	Voltage1=aVoltage1;
	Voltage2=aVoltage2;
	Voltage3=aVoltage3;
	SetVoltages();
}

void CBeamGuiding::SetVoltages()
{
	if (!HardwareAccess) return;
	i16 iStatus = 0;
	i16 iRetVal = 0;
	i16 iIgnoreWarning = 0;
	iStatus = AO_VWrite(iDevice, 0,Voltage0);
    iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SetVoltages AO_VWrite0", iIgnoreWarning);
	iStatus = AO_VWrite(iDevice, 1,Voltage1);
    iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SetVoltages AO_VWrite1", iIgnoreWarning);
	iStatus = AO_VWrite(iDevice, 2,Voltage2);
    iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SetVoltages AO_VWrite2", iIgnoreWarning);
	iStatus = AO_VWrite(iDevice, 3,Voltage3);
    iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SetVoltages AO_VWrite3", iIgnoreWarning);	
}

void CBeamGuiding::SwitchBeam(bool onoff)
{	
	if (BeamOn==onoff) return;
	BeamOn=onoff;
	if (!BeamOn) {
		//if (ScanningModeOn) 
			SwitchScanningModeOff(); 
		/*else {
			Voltage0=OffVoltage0;
			Voltage1=OffVoltage1;
			Voltage2=OffVoltage2;
			Voltage3=OffVoltage3;
			SetVoltages();
		}*/
	} else {
		if (BeamPathChanged) {
			CBeamPath * help=new CBeamPath(BeamPath);
			SetNewPath(help,true);
			delete help;			
		}
		if (BeamPathBuffer!=NULL) SwitchScanningModeOn(BeamPathBufferLength,BeamPathBuffer);
	}
}

bool CBeamGuiding::ToggleLaserOnOff()
{	
	SwitchBeam(!BeamOn);
	return BeamOn;
}

bool CBeamGuiding::IsLaserOn()
{
	return BeamOn;
}

void CBeamGuiding::CalcX1Y1Position(const double X2, const double Y2, double &X1, double &Y1)
{
	X1 = 0.08551 + -0.33188 * X2 + 0.01824 * Y2 + 0.00071 * X2 * Y2 + -0.00222 * X2 * X2 + -0.00222 * Y2 * Y2 + 0.0009 * X2 * X2 * X2 + 0.00003 * X2 * X2 * Y2 + 0.00036 * X2 * Y2 * Y2 + 0.00002 * Y2 * Y2 * Y2;
	Y1 = 0.10753 + -0.02043 * X2 + 0.48062 * Y2 + 0.01249 * X2 * Y2 + 0.00581 * X2 * X2 + -0.00104 * Y2 * Y2 + -0.00014 * X2 * X2 * X2 + 0.00009 * X2 * X2 * Y2 + 0.00002 * X2 * Y2 * Y2 + -0.00145 * Y2 * Y2 * Y2;
	//replaced030130 X1 = -0.05798 + -0.3366 * X2 + 0.0189 * Y2 + 0.00078 * X2 * Y2 + -0.00262 * X2 * X2 + 0.00783 * Y2 * Y2 + 0.00085 * X2 * X2 * X2 + 0.00014 * X2 * X2 * Y2 + 0.00044 * X2 * Y2 * Y2 + -0.00032 * Y2 * Y2 * Y2;
	//replaced030130 Y1 = -0.01235 + -0.02903 * X2 + 0.4887 * Y2 + 0.01481 * X2 * Y2 + 0.00477 * X2 * X2 + -0.00168 * Y2 * Y2 + 0.00024 * X2 * X2 * X2 + 0.00032 * X2 * X2 * Y2 + -0.00025 * X2 * Y2 * Y2 + -0.00144 * Y2 * Y2 * Y2;
	//X1 = -0.04384 + -0.3275 * X2 + 0.0166 * Y2 + -0.00036 * X2 * Y2 + -0.00288 * X2 * X2 + 0.00752 * Y2 * Y2 + 0.00066 * X2 * X2 * X2 + 0.0001 * X2 * X2 * Y2 + 0.00025 * X2 * Y2 * Y2 + -0.0003 * Y2 * Y2 * Y2	;
	//Y1 = 0.08942 + -0.03356 * X2 + 0.47979 * Y2 + 0.01289 * X2 * Y2 + 0.00481 * X2 * X2 + -0.00027 * Y2 * Y2 + -0.00003 * X2 * X2 * X2 + 0.0001 * X2 * X2 * Y2 + 0.00023 * X2 * Y2 * Y2 + -0.00125 * Y2 * Y2 * Y2;
	//X1=0;//-0.66//.3;//-0.25;//-0.8;
	//Y1=-0.1;//1.00///0.15;//-0.05;//0.95;
}

void CBeamGuiding::ConvertPositionToVoltage(CPoint Pos, bool TopPoint, double &X2, double &Y2)
{
	//Pos contains position in nm, 0,0 is upper left corner.
	double x=Pos.x;
	double y=Pos.y;	
	x/=1000;
	y/=1000;
	double Xpos,Ypos;	

	//The calibration for the laser position
	Xpos=x;
	Ypos=y;
	//2 lines above are meaningless statements that stand for the real beam spot calibration formulae
	//the following are in microns, and this calibration works esp. well for the 741/494/1590/1060 area of the image.
	//Xpos = 26.57462 + 1.00175 * x + -0.3004 * y + 0.00041 * x * y + -0.00003 * x * x + 0.00012 * y * y;
	//Ypos = -11.09291 + 0.14151 * x + 0.89163 * y + -0.00003 * x * y + -0.00015 * x * x + 0.00018 * y * y;
	//below is new calibtaion as of 030205
	//Xpos = 17.04996 + 0.94333 * x + -0.09223 * y + 0.00031 * x * y + -0.00002 * x * x + -0.00003 * y * y;
	//Ypos = -32.73411 + 0.13687 * x + 0.9448 * y + -0.00005 * x * y + -0.00013 * x * x + 0.00016 * y * y;
	//Xpos = -18.72902 + 1.00864 * x + -0.07574 * y + 0.00038 * x * y + -0.00004 * x * x + -0.00012 * y * y;
	//Ypos = -24.94474 + 0.12993 * x + 0.85752 * y + -5.5359E-6 * x * y + -0.00014 * x * x + 0.00023 * y * y;
	//below is new calibration as of 030223
	Xpos = 19.92018 + 0.92078 * x + -0.09201 * y + 0.00027 * x * y + 0.00002 * x * x + -0.00005 * y * y;
	Ypos = -27.26495 + 0.13814 * x + 0.89595 * y + 0.00008 * x * y + -0.00017 * x * x + 0.00015 * y * y;

	Xpos*=1000;
	Ypos*=1000;
	double PixelWidth=3000;
	double PixelHeight=2000;
	double Magnification=30;
	double width=9.0*PixelWidth*1000.0/(Magnification*2.0); //Halfwidth of ViewField in nm
	double height=9.0*PixelHeight*1000.0/(Magnification*2.0); //Halfheight of ViewField in nm
	X2=-6.1*(double)(Xpos-width)/width+1.1; // +/-4V is approx the range  //here half image
	Y2=-5.0*(double)(Ypos-height)/height-1.9; // +/-4V is approx the range
	//X2=-5.55*(double)(Xpos-width)/width+1.1; // +/-4V is approx the range  //here half image
	//Y2=-4.65*(double)(Ypos-height)/height+1.5; // +/-4V is approx the range
	//X2=-3.1*(double)(Xpos-width)/width+0; // +/-4V is approx the range  //here half image
	//Y2=-4.6*(double)(Ypos-height)/height+0; // +/-4V is approx the range
	if (X2>8) X2=0;
	if (X2<-8) X2=0;
	if (Y2>8) Y2=0;
	if (Y2<-8) Y2=0;

	//X2=0;//2;
	//Y2=0;//2;

	CString buf;
	buf.Format("x2=%.1f y2=%.1f", X2, Y2);
	((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->SetPaneText(((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->CommandToIndex(ID_INDICATOR_MESSAGE),buf);
}

void Normalize(CPoint &p) {
	double x=p.x;
	double y=p.y;
	double l=x*x+y*y;
	if (l<0.000001) l=1;
	l=sqrt(l);
	p.x=1000.0*x/l;
	p.y=1000.0*y/l;
}

void CBeamGuiding::AddToBuffer(int i,CPoint LaserPosition,bool LaserTop) {
	if (i*4+3>BeamPathBufferLength) return;
	double X1,Y1,X2,Y2;
	ConvertPositionToVoltage(LaserPosition,LaserTop,X2,Y2);
	CalcX1Y1Position(X2,Y2,X1,Y1);
	i16	DAC0,DAC1,DAC2,DAC3;
	DAC1=2048*X2/10.0;
	DAC3=2048*Y2/10.0;
	DAC0=X1*2048/10.0;
	DAC2=Y1*2048/10.0;
	AddToBuffer(i,DAC0,DAC1,DAC2,DAC3);
}

void CBeamGuiding::AddToBuffer(int i,i16 DAC0,i16 DAC1,i16 DAC2,i16 DAC3) {
	if (i*4+3>BeamPathBufferLength) return;		
	if (DAC1>2047) DAC1=2047;
	if (DAC1<-2048) DAC1=-2048;	
	if (DAC3>2047) DAC3=2047;
	if (DAC3<-2048) DAC3=-2048;						
	if (DAC0>2047) DAC0=2047;
	if (DAC0<-2048) DAC0=-2048;	
	if (DAC2>2047) DAC2=2047;
	if (DAC2<-2048) DAC2=-2048;
	BeamPathBuffer[i*4+0]=DAC0;
	BeamPathBuffer[i*4+1]=DAC1;
	BeamPathBuffer[i*4+2]=DAC2;
	BeamPathBuffer[i*4+3]=DAC3;
	if (!ParabolicHopEndFound) {
		ParabolicHopEndDAC0=DAC0;
		ParabolicHopEndDAC1=DAC1;
		ParabolicHopEndDAC2=DAC2;
		ParabolicHopEndDAC3=DAC3;
		ParabolicHopEndFound=true;
	}
}

void CBeamGuiding::AddParabolicHop() {
	if (InsertTime<=0) return;
	int di=(InsertTime/1000.0)*Frequency;
	if (ParabolicHopTime>0) {		
		double t;
		for (int i=ParabolicHopStarti;i<ParabolicHopStarti+di;i++) {
			t=i-ParabolicHopStarti;
			t/=di;
			if (t<0.5) {
				t=2*t*t;
			} else {
				t=1-t;
				t=2*t*t;
				t=1-t;
			}		
			if (i*4+3<BeamPathBufferLength) {
				BeamPathBuffer[i*4+0]=t*ParabolicHopEndDAC0+(1-t)*ParabolicHopStartDAC0;
				BeamPathBuffer[i*4+1]=t*ParabolicHopEndDAC1+(1-t)*ParabolicHopStartDAC1;
				BeamPathBuffer[i*4+2]=t*ParabolicHopEndDAC2+(1-t)*ParabolicHopStartDAC2;
				BeamPathBuffer[i*4+3]=t*ParabolicHopEndDAC3+(1-t)*ParabolicHopStartDAC3;
			}
		}		
	} else if (AntiHysteresisHopRestingTime>0) {			
		/*CPoint LaserPosition=BeamPath->Center;		
		LaserPosition+=BeamPath->Offset;						
		double X1,X2,Y1,Y2;
		ConvertPositionToVoltage(LaserPosition,false,X2,Y2);
		CalcX1Y1Position(X2,Y2,X1,Y1);	*/
		i16	DAC0,DAC1,DAC2,DAC3;
		DAC1=2048*FixPointVoltageDAC0/10.0;
		DAC3=2048*FixPointVoltageDAC1/10.0;
		DAC0=2048*FixPointVoltageDAC2/10.0;
		DAC2=2048*FixPointVoltageDAC3/10.0;	
		for (int i=ParabolicHopStarti;i<ParabolicHopStarti+di;i++) AddToBuffer(i,DAC0,DAC1,DAC2,DAC3);		
	}
	HopList[AktHop]=ParabolicHopStarti+di;
	if (AktHop<100) AktHop++;
}

void CBeamGuiding::SetNewPath(CBeamPath *NewPath, bool ForceCalculation)
{
	if (NewPath==NULL) {
		if (BeamPath) delete BeamPath;
		BeamPath=NULL;
		SwitchScanningModeOff();
		return;
	}
	InsertTime=0;
	AntiHysteresisHopToCenter=AntiHysteresisHopToCenter && (AntiHysteresisHopRestingTime!=0);
	if (AntiHysteresisHopToCenter) {
		ParabolicHopTime=0;
		InsertTime=AntiHysteresisHopRestingTime;
	}
	if (ParabolicHopTime>0) InsertTime=ParabolicHopTime;	

	HopList[100];
	AktHop=0;
	if (BeamPath) delete BeamPath;
	BeamPath=new CBeamPath(NewPath);
	if ((!ScanningModeOn) && (!ForceCalculation)) {
		BeamPathChanged=true;
		return;
	}
	BeamPathChanged=false;
	POSITION pos=NewPath->GetHeadPosition();	
	int StopNr=0;
	while (pos!=NULL) {
		CBeamStop* BeamStop=(CBeamStop*)NewPath->GetNext(pos);
		if (BeamStop->DwellTime>0) StopNr++;
	}
	if (StopNr==0) {		
		SwitchScanningModeOff();
		return;
	}
	pos=NewPath->GetHeadPosition();	
	TotalTime=0;
	while (pos!=NULL) {
		CBeamStop* BeamStop=(CBeamStop*)NewPath->GetNext(pos);
		if (BeamStop->DwellTime>0) {			
			if ((BeamStop->Range>0) && (BeamStop->ScanCurve)) {	
				double h=BeamStop->Range;
				h/=ScanSpeed;
				double TimePerPassage=1000.0*h;
				h=BeamStop->DwellTime;
				int Passages=h/TimePerPassage;
				if (Passages<=0) Passages=1;				
				if (StopNr==1) Passages=2;
				TotalTime+=TimePerPassage*Passages;
			} else TotalTime+=BeamStop->DwellTime;
		}
	}
	if ((StopNr>1) || ((StopNr==1) && AntiHysteresisHopToCenter && AntiHysteresisHopAlsoForOneStop)) TotalTime+=InsertTime*StopNr;		
	//Time in ms
	unsigned long BufferLength=Frequency*(TotalTime/1000.0)*iNumChans;
	if (BufferLength!=BeamPathBufferLength) {
		BeamPathBufferLength=BufferLength;
		if (BeamPathBuffer) delete BeamPathBuffer;
		BeamPathBuffer=new i16[BeamPathBufferLength];		
	}
	Starti=0;
	pos=NewPath->GetHeadPosition();
	bool first=true;
	CBeamStop* LastBeamStop=NULL;
	CBeamStop* BeamStop=NULL;
	bool InsertParabolicHop=false;
	ParabolicHopEndFound=true;
	while (pos!=NULL) {
		LastBeamStop=BeamStop;
		BeamStop=(CBeamStop*)NewPath->GetNext(pos);		
		if (BeamStop->DwellTime>0) {
			if (first) {		
				first=false;
			} else {
				if ((StopNr>1) && (InsertTime>0)) {
					//AddParabolicHop to next beamstop				
					ParabolicHopStarti=Starti;				
					ParabolicHopStartDAC0=BeamPathBuffer[Starti*4+0-4];
					ParabolicHopStartDAC1=BeamPathBuffer[Starti*4+1-4];
					ParabolicHopStartDAC2=BeamPathBuffer[Starti*4+2-4];
					ParabolicHopStartDAC3=BeamPathBuffer[Starti*4+3-4];
					Starti+=InsertTime*Frequency/1000.0;
					InsertParabolicHop=true;
					ParabolicHopEndFound=false;
				}
			}	
			bool LaserTop;
			CPoint Laser=GetLaserLongAxisVector(BeamStop->Position);
			if ((BeamStop->NormalDirection.x==0) && (BeamStop->NormalDirection.y==0)) LaserTop=false;
			else {
				double scalar=BeamStop->NormalDirection.x*Laser.x+BeamStop->NormalDirection.y*Laser.y;
				LaserTop=scalar<0;
			}
			double X2,Y2,X1,Y1;
			if ((BeamStop->Range>0)  && (BeamStop->ScanCurve)) {				
				double h=BeamStop->Range;
				h/=ScanSpeed;
				double TimePerPassage=1000.0*h;
				h=BeamStop->DwellTime;
				int Passages=h/TimePerPassage;
				if (Passages<=0) Passages=1;
				if (StopNr==1) Passages=2;
				double ThisStopTime=TimePerPassage*Passages;				
				Normalize(Laser);
				double lx=Laser.x;
				double ly=Laser.y;
				lx*=LaserCellOverlap/1000;
				ly*=LaserCellOverlap/1000;
				CPoint OverlapShift=CPoint::CPoint(lx,ly);				
				double Pos=0;
				bool Direction=true;
				int di=TimePerPassage*Frequency/1000.0;
				double dr=1.0/di;
				for (int n=0;n<Passages;n++) {										
					for (int i=Starti;i<Starti+di;i++) {
						//Pos sweeps triangle from 0 to 1 and back			
						double p=(SinusSweep) ? BeamStop->Range*cos(Pos*3.14159)/2.0 : BeamStop->Range*Pos-BeamStop->Range/2.0;
						CPoint LaserPosition=BeamStop->GetScanCurvePosition(p);
						if (Direction) Pos+=dr; else Pos-=dr;
						if (LaserCellOverlap>0) {
							if (LaserTop) {
								LaserPosition+=OverlapShift;
							} else {
								LaserPosition-=OverlapShift;
							}
						}
						LaserPosition.x*=NewPath->Binning;
						LaserPosition.y*=NewPath->Binning;
						LaserPosition+=NewPath->Offset;
						LaserPosition+=BeamStop->Offset;
						AddToBuffer(i,LaserPosition,LaserTop);
					}
					Starti+=di;
					Direction=!Direction;
				}
			} else {	
				CPoint LaserPosition=BeamStop->Position;
				LaserPosition.x*=NewPath->Binning;
				LaserPosition.y*=NewPath->Binning;
				LaserPosition+=NewPath->Offset;
				LaserPosition+=BeamStop->Offset;

				//CString buf;
				//	buf.Format("x2=%.1f y2=%.1f", X2, Y2);
				//if (LaserTop) buf="Laser Top"; else buf="Laser Bottom";
				//((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->SetPaneText(((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->CommandToIndex(ID_INDICATOR_MESSAGE),buf);

				ConvertPositionToVoltage(LaserPosition,LaserTop,X2,Y2);
				CalcX1Y1Position(X2,Y2,X1,Y1);

				CString buf;
				buf.Format("x1=%.1f y1=%.1f x2=%.1f y2=%.1f", X1, Y1, X2, Y2);				
				((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->SetPaneText(((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->CommandToIndex(ID_INDICATOR_MESSAGE),buf);

				//calibration:
				//x1: voltage0: +/-0.15 = 4 mym
				//y1: voltage2: +/-0.15 = 8 mym
				//size given in nm
				double SizeX=BeamStop->Size.x;
				double SizeY=BeamStop->Size.y;
				double WobbleX=fabs(SizeX/1000.0*0.15/4.0);
				double WobbleY=fabs(SizeY/1000.0*0.15/8.0);
				if (WobbleX>0.15) WobbleX=0.15;
				if (WobbleY>0.15) WobbleY=0.15;
				i16	DAC0,DAC1,DAC2,DAC3;
				DAC1=2048*X2/10.0;				
				DAC3=2048*Y2/10.0;				
				for (int i=Starti;i<Starti+BeamStop->DwellTime*Frequency/1000.0;i++) {
					DAC0=(X1+WobbleX*cos(2*pi*i*WobbleXFrequency/Frequency))*2048/10.0;					
					DAC2=(Y1+WobbleY*cos(2*pi*i*WobbleYFrequency/Frequency))*2048/10.0;
					AddToBuffer(i,DAC0,DAC1,DAC2,DAC3);					
				}
				Starti+=(BeamStop->DwellTime/1000.0)*Frequency;
			}			
		}
		if (InsertParabolicHop && ParabolicHopEndFound) {
			AddParabolicHop();		
			InsertParabolicHop=false;			
		}
	}	
	if  ((InsertTime>0) && ((StopNr>1) || ((StopNr==1) && AntiHysteresisHopToCenter && AntiHysteresisHopAlsoForOneStop))) {
		ParabolicHopEndDAC0=BeamPathBuffer[0];
		ParabolicHopEndDAC1=BeamPathBuffer[1];
		ParabolicHopEndDAC2=BeamPathBuffer[2];
		ParabolicHopEndDAC3=BeamPathBuffer[3];
		ParabolicHopStartDAC0=BeamPathBuffer[Starti*4+0-4];
		ParabolicHopStartDAC1=BeamPathBuffer[Starti*4+1-4];
		ParabolicHopStartDAC2=BeamPathBuffer[Starti*4+2-4];
		ParabolicHopStartDAC3=BeamPathBuffer[Starti*4+3-4];
		ParabolicHopStarti=Starti;
		AddParabolicHop();
		Starti+=(InsertTime/1000.0)*Frequency;
	}
	if (Starti*4<BufferLength) {
		//error, buffer underflow
		i16	DAC0,DAC1,DAC2,DAC3;
		DAC0=BeamPathBuffer[Starti*4+0-4];
		DAC1=BeamPathBuffer[Starti*4+1-4];
		DAC2=BeamPathBuffer[Starti*4+2-4];
		DAC3=BeamPathBuffer[Starti*4+3-4];
		for (int i=Starti;i<BufferLength/4;i++) {
			BeamPathBuffer[i*4+0]=DAC0;
			BeamPathBuffer[i*4+1]=DAC1;
			BeamPathBuffer[i*4+2]=DAC2;
			BeamPathBuffer[i*4+3]=DAC3;
		}
	}
	//Add anti hysteresis wiggle:
	if (AntiHysteresis) for (long i=0;i<BeamPathBufferLength/4;i++) {
		double help=DAC0Amplitude*sin(2.0*3.141592*DAC0Frequency*i/Frequency)+BeamPathBuffer[i*4+0];
		if (help>2047) help=2047;
		if (help<-2048) help=-2048;
		BeamPathBuffer[i*4+0]=help;
		help=DAC1Amplitude*sin(2.0*3.141592*DAC1Frequency*i/Frequency)+BeamPathBuffer[i*4+1];
		if (help>2047) help=2047;
		if (help<-2048) help=-2048;
		BeamPathBuffer[i*4+1]=help;
		help=DAC2Amplitude*sin(2.0*3.141592*DAC2Frequency*i/Frequency)+BeamPathBuffer[i*4+2];
		if (help>2047) help=2047;
		if (help<-2048) help=-2048;
		BeamPathBuffer[i*4+2]=help;
		help=DAC3Amplitude*sin(2.0*3.141592*DAC3Frequency*i/Frequency)+BeamPathBuffer[i*4+3];
		if (help>2047) help=2047;
		if (help<-2048) help=-2048;
		BeamPathBuffer[i*4+3]=help;
	}
	if (AntiHysteresisAfterHop && (StopNr>1) && (AktHop>0) && (AntiHysteresisDecayTime!=0)) {
		HopList[AktHop-1]-=BeamPathBufferLength/4;
		for (int j=0;j<AktHop;j++) {
			int di=(AntiHysteresisDecayTime/1000.0)*Frequency;			
			for (int ci=HopList[j];ci<HopList[j]+di;ci++) {
				if (ci*4+3<BeamPathBufferLength) {					
					int i=ci-HopList[j];
					double t=i;
					double Decay=1-t/di;					
					double help=Decay*DAC0AmplitudeAfterHop*sin(2.0*3.141592*DAC0FrequencyAfterHop*i/Frequency)+BeamPathBuffer[ci*4+0];
					if (help>2047) help=2047;
					if (help<-2048) help=-2048;
					BeamPathBuffer[ci*4+0]=help;
					help=Decay*DAC1AmplitudeAfterHop*sin(2.0*3.141592*DAC1FrequencyAfterHop*i/Frequency)+BeamPathBuffer[ci*4+1];
					if (help>2047) help=2047;
					if (help<-2048) help=-2048;
					BeamPathBuffer[ci*4+1]=help;
					help=Decay*DAC2AmplitudeAfterHop*sin(2.0*3.141592*DAC2FrequencyAfterHop*i/Frequency)+BeamPathBuffer[ci*4+2];
					if (help>2047) help=2047;
					if (help<-2048) help=-2048;
					BeamPathBuffer[ci*4+2]=help;
					help=Decay*DAC3AmplitudeAfterHop*sin(2.0*3.141592*DAC3FrequencyAfterHop*i/Frequency)+BeamPathBuffer[ci*4+3];
					if (help>2047) help=2047;
					if (help<-2048) help=-2048;
					BeamPathBuffer[ci*4+3]=help;
				}
			}
		}
	}				
	if (AntiHysteresisHopToCenter && (StopNr==1) && (!AntiHysteresisHopAlsoForOneStop)) {
		/*CPoint LaserPosition=BeamPath->Center;		
		LaserPosition+=NewPath->Offset;				
		double X1,X2,Y1,Y2;
		ConvertPositionToVoltage(LaserPosition,false,X2,Y2);
		CalcX1Y1Position(X2,Y2,X1,Y1);*/
		SwitchScanningModeOffAndSetVoltages(FixPointVoltageDAC0,FixPointVoltageDAC1,FixPointVoltageDAC2,FixPointVoltageDAC3); //X1,X2,Y1,Y2);
		Delay(AntiHysteresisHopRestingTime);
	}
	if (BeamOn) SwitchScanningModeOn(BeamPathBufferLength,BeamPathBuffer);
/*	if (BeamPathBuffer) {
		ofstream out("c:\\Neuron\\data\\BeamGuiding.dat");
		for (int l=0;l<BeamPathBufferLength/4;l+=10) {
			if (l*4+3<BeamPathBufferLength) out<<l<<" "
				<<BeamPathBuffer[l*4+0]<<" "
				<<BeamPathBuffer[l*4+1]<<" "	
				<<BeamPathBuffer[l*4+2]<<" "	
				<<BeamPathBuffer[l*4+3]<<" "	
				<<endl;
		}
	}	*/
}

void CBeamGuiding::SerializeParams(CArchive &ar)
{
	if (ar.IsStoring()) {			
		if (SinusSweep) ar<<'Y'; else ar<<'N';
		ar<<Frequency;
		ar<<OffVoltage0;
		ar<<OffVoltage2;
		ar<<WobbleXFrequency;
		ar<<WobbleYFrequency;
		ar<<ParabolicHopTime;
		ar<<LaserCellOverlap;
		ar<<ScanSpeed;
		if (AntiHysteresis) ar<<'Y'; else ar<<'N';
		ar<<DAC0Frequency;
		ar<<DAC2Frequency;
		ar<<DAC1Frequency;
		ar<<DAC3Frequency;
		ar<<DAC0Amplitude;
		ar<<DAC1Amplitude;
		ar<<DAC2Amplitude;
		ar<<DAC3Amplitude;
		if (AntiHysteresisAfterHop) ar<<'Y'; else ar<<'N';
		ar<<DAC0FrequencyAfterHop;
		ar<<DAC2FrequencyAfterHop;
		ar<<DAC1FrequencyAfterHop;
		ar<<DAC3FrequencyAfterHop;
		ar<<DAC0AmplitudeAfterHop;
		ar<<DAC1AmplitudeAfterHop;
		ar<<DAC2AmplitudeAfterHop;
		ar<<DAC3AmplitudeAfterHop;
		ar<<AntiHysteresisDecayTime;
		ar<<AntiHysteresisHopToCenter;
		ar<<AntiHysteresisHopAlsoForOneStop;
		ar<<AntiHysteresisHopRestingTime;
		ar<<FixPointVoltageDAC0;
		ar<<FixPointVoltageDAC1;
		ar<<FixPointVoltageDAC2;
		ar<<FixPointVoltageDAC3;		
	} else {		
		char help;
		ar>>help;
		SinusSweep=help=='Y';
		ar>>Frequency;
		ar>>OffVoltage0;
		ar>>OffVoltage2;
		ar>>WobbleXFrequency;
		ar>>WobbleYFrequency;
		ar>>ParabolicHopTime;
		ar>>LaserCellOverlap;		
		ar>>ScanSpeed;
		ar>>help;
		AntiHysteresis=help=='Y';
		ar>>DAC0Frequency;
		ar>>DAC2Frequency;
		ar>>DAC1Frequency;
		ar>>DAC3Frequency;
		ar>>DAC0Amplitude;
		ar>>DAC1Amplitude;
		ar>>DAC2Amplitude;
		ar>>DAC3Amplitude;
		ar>>help;
		AntiHysteresisAfterHop=help=='Y';
		ar>>DAC0FrequencyAfterHop;
		ar>>DAC2FrequencyAfterHop;
		ar>>DAC1FrequencyAfterHop;
		ar>>DAC3FrequencyAfterHop;
		ar>>DAC0AmplitudeAfterHop;
		ar>>DAC1AmplitudeAfterHop;
		ar>>DAC2AmplitudeAfterHop;
		ar>>DAC3AmplitudeAfterHop;
		ar>>AntiHysteresisDecayTime;
		ar>>AntiHysteresisHopToCenter;
		ar>>AntiHysteresisHopAlsoForOneStop;
		ar>>AntiHysteresisHopRestingTime;
		ar>>FixPointVoltageDAC0;
		ar>>FixPointVoltageDAC1;
		ar>>FixPointVoltageDAC2;
		ar>>FixPointVoltageDAC3;		
	}
}

/////////////////////////////////////////////////////////////////////////////
// CBeamGuiding message handlers
/////////////////////////////////////////////////////////////////////////////
// CBeamPath

IMPLEMENT_SERIAL(CBeamPath,CDataPointObjectList,1)

CBeamPath::CBeamPath()
{
//	CDataPointObjectList::CDataPointObjectList();
	BeamOn=false;
	MaxNumber=0;
	Offset=CPoint::CPoint(0,0);
	Center=CPoint::CPoint(0,0);
	Binning=1;
}

CBeamPath::CBeamPath(CBeamPath* OrgPath)
{	
	POSITION pos=OrgPath->GetHeadPosition();	
	while (pos!=NULL) {
		CBeamStop* help=((CBeamStop*)OrgPath->GetNext(pos));		
		AddTail(new CBeamStop(help));	
	}	
	MaxNumber=OrgPath->MaxNumber;
	Offset=OrgPath->Offset;
	Center=OrgPath->Center;
	Binning=OrgPath->Binning;
}

CBeamPath::~CBeamPath()
{
}

void CBeamPath::Serialize(CArchive& ar)
{
	CDataPointObjectList::Serialize(ar);
	if (ar.IsStoring())
	{
		ar<<MaxNumber;
		ar<<Offset;
		ar<<Center;
		ar<<Binning;
		if (BeamOn) ar<<'Y'; else ar<<'N';
	}
	else
	{
		ar>>MaxNumber;
		ar>>Offset;
		ar>>Center;
		ar>>Binning;
		char help;
		ar>>help;
		BeamOn=help=='Y';
	}	
}

/////////////////////////////////////////////////////////////////////////////
// CBeamPath message handlers
/////////////////////////////////////////////////////////////////////////////
// CBeamStop

IMPLEMENT_SERIAL( CBeamStop, CDataPointObject,1 )

CBeamStop::CBeamStop(CPoint aPosition,CPoint aDirection,CPoint aRealSize,CBeamStopDlg &BeamStopParamsTemplate)
{
	Direction.x=BeamStopParamsTemplate.m_DirectionX;
	Direction.y=BeamStopParamsTemplate.m_DirectionY;	
	DwellTime=BeamStopParamsTemplate.m_DwellTime;
	Position.x=BeamStopParamsTemplate.m_PositionX;
	Position.y=BeamStopParamsTemplate.m_PositionY;
	Size.x=BeamStopParamsTemplate.m_SizeX;
	Size.y=BeamStopParamsTemplate.m_SizeY;
	Speed=BeamStopParamsTemplate.m_Speed;
	Range=BeamStopParamsTemplate.m_Range;
	PolynomDegree=BeamStopParamsTemplate.m_PolynomDegree;

	Position=aPosition;
	Direction=aDirection;
	RealSize=aRealSize;

	double x=Direction.x;
	double y=Direction.y;
	double hSpeed=sqrt(x*x+y*y)*6.0; //convert dinstance in nm per 10 min to nm/hour
	Speed=hSpeed;
	if (Speed==0) Speed=1;
	if (Speed>500000) Speed=500000; //not more than 500 micrometer per hour.	
	Size.x=1;
	Size.y=1;
	Calibration.x=1;
	Calibration.y=1;
	CellBinning=1;
	NeuronPositionOnSpline=0;
	MyNumber=0;
	AxonArea=0;		
	CellArea=0;	
	AxonStartLeft,AxonStartRight,AxonStartPos=CPoint::CPoint(0,0);
	ScanCurve=NULL;
	FrontierPoints=NULL;
	for (int i=0;i<4;i++) LaserShape[i]=CPoint::CPoint(0,0);
	NormalDirection=Direction;
	Offset=CPoint::CPoint(0,0);
}

CBeamStop::CBeamStop()
{	
	FrontierPoints=NULL;
	ScanCurve=NULL;
	PolynomDegree=5;
}	

CBeamStop::CBeamStop(CBeamStop* Org)
{
	for (int i=0;i<4;i++) LaserShape[i]=Org->LaserShape[i];
	PolynomDegree=Org->PolynomDegree;
	Range=Org->Range;
	AxonStartPos=Org->AxonStartPos;
	AxonStartLeft=Org->AxonStartLeft;
	AxonStartRight=Org->AxonStartRight;	
	AxonArea=Org->AxonArea;		
	CellArea=Org->CellArea;
	MyNumber=Org->MyNumber;
	Position=Org->Position;
	Direction=Org->Direction;
	NormalDirection=Org->NormalDirection;
	Offset=Org->Offset;
	DwellTime=Org->DwellTime;
	NeuronPositionOnSpline=Org->NeuronPositionOnSpline;
	Speed=Org->Speed;
	Size=Org->Size;
	RealSize=Org->RealSize;	
	if (Org->ScanCurve) ScanCurve=new CSmoothCurve(Org->ScanCurve);
	else ScanCurve=NULL;
	SplineControlPoints.SetSize(Org->SplineControlPoints.GetSize());
	for (i=0;i<Org->SplineControlPoints.GetSize();i++) SplineControlPoints[i]=Org->SplineControlPoints[i];
	Calibration=Org->Calibration;	
	CellBinning=Org->CellBinning;
	if (Org->FrontierPoints) {
		FrontierPoints=new CArray<POINT,POINT>;
		FrontierPoints->SetSize(Org->FrontierPoints->GetSize());
		for (int i=0;i<Org->FrontierPoints->GetSize();i++) (*FrontierPoints)[i]=(*Org->FrontierPoints)[i];
	} else FrontierPoints=NULL;	
}

CBeamStop::~CBeamStop()
{
	if (FrontierPoints) delete FrontierPoints;
	if (ScanCurve) delete ScanCurve;
}

void DrawCross(POINT point, COLORREF rgbColor, int extent, HDC hDC)
{
	HPEN hPen,oldPen;
	hPen = CreatePen(PS_SOLID,2,rgbColor);
    oldPen = (HPEN)SelectObject(hDC,hPen);
	
	MoveToEx(hDC,point.x - extent, point.y,NULL);
	LineTo(hDC, point.x + extent, point.y);
    
	MoveToEx(hDC, point.x, point.y - extent,NULL);
    LineTo(hDC,point.x, point.y + extent);
	SelectObject(hDC,oldPen);
	DeleteObject(hPen);
}

void CBeamStop::Draw(CDC* pDC, CDataPointDoc* pDoc,CDataPointView* pView, int mode) {
	if ((mode!=1) && (SplineControlPoints.GetSize()>1)) {
		//convert to pixel coordinates
		CArray<POINT,POINT> ViewSplineControlPoints;  
		ViewSplineControlPoints.SetSize(SplineControlPoints.GetSize());
		for (int i=0;i<SplineControlPoints.GetSize();i++) {			
			CPoint Help=SplineControlPoints[i];
			pDoc->ConvertCoordRealToPixel(Help);
			ConvertCoordPixelToSplinePixel(Help);		
			ViewSplineControlPoints[i]=Help;
		}
		//create a spline object
		Spline spline(ViewSplineControlPoints.GetData(), ViewSplineControlPoints.GetSize());
		//generate a curve
		spline.Generate();
		//get the curve points number
		CArray<POINT, POINT> m_CurvePoints;
		m_CurvePoints.SetSize(spline.GetCurveCount());
		//get the points number
		int PointCount = 0;
		spline.GetCurve(m_CurvePoints.GetData(), PointCount);
		//convert to view coordinates
		for (i=0;i<m_CurvePoints.GetSize();i++) {
			CPoint h=m_CurvePoints[i];
			ConvertCoordSplinePixelToPixel(h);
			pView->ConvertCoordsDocToView(h);
			m_CurvePoints[i]=h;
		}
		for (i=0;i<ViewSplineControlPoints.GetSize();i++) {			
			CPoint Help=ViewSplineControlPoints[i];
			ConvertCoordSplinePixelToPixel(Help);
			pView->ConvertCoordsDocToView(Help);			
			ViewSplineControlPoints[i]=Help;
		}
		//draw the control points
		DrawCross(ViewSplineControlPoints[0], RGB(0,255,0), 8, pDC->m_hDC);
		DrawCross(ViewSplineControlPoints[0], RGB(255,0,0), 6, pDC->m_hDC);
		for (i=1; i<ViewSplineControlPoints.GetSize(); i++)
			DrawCross(ViewSplineControlPoints[i], RGB(255,0,0), 8, pDC->m_hDC);
	//	for (i=0; i<m_CurvePoints.GetSize(); i++)
	//		DrawCross(m_CurvePoints[i], RGB(128,128,128), 4, pDC->m_hDC);
		//draw the curve
		CPen penStroke;
		penStroke.CreatePen(PS_SOLID, 2, RGB(64,128+64,0));		
		CPen* pOldPen = pDC->SelectObject(&penStroke);
		if(m_CurvePoints.GetSize()>1)
			pDC->Polyline( m_CurvePoints.GetData(), m_CurvePoints.GetSize());
		pDC->SelectObject(pOldPen);
	}
		
	if ((LaserShape[0]==CPoint::CPoint(0,0)) || (Range==0) || (ScanCurve==NULL)) {
		CPoint hSize=Size;
		hSize+=RealSize;
		pDoc->ConvertCoordRealToPixel(hSize);
		pView->ConvertCoordsDocToView(hSize);	
		if (hSize.x<10) hSize.x=10;
		if (hSize.y<10) hSize.y=10;
		hSize.x/=2;
		hSize.y/=2;
		CPoint StartPos(Position);
		pDoc->ConvertCoordRealToPixel(StartPos);
		pView->ConvertCoordsDocToView(StartPos);
		CPoint ul=StartPos-hSize;
		CPoint lr=StartPos+hSize;
		CRect rect(ul.x,ul.y,lr.x,lr.y);	
		if (mode==1) {
			CBrush br(RGB(128,0,0));
			pDC->FillRect( rect, &br );
		} else {
			CBrush br( RGB(255,0,0));
			pDC->FillRect( rect, &br );
		}
	} else {
		CPen penStroke;
		penStroke.CreatePen(PS_SOLID, 1, RGB((mode==1) ? 128 : 255,0,0));		
		CPen* pOldPen = pDC->SelectObject(&penStroke);
		CBrush br(RGB((mode==1) ? 128 : 255 ,0,0));					
		CBrush* OldBrush=pDC->SelectObject(&br);
		pDC->SetPolyFillMode(ALTERNATE);		
		CPoint ViewLaserShape[4];
		for (int i=0;i<4;i++) {
			ViewLaserShape[i]=LaserShape[i];
			pView->ConvertCoordsDocToView(ViewLaserShape[i]);
		}
		pDC->Polygon(ViewLaserShape,4);
		pDC->SelectObject(OldBrush);
		pDC->SelectObject(pOldPen);
	}	

	if ((mode!=1) /*&& (SplineControlPoints.GetSize()<=1)*/) {
		CPen penStroke;		
		penStroke.CreatePen(PS_SOLID, 2, RGB(255,128+64,0));		
		CPen* pOldPen = pDC->SelectObject(&penStroke);
		CPoint StartPos(Position);
		pDoc->ConvertCoordRealToPixel(StartPos);
		pView->ConvertCoordsDocToView(StartPos);		
		pDC->MoveTo(StartPos);
		CPoint EndPos=Position+Direction;
		pDoc->ConvertCoordRealToPixel(EndPos);
		pView->ConvertCoordsDocToView(EndPos);
		pDC->LineTo(EndPos);
		pDC->SelectObject(pOldPen);

		if (ScanCurve) ScanCurve->Draw(pDC,this,pView);
		if ((FrontierPoints) && (FrontierPoints->GetSize()>0)) {			
			for (int k=0;k<FrontierPoints->GetSize();k++) {
				CPoint help=(*FrontierPoints)[k];
				pView->ConvertCoordsDocToView(help);
				pDC->SetPixel(help,RGB(0,0,255));
			}						
		}		
		if ((AxonStartLeft.x!=0) || (AxonStartLeft.y!=0)) {
			CPen penStroke;		
			penStroke.CreatePen(PS_SOLID, 2, RGB(0,0,255));		
			CPen* pOldPen = pDC->SelectObject(&penStroke);					
			CPoint help=AxonStartLeft;
			ConvertCoordRealToPixel(help);
			pView->ConvertCoordsDocToView(help);
			pDC->MoveTo(help);
			help=AxonStartRight;
			ConvertCoordRealToPixel(help);
			pView->ConvertCoordsDocToView(help);
			pDC->LineTo(help);
			pDC->SelectObject(pOldPen);
		}
	}	
}

/////////////////////////////////////////////////////////////////////////////
// CBeamStop message handlers

void CBeamStop::CalcBoundingRect()  //in real coords
{
	// Calculate the bounding rectangle.  It's needed for smart
	// repainting.
	
	m_rectBounding = CRect(Position.x, Position.y, Position.x, Position.y);
	CPoint pt=Position+Direction;
	m_rectBounding.left     = min(m_rectBounding.left, pt.x);
	m_rectBounding.right    = max(m_rectBounding.right, pt.x);
	m_rectBounding.top      = min(m_rectBounding.top, pt.y);
	m_rectBounding.bottom   = max(m_rectBounding.bottom, pt.y);
	for (int i=0;i<SplineControlPoints.GetSize();i++) {
		CPoint pt=SplineControlPoints[i];
		m_rectBounding.left     = min(m_rectBounding.left, pt.x);
		m_rectBounding.right    = max(m_rectBounding.right, pt.x);
		m_rectBounding.top      = min(m_rectBounding.top, pt.y);
		m_rectBounding.bottom   = max(m_rectBounding.bottom, pt.y);
	}

	// Add the pen width to the bounding rectangle.  This is necessary
	// to account for the width of the stroke when invalidating
	// the screen.
	m_rectBounding.InflateRect(CSize(1000, 1000));
}

unsigned long CBeamStop::GetSqrDistance(CPoint point, CPoint &pos,int &subPoint)  //points in real coords
{
	CPoint h1=Position-point;
	CPoint h2=Position+Direction-point;
	unsigned long d1=h1.x*h1.x+h1.y*h1.y;
	unsigned long d2=h2.x*h2.x+h2.y*h2.y;
	unsigned long d;
	if (d1<d2) {
		pos=Position;
		subPoint=1;
		d=d1;
	} else {
		pos=Position+Direction;
		subPoint=2;
		d=d2;
	}

	for (int i=0;i<SplineControlPoints.GetSize();i++) {
		CPoint pt=SplineControlPoints[i];
		pt-=point;
		unsigned long d3=pt.x*pt.x+pt.y*pt.y;
		if (d3<d) {
			d=d3;
			subPoint=3+i;
			pos=SplineControlPoints[i];
		}		
	}
	return d;
}

void CBeamStop::Move(CPoint move,CRect Limit,int subPoint,CDataPointDoc* pDoc) {
	if (subPoint==1) {
		CPoint hpos=Position+move;
		if ((hpos.x<Limit.TopLeft().x) || (hpos.x>Limit.BottomRight().x) ||
		    (hpos.y<Limit.TopLeft().y) || (hpos.y>Limit.BottomRight().y)) return;
		Position=hpos;
	} else if (subPoint==2) {	
		Direction+=move;
		double x=Direction.x;
		double y=Direction.y;
		double aktSpeed=sqrt(x*x+y*y)*6.0; //convert distance in nm per 10 min to nm/hour			
		if (aktSpeed<1) aktSpeed=1;
		Speed=aktSpeed;		
	} else if (subPoint-3<SplineControlPoints.GetSize()) {
		if (subPoint-3==0) {
			AxonStartLeft=CPoint::CPoint(0,0);
			AxonStartRight=CPoint::CPoint(0,0);
		}
		CPoint help=SplineControlPoints[subPoint-3];
		help+=move;
		CArray<POINT, POINT> CurvePoints;
		GetSpline(CurvePoints);
		double lpos=GetLaserPositionOnSpline(CurvePoints);
		SplineControlPoints[subPoint-3]=help;
		GetSpline(CurvePoints);
		SetLaserPositionOnSpline(CurvePoints,lpos);
	}
	if (((subPoint==1) || (subPoint==2)) && (SplineControlPoints.GetSize()>1)) {
		CArray<POINT, POINT> CurvePoints;
		GetSpline(CurvePoints);
		SetLaserPositionOnSpline(CurvePoints,GetLaserPositionOnSpline(CurvePoints));		
	}
	if (ScanCurve) {
		delete ScanCurve;
		ScanCurve=NULL;
	}
	if (FrontierPoints) {
		delete FrontierPoints;
		FrontierPoints=NULL;
	}
	NormalDirection=Direction;
	CalcBoundingRect();
}

int CBeamStop::PropertyDlg(bool Update)
{
	CBeamStopDlg dlg;
// Initialize dialog data	
	dlg.m_DirectionX=Direction.x;
	dlg.m_DirectionY=Direction.y;
	dlg.m_DwellTime=DwellTime;
	dlg.m_PositionX=Position.x;
	dlg.m_PositionY=Position.y;
	dlg.m_SizeX=Size.x;
	dlg.m_SizeY=Size.y;
	dlg.m_Speed=Speed;
	dlg.m_Range=Range;
	dlg.m_PolynomDegree=PolynomDegree;
	dlg.m_OffsetX=Offset.x;
	dlg.m_OffsetY=Offset.y;
	// Invoke the dialog box
	if (dlg.DoModal() == IDOK) { 
		if (Update) {
			// retrieve the dialog data		
			DwellTime=dlg.m_DwellTime;
			Position.x=dlg.m_PositionX;
			Position.y=dlg.m_PositionY;
			Size.x=dlg.m_SizeX;
			Size.y=dlg.m_SizeY;
			double x=dlg.m_DirectionX;
			double y=dlg.m_DirectionY;
			double aktSpeed=sqrt(x*x+y*y)*6.0; //convert distance in nm per 10 min to nm/hour			
			if (aktSpeed<1) aktSpeed=1;
			double hSpeed=dlg.m_Speed;
			if (Speed!=dlg.m_Speed) {			
				x=x*hSpeed/aktSpeed;
				y=y*hSpeed/aktSpeed;
				Direction.x=x;
				Direction.y=y;
			} 
			Speed=aktSpeed;	
			Range=dlg.m_Range;
			PolynomDegree=dlg.m_PolynomDegree;
			Offset.x=dlg.m_OffsetX;
			Offset.y=dlg.m_OffsetY;
			return dlg.m_ModifyOthers;		
		}
	}
	return -1;
}

void CBeamStop::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{		
		ar<<Direction;
		ar<<NormalDirection;
		ar<<Offset;
		ar<<DwellTime;
		ar<<Position;
		ar<<Size;
		ar<<Speed;
		ar<<RealSize;		
		ar<<MyNumber;
		ar<<NeuronPositionOnSpline;
		ar<<AxonArea;
		ar<<CellArea;
		ar<<Range;
		ar<<AxonStartPos;
		ar<<AxonStartLeft;
		ar<<AxonStartRight;
		ar<<SplineControlPoints.GetSize();				
		for (int i=0;i<SplineControlPoints.GetSize();i++) {
			CPoint help=SplineControlPoints[i];
			ar<<help.x;
			ar<<help.y;
		}
		if (FrontierPoints) {
			ar<<'Y';
			ar<<FrontierPoints->GetSize();		
			for (int i=0;i<FrontierPoints->GetSize();i++) {
				CPoint help=(*FrontierPoints)[i];
				ar<<help.x;
				ar<<help.y;
			}
		} else ar<<'N';
		if (ScanCurve) {
			ar<<'Y';
			ScanCurve->Serialize(ar);
		} else ar<<'N';
		ar<<Calibration;		
		ar<<CellBinning;
		ar<<PolynomDegree;
		for (i=0;i<4;i++) ar<<LaserShape[i];
	}
	else
	{
		ar>>Direction;
		ar>>NormalDirection;
		ar>>Offset;
		ar>>DwellTime;
		ar>>Position;
		ar>>Size;
		ar>>Speed;
		ar>>RealSize;
		ar>>MyNumber;
		ar>>NeuronPositionOnSpline;
		ar>>AxonArea;
		ar>>CellArea;
		ar>>Range;
		ar>>AxonStartPos;
		ar>>AxonStartLeft;
		ar>>AxonStartRight;
		int size;
		ar>>size;			
		SplineControlPoints.SetSize(size);		
		for (int i=0;i<size;i++) {
			CPoint help;
			ar>>help.x;
			ar>>help.y;
			SplineControlPoints[i]=help;			
		}
		char hc;
		ar>>hc;
		if (hc=='Y') {
			int size;
			ar>>size;	
			if (FrontierPoints) delete FrontierPoints;
			FrontierPoints=new CArray<POINT, POINT>;
			FrontierPoints->SetSize(size);		
			for (int i=0;i<size;i++) {
				CPoint help;
				ar>>help.x;
				ar>>help.y;
				(*FrontierPoints)[i]=help;			
			}		
		}
		ar>>hc;
		if (hc=='Y') {
			if (ScanCurve==NULL) ScanCurve=new CSmoothCurve();
			ScanCurve->Serialize(ar);
		}
		ar>>Calibration;		
		ar>>CellBinning;
		ar>>PolynomDegree;
		for (i=0;i<4;i++) ar>>LaserShape[i];
	}	
}

void CBeamStop::CalculateLaserPosition(CBeamGuiding *BeamGuiding)
{
	if (Range>0) {
		CPoint Laser=BeamGuiding->GetLaserLongAxisVector(Position);		
		double scalar=NormalDirection.x*Laser.x+NormalDirection.y*Laser.y;
		bool LaserTop=scalar<0;			
		CPoint LaserNorm=Laser;
		Normalize(LaserNorm);
		double lx=LaserNorm.x;
		double ly=LaserNorm.y;
		lx*=BeamGuiding->LaserCellOverlap/1000;
		ly*=BeamGuiding->LaserCellOverlap/1000;
		CPoint OverlapShift=CPoint::CPoint(lx,ly);				
		CPoint LaserPosition=GetScanCurvePosition(0);		
		CPoint EndPosition;
		if (BeamGuiding->LaserCellOverlap>0) {
			if (LaserTop) {
				LaserPosition+=OverlapShift;
				EndPosition=LaserPosition-Laser;
			} else {
				LaserPosition-=OverlapShift;
				EndPosition=LaserPosition+Laser;
			}
		} 
		if (LaserTop) EndPosition=LaserPosition-Laser;
		else EndPosition=LaserPosition+Laser;					
		ConvertCoordRealToPixel(LaserPosition);
		ConvertCoordRealToPixel(EndPosition);
		CPoint hSize=Size;
		ConvertCoordRealToPixel(hSize);
		LaserShape[0]=CPoint::CPoint(LaserPosition.x-hSize.x/2,LaserPosition.y);
		LaserShape[1]=CPoint::CPoint(LaserPosition.x+hSize.x/2,LaserPosition.y);
		LaserShape[2]=CPoint::CPoint(EndPosition.x+hSize.x/2,EndPosition.y);
		LaserShape[3]=CPoint::CPoint(EndPosition.x-hSize.x/2,EndPosition.y);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CBeamStopDlg dialog


CBeamStopDlg::CBeamStopDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBeamStopDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBeamStopDlg)
	m_DirectionX = 1;
	m_DirectionY = 0;
	m_DwellTime = 200;
	m_PositionX = 0;
	m_PositionY = 0;
	m_SizeX = 1;
	m_SizeY = 1;
	m_Speed = 1;
	m_Range = 0;
	m_PolynomDegree = 5;
	m_ModifyOthers=0;
	m_OffsetX = 0;
	m_OffsetY = 0;
	//}}AFX_DATA_INIT
}


void CBeamStopDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBeamStopDlg)
	DDX_Text(pDX, IDC_DIRECTIONX, m_DirectionX);
	DDV_MinMaxLong(pDX, m_DirectionX, -1000000, 1000000);
	DDX_Text(pDX, IDC_DIRECTIONY, m_DirectionY);
	DDV_MinMaxLong(pDX, m_DirectionY, -1000000, 1000000);
	DDX_Text(pDX, IDC_DWELLTIME, m_DwellTime);
	DDV_MinMaxLong(pDX, m_DwellTime, 0, 10000);
	DDX_Text(pDX, IDC_POSITIONX, m_PositionX);
	DDV_MinMaxLong(pDX, m_PositionX, -1000000, 1000000);
	DDX_Text(pDX, IDC_POSITIONY, m_PositionY);
	DDV_MinMaxLong(pDX, m_PositionY, -1000000, 1000000);
	DDX_Text(pDX, IDC_SIZEX, m_SizeX);
	DDV_MinMaxLong(pDX, m_SizeX, 0, 100000);
	DDX_Text(pDX, IDC_SIZEY, m_SizeY);
	DDV_MinMaxLong(pDX, m_SizeY, 0, 100000);
	DDX_Text(pDX, IDC_SPEED, m_Speed);
	DDV_MinMaxLong(pDX, m_Speed, 0, 1000000);
	DDX_Text(pDX, IDC_RANGE, m_Range);
	DDV_MinMaxLong(pDX, m_Range, 0, 100000);
	DDX_Text(pDX, IDC_POLYNOMDEGREE, m_PolynomDegree);
	DDV_MinMaxLong(pDX, m_PolynomDegree, 1, 100);
	DDX_Text(pDX, IDC_OFFSETX, m_OffsetX);
	DDV_MinMaxLong(pDX, m_OffsetX, -100000, 100000);
	DDX_Text(pDX, IDC_OFFSETY, m_OffsetY);
	DDV_MinMaxLong(pDX, m_OffsetY, -100000, 100000);
	//}}AFX_DATA_MAP
	if (pDX->m_bSaveAndValidate!=0) {	
		m_ModifyOthers=0;
		if (IsDlgButtonChecked( IDC_SIZE_UPDATE_ALL)) m_ModifyOthers+=1;
		if (IsDlgButtonChecked( IDC_DWELLTIME_UPDATE_ALL)) m_ModifyOthers+=2;
	}	
}


BEGIN_MESSAGE_MAP(CBeamStopDlg, CDialog)
	//{{AFX_MSG_MAP(CBeamStopDlg)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBeamStopDlg message handlers

void CBeamStop::AddSpline(CPoint point,CPoint aCalibration,long aCellBinning) //point in real coords
{	
	Calibration=aCalibration;
	CellBinning=aCellBinning;
	if (SplineControlPoints.GetSize()>1) SplineControlPoints.Add(point);
	else {
		SplineControlPoints.Add(Position);
		SplineControlPoints.Add(Position+Direction);
	}
}

double CBeamStop::GetLaserPositionOnSpline(CArray<POINT, POINT>& CurvePoints,bool CalcIntermediatePosition)  //returns pos in 0..1
{
	if (CurvePoints.GetSize()==0) return 0;
	CPoint hPos=Position;
	ConvertCoordRealToPixel(hPos);
	unsigned long d=0;
	int f=0;
	for (int i=0;i<CurvePoints.GetSize();i++) {
		CPoint hp=CurvePoints[i];
		hp-=hPos;
		unsigned long d2=hp.x*hp.x+hp.y*hp.y;
		if ((i==0) || (d2<d)) {
			f=i;
			d=d2;
		}
	}		
	double l=GetSplineLength(CurvePoints);
	double pl=GetSplineLength(CurvePoints,f);
	double t=pl/l;
	if (CalcIntermediatePosition) {
		//have to calc intermediate position
		if (f>0) {
			
		}
	}
	return t;		
}

void CBeamStop::GetSpline(CArray<POINT, POINT>& m_CurvePoints)
{
	if (SplineControlPoints.GetSize()<2) return;
	//convert to Pixel coordinates
	CArray<POINT,POINT> DocSplineControlPoints;  
	DocSplineControlPoints.SetSize(SplineControlPoints.GetSize());
	for (int i=0;i<SplineControlPoints.GetSize();i++) {			
		CPoint Help=SplineControlPoints[i];
		ConvertCoordRealToPixel(Help);		
		ConvertCoordPixelToSplinePixel(Help);		
		DocSplineControlPoints[i]=Help;
	}
	//create a spline object
	Spline spline(DocSplineControlPoints.GetData(), DocSplineControlPoints.GetSize());
	//generate a curve
	spline.Generate();
	//get the curve points number
	m_CurvePoints.SetSize(spline.GetCurveCount());
	//get the points number
	int PointCount = 0;
	spline.GetCurve(m_CurvePoints.GetData(), PointCount);	
}

void CBeamStop::SetLaserPositionOnSpline(CArray<POINT, POINT>& CurvePoints,double pos) //pos in 0..1
{	
	if (CurvePoints.GetSize()==0) return;
	double Length=GetSplineLength(CurvePoints);  //total length in real coords
	pos*=Length;
	ConvertCoordRealToPixel(pos);
	int i;
	Position=GetIntermediateSplinePos(CurvePoints,pos,i);	//in pixel coords
	ConvertCoordPixelToReal(Position);

	if (i==0) i=1;
	int i1=i-1;
	int i2=i;
	CPoint hDir;
	double h2;
	double xn,yn;
	do {
		CPoint p1=CurvePoints[i1];
		CPoint p2=CurvePoints[i2];	
		hDir=p2-p1;
		if (i1>0) i1--;
		if (i2<CurvePoints.GetSize()-1) i2++;
		xn=hDir.x;
		yn=hDir.y;
		h2=xn*xn+yn*yn;
	} while (((i1>0) || (i2<CurvePoints.GetSize()-1)) && (h2<30));
	if (h2>=30) {
		double newSpeed=sqrt(h2)*6.0;	
		xn=xn*Speed/newSpeed;
		yn=yn*Speed/newSpeed;
		Direction.x=xn;
		Direction.y=yn;
	}
}

void CBeamStop::ConvertCoordRealToPixel(CPoint &point)
{
	point.x=point.x/(Calibration.x*CellBinning);
	point.y=point.y/(Calibration.y*CellBinning);
}

void CBeamStop::ConvertCoordPixelToReal(CPoint &point)
{
	point.x=point.x*Calibration.x*CellBinning;
	point.y=point.y*Calibration.y*CellBinning;
}

void CBeamStop::ConvertCoordRealToPixel(double& point)
{
	point=point/(Calibration.x*CellBinning);	
}

void CBeamStop::ConvertCoordPixelToReal(double &point)
{
	point=point*(Calibration.x*CellBinning);	
}

void CBeamStop::ConvertCoordSplinePixelToPixel(CPoint &point)
{
	point.x=point.x/SplineGraining;
	point.y=point.y/SplineGraining;
}

void CBeamStop::ConvertCoordPixelToSplinePixel(CPoint &point)
{
	point.x=point.x*SplineGraining;
	point.y=point.y*SplineGraining;
}

void CBeamStop::ConvertCoordSplinePixelToPixel(double& point)
{
	point=point/SplineGraining;	
}

void CBeamStop::ConvertCoordPixelToSplinePixel(double &point)
{
	point=point*SplineGraining;	
}

double CBeamStop::GetSplineLength(CArray<POINT, POINT>& CurvePoints,int EndPoint,bool realcoords) //in real coords
{
	double l=0;
	if (EndPoint==-1) EndPoint=CurvePoints.GetSize()-1;
	if (EndPoint>CurvePoints.GetSize()-1) EndPoint=CurvePoints.GetSize()-1;
	for (int i=0;i<EndPoint;i++) {		
		CPoint p1=CurvePoints[i];
		CPoint p2=CurvePoints[i+1];
		double x=p2.x-p1.x;
		double y=p2.y-p1.y;
		l+=sqrt(x*x+y*y);
	}
	if (realcoords) return l*Calibration.x*CellBinning;  //real coords
	else return l;  //pixel coords
}

CPoint CBeamStop::GetIntermediateSplinePos(CArray<POINT, POINT>& CurvePoints,double EndLength,int& i) //EndLength in pixel coord
{
	CPoint pos;
	double addlength;
	i=GetSplinePos(CurvePoints,EndLength,addlength,false,false);	//delivers next point after having reached length t
	if ((addlength>0) && (i>0)) {
		CPoint startpos=CurvePoints[i-1];
		CPoint endpos=CurvePoints[i];
		double xd=endpos.x-startpos.x;
		double yd=endpos.y-startpos.y;
		double l=xd*xd+yd*yd;
		if (l>0.0001) l=sqrt(l); 
		else return startpos;  //this should never happen because of the way GetSplinePos works
		pos.x=startpos.x+(addlength/l)*xd;
		pos.y=startpos.y+(addlength/l)*yd;
	} else {
		pos=CurvePoints[i];
	}
	return pos;
}

int CBeamStop::GetSplinePos(CArray<POINT, POINT>& CurvePoints,double EndLength,double &addlength,bool realcoords, bool randomize) //EndLength in real or pixel coord
{
	double l=0;
	int i=0;
	while ((l<EndLength) && (i<CurvePoints.GetSize()-1)) {		
		i++;
		CPoint p1=CurvePoints[i-1];
		CPoint p2=CurvePoints[i];
		double x=p2.x-p1.x;
		double y=p2.y-p1.y;
		double l2=sqrt(x*x+y*y);
		if (realcoords) l2*=Calibration.x*CellBinning;  //if yes real if no pixel coords
		l+=l2;		
	}
	addlength=l-EndLength;
	if (randomize) {
		double a=RAND_MAX;
		double b=rand()/a;
		if ((b>0.5) && (i>0)) i--;
	}
	if (i>=CurvePoints.GetSize()) {
		i--;
		addlength=0;
	}
	return i;
}

CPoint CBeamStop::GetDirection(CArray<POINT, POINT>& CurvePoints,double EndLength, bool realcoords) //EndLength in real or pixel coord
//returns direction of spline at position EndLength, non normalized
{
	double addlength;
	int i=GetSplinePos(CurvePoints,EndLength,addlength,realcoords,false); //EndLength in real or pixel coord	
	if (i<0) return CPoint(0,0);
	double l=0;
	CPoint Start=CurvePoints[i];
	CPoint End=CurvePoints[i];		
	while ((i<CurvePoints.GetSize()-1) && (l==0)) {		
		i++;
		End=CurvePoints[i];		
		double x=End.x-Start.x;
		double y=End.y-Start.y;
		l=x*x+y*y;		
	}
	if (l>0) {
		return End-Start;
	} else {
		double l=0;
		CPoint Start=CurvePoints[i];
		CPoint End=CurvePoints[i];		
		while ((i>0) && (l==0)) {		
			i--;
			End=CurvePoints[i];		
			double x=End.x-Start.x;
			double y=End.y-Start.y;
			l=x*x+y*y;		
		}
		if (l>0) {
			return Start-End;
		} else return CPoint(0,0);
	} 
}

void CBeamStop::DeleteSplinePoint(int subPoint)
{
	if ((subPoint>2) && (subPoint-3<SplineControlPoints.GetSize())) {
		CArray<POINT, POINT> CurvePoints;
		GetSpline(CurvePoints);
		double lpos=GetLaserPositionOnSpline(CurvePoints);
		for (int i=subPoint-3;i<SplineControlPoints.GetSize()-1;i++) {
			SplineControlPoints[i]=SplineControlPoints[i+1];
		}
		SplineControlPoints.RemoveAt(SplineControlPoints.GetSize()-1);
		if (SplineControlPoints.GetSize()<2) {
			SplineControlPoints.RemoveAt(0);
			AxonStartLeft=CPoint::CPoint(0,0);
			AxonStartRight=CPoint::CPoint(0,0);			
		} else {
			GetSpline(CurvePoints);
			SetLaserPositionOnSpline(CurvePoints,lpos);
		}
	}
}

unsigned short CBeamPath::GetNumberStops()
{
	return MaxNumber;
	/*POSITION pos=GetHeadPosition();	
	unsigned short MaxNumber=0;
	while (pos!=NULL) {
		CBeamStop* help=((CBeamStop*)GetNext(pos));		
		if (help->MyNumber>MaxNumber) MaxNumber=help->MyNumber;
	}
	return MaxNumber;*/
}

void CBeamPath::AddBeamStop(CBeamStop *BeamStop)
{
	MaxNumber++;
	BeamStop->SetNumber(MaxNumber);
	AddHead(BeamStop);
}

CBeamStop* CBeamPath::GetBeamStop(unsigned short Nr)
{
	CBeamStop* result=NULL;	
	POSITION pos=GetHeadPosition();	
	while (pos!=NULL) {
		CBeamStop* help=((CBeamStop*)GetNext(pos));		
		if (help->MyNumber==Nr) result=help;
	}
	return result;
}

void CBeamGuiding::SetBeamIntense(bool aIntensity)
{
	i16 iStatus = 0;
    i16 iRetVal = 0;  
    i16 iIgnoreWarning = 0;  

	BeamIntensity=aIntensity;
	if (HardwareAccess) {
		iStatus = DIG_Out_Line(iDevice, iPort, iBeamIntensityLine, (BeamIntensity) ? 1 : 0);
		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SetBeamIntense DIG_Out_Line", iIgnoreWarning);
	}
}

void CBeamStopDlg::Serialize(CArchive &ar)
{
	if (ar.IsStoring())
	{	
		ar<<m_DirectionX;
		ar<<m_DirectionY;
		ar<<m_DwellTime;
		ar<<m_PositionX;
		ar<<m_PositionY;
		ar<<m_SizeX;
		ar<<m_SizeY;
		ar<<m_Speed;
		ar<<m_Range;
		ar<<m_PolynomDegree;
	} else {
		ar>>m_DirectionX;
		ar>>m_DirectionY;
		ar>>m_DwellTime;
		ar>>m_PositionX;
		ar>>m_PositionY;
		ar>>m_SizeX;
		ar>>m_SizeY;
		ar>>m_Speed;
		ar>>m_Range;
		ar>>m_PolynomDegree;
	}
}

CPoint CBeamGuiding::GetLaserLongAxisVector(CPoint pos) 
// returns a vector from the lower to the upper end of the laser at position pos in nm
//pos is in real coords and gives lower end of laser
{		
	double x=pos.x;
	double y=pos.y;
	x/=1000.0;
	y/=1000.0;
	double XSize = -3.17552 + 0.00681 * x -0.01818 * y + 0.00009 * x * y -0.00003 * x * x + 0.00017 * y * y + 7.9132E-8 * x * x * x -3.3317E-8 * x * x * y -2.6114E-7 * x * y * y -2.6435E-7 * y * y * y;
	double YSize = -15.3738 -0.01095* x + 0.006 * y + 0.00014 * x * y + 0.00002 * x * x -0.00019* y * y -1.7654E-8* x * x * x -4.6104E-8* x * x * y -3.5449E-7* x * y * y + 6.1936E-7 * y * y * y;


//	double XSize=-2.04102-0.00668*x-0.02131*y+0.00002*x*y+0.00003*x*x+0.00037*y*y-7.3678E-8*x*x*x-2.7404E-8*x*x*y+1.7186E-8*x*y*y-9.5348E-7*y*y*y;
//	double YSize=-20.30744-0.02809*x+0.03801*y-0.00004*x*y+0.00018*x*x-0.0005*y*y-3.0312E-7*x*x*x+2.9053E-7*x*x*y-1.0486E-7*x*y*y+1.4329E-6*y*y*y;
	return CPoint::CPoint(XSize*1000,YSize*1000);
}

CPoint CBeamStop::GetScanCurvePosition(double t) //t from -Range/2 to +Range/2 in nm //return in real coords
{
	if (ScanCurve) {
		ConvertCoordRealToPixel(t);
		CPoint p=ScanCurve->GetPoint(t);
		ConvertCoordPixelToReal(p);
		return p;
	} else return CPoint::CPoint(0,0);
}

void CBeamGuiding::ConfigDlg()
{
	CBeamGuidingDlg dlg;	
	dlg.m_HopTime=ParabolicHopTime;
	dlg.m_LaserCellOverlap=LaserCellOverlap;
	dlg.m_OffVoltageDAC0=OffVoltage0;
	dlg.m_OffVoltageDAC2=OffVoltage2;
	dlg.m_SamplingFrequency=Frequency;
	dlg.m_SinusSweep=SinusSweep;
	dlg.m_WobbleXFrequency=WobbleXFrequency;
	dlg.m_WobbleYFrequency=WobbleYFrequency;
	dlg.m_ScanSpeed=ScanSpeed;
	dlg.m_AntiHysteresis=AntiHysteresis;
	dlg.m_DAC0Frequency=DAC0Frequency;
	dlg.m_DAC2Frequency=DAC2Frequency;
	dlg.m_DAC1Frequency=DAC1Frequency;
	dlg.m_DAC3Frequency=DAC3Frequency;
	dlg.m_DAC0Amplitude=DAC0Amplitude;
	dlg.m_DAC1Amplitude=DAC1Amplitude;
	dlg.m_DAC2Amplitude=DAC2Amplitude;
	dlg.m_DAC3Amplitude=DAC3Amplitude;
	dlg.m_AntiHysteresisAfterHop=AntiHysteresisAfterHop;
	dlg.m_DAC0FrequencyAfterHop=DAC0FrequencyAfterHop;
	dlg.m_DAC2FrequencyAfterHop=DAC2FrequencyAfterHop;
	dlg.m_DAC1FrequencyAfterHop=DAC1FrequencyAfterHop;
	dlg.m_DAC3FrequencyAfterHop=DAC3FrequencyAfterHop;
	dlg.m_DAC0AmplitudeAfterHop=DAC0AmplitudeAfterHop;
	dlg.m_DAC1AmplitudeAfterHop=DAC1AmplitudeAfterHop;
	dlg.m_DAC2AmplitudeAfterHop=DAC2AmplitudeAfterHop;
	dlg.m_DAC3AmplitudeAfterHop=DAC3AmplitudeAfterHop;
	dlg.m_AntiHysteresisDecayTime=AntiHysteresisDecayTime;
	dlg.m_AntiHysteresisHopToCenter=AntiHysteresisHopToCenter;
	dlg.m_AntiHysteresisHopAlsoForOneStop=AntiHysteresisHopAlsoForOneStop;
	dlg.m_AntiHysteresisHopRestingTime=AntiHysteresisHopRestingTime;
	dlg.m_FixPointVoltageDAC0=FixPointVoltageDAC0;
	dlg.m_FixPointVoltageDAC1=FixPointVoltageDAC1;
	dlg.m_FixPointVoltageDAC2=FixPointVoltageDAC2;
	dlg.m_FixPointVoltageDAC3=FixPointVoltageDAC3;
	if (dlg.DoModal()==IDOK) {
		ParabolicHopTime=dlg.m_HopTime;
		LaserCellOverlap=dlg.m_LaserCellOverlap;
		OffVoltage0=dlg.m_OffVoltageDAC0;
		OffVoltage2=dlg.m_OffVoltageDAC2;
		Frequency=dlg.m_SamplingFrequency;
		SinusSweep=dlg.m_SinusSweep;
		WobbleXFrequency=dlg.m_WobbleXFrequency;
		WobbleYFrequency=dlg.m_WobbleYFrequency;
		ScanSpeed=dlg.m_ScanSpeed;
		AntiHysteresis=dlg.m_AntiHysteresis;
		DAC0Frequency=dlg.m_DAC0Frequency;
		DAC2Frequency=dlg.m_DAC2Frequency;
		DAC1Frequency=dlg.m_DAC1Frequency;
		DAC3Frequency=dlg.m_DAC3Frequency;
		DAC0Amplitude=dlg.m_DAC0Amplitude;
		DAC1Amplitude=dlg.m_DAC1Amplitude;
		DAC2Amplitude=dlg.m_DAC2Amplitude;
		DAC3Amplitude=dlg.m_DAC3Amplitude;
		AntiHysteresisAfterHop=dlg.m_AntiHysteresisAfterHop;
		DAC0FrequencyAfterHop=dlg.m_DAC0FrequencyAfterHop;
		DAC2FrequencyAfterHop=dlg.m_DAC2FrequencyAfterHop;
		DAC1FrequencyAfterHop=dlg.m_DAC1FrequencyAfterHop;
		DAC3FrequencyAfterHop=dlg.m_DAC3FrequencyAfterHop;
		DAC0AmplitudeAfterHop=dlg.m_DAC0AmplitudeAfterHop;
		DAC1AmplitudeAfterHop=dlg.m_DAC1AmplitudeAfterHop;
		DAC2AmplitudeAfterHop=dlg.m_DAC2AmplitudeAfterHop;
		DAC3AmplitudeAfterHop=dlg.m_DAC3AmplitudeAfterHop;
		AntiHysteresisDecayTime=dlg.m_AntiHysteresisDecayTime;
		AntiHysteresisHopToCenter=dlg.m_AntiHysteresisHopToCenter;
		AntiHysteresisHopAlsoForOneStop=dlg.m_AntiHysteresisHopAlsoForOneStop;
		AntiHysteresisHopRestingTime=dlg.m_AntiHysteresisHopRestingTime;
		FixPointVoltageDAC0=dlg.m_FixPointVoltageDAC0;
		FixPointVoltageDAC1=dlg.m_FixPointVoltageDAC1;
		FixPointVoltageDAC2=dlg.m_FixPointVoltageDAC2;
		FixPointVoltageDAC3=dlg.m_FixPointVoltageDAC3;
		if (BeamPath) SetNewPath(BeamPath);
	}	
}

void CBeamPath::CalculateLaserPosition(CBeamGuiding *BeamGuiding)
{
	POSITION pos=GetHeadPosition();	
	while (pos!=NULL) {
		CBeamStop* help=((CBeamStop*)GetNext(pos));		
		if (help) help->CalculateLaserPosition(BeamGuiding);
	}	
}


//DEL void CBeamGuiding::ScopeLightShutter(bool on)
//DEL {
//DEL 	i16 iStatus = 0;
//DEL 	i16 iRetVal = 0;
//DEL 	i16 iIgnoreWarning = 0;
//DEL 	if (on)
//DEL 	{
//DEL 		iStatus = AO_VWrite(iDevice, 4, 0);
//DEL 		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SetVoltages AO_VWrite4", iIgnoreWarning);	
//DEL 	}
//DEL 	else
//DEL 	{
//DEL 		iStatus = AO_VWrite(iDevice, 4, 5);
//DEL 		iRetVal = NIDAQErrorHandler(iStatus, "CBeamGuiding::SetVoltages AO_VWrite4", iIgnoreWarning);	
//DEL 	}
//DEL }
