#if !defined(AFX_BEAM_H__35CBA332_F7F9_4B32_AB36_CCDD37FCDFB0__INCLUDED_)
#define AFX_BEAM_H__35CBA332_F7F9_4B32_AB36_CCDD37FCDFB0__INCLUDED_

#include "Neuron.h"	// Added by ClassView
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxtempl.h>
#include "DataPointObject.h"
#include "DataPointObjectList.h"
#include "stdafx.h"
#include "nidaqex.h"
#include "SmoothCurve.h"
#include "BeamGuidingDlg.h"	// Added by ClassView

// Beam.h : header file
//

class CBeamStopDlg;
class CBeamGuiding;
/////////////////////////////////////////////////////////////////////////////
// CBeamStop window

class CBeamStop : public CDataPointObject
{
// Construction
public:
	DECLARE_SERIAL( CBeamStop )
	CBeamStop();
	CBeamStop(CPoint aPosition,CPoint aDirection,CPoint aRealSize,CBeamStopDlg &BeamStopParamsTemplate);
	CBeamStop(CBeamStop* Org);

// Attributes
public:
	CArray<POINT, POINT> *FrontierPoints;
// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBeamStop)
	virtual void CalcBoundingRect();
	virtual void Draw(CDC* pDC, CDataPointDoc* pDoc,CDataPointView* pView, int mode);
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:		
	CPoint Offset;
	CPoint LaserShape[4];
	CPoint GetScanCurvePosition(double t);
	unsigned short PolynomDegree;
	CPoint AxonStartLeft,AxonStartRight;
	CSmoothCurve* ScanCurve;
	CPoint AxonStartPos;
	long Range;
	CPoint GetDirection(CArray<POINT, POINT>& CurvePoints,double EndLength, bool realcoords); //EndLength in real or pixel coord
	double AxonArea; //in nm^2
	double CellArea; //in nm^2
	unsigned short MyNumber;
	long NeuronPositionOnSpline;
	void DeleteSplinePoint(int subPoint);
	long CellBinning;
	CPoint Calibration;
	void AddSpline(CPoint point,CPoint aCalibration,long aCellBinnning);
	CPoint RealSize;
	void Move(CPoint move,CRect Limit,int subPoint,CDataPointDoc* pDoc);
	int PropertyDlg(bool Update);
	virtual unsigned long GetSqrDistance(CPoint point, CPoint &pos,int &subPoint);
	CPoint Size;
	CArray<POINT,POINT> SplineControlPoints;  //in real coords in nm
	
	long Speed; //in nm per hour
	long DwellTime;  //in ms
	CPoint Direction;  //in nm per 10 minuts
	CPoint NormalDirection; //normalized to 10000 nm
	CPoint Position; //in nm
	virtual ~CBeamStop();
	void SetNumber(unsigned short aNumber) {MyNumber=aNumber;};
	void ConvertCoordRealToPixel(CPoint &point);
	void ConvertCoordPixelToReal(CPoint &point);
	void ConvertCoordRealToPixel(double& point);
	void ConvertCoordPixelToReal(double &point);
	void ConvertCoordSplinePixelToPixel(CPoint &point);
	void ConvertCoordPixelToSplinePixel(CPoint &point);
	void ConvertCoordSplinePixelToPixel(double& point);
	void ConvertCoordPixelToSplinePixel(double &point);
	void GetSpline(CArray<POINT, POINT>& m_CurvePoints);
	double GetSplineLength(CArray<POINT, POINT>& CurvePoints,int EndPoint=-1,bool realcoords=true);
	double GetLaserPositionOnSpline(CArray<POINT, POINT>& CurvePoints,bool CalcIntermediatePosition=false);	
	void SetLaserPositionOnSpline(CArray<POINT, POINT>& CurvePoints,double pos);
	int GetSplinePos(CArray<POINT, POINT>& CurvePoints,double EndLength,double &addlength,bool realcoords=true,bool randomize=true);
	CPoint GetIntermediateSplinePos(CArray<POINT, POINT>& CurvePoints,double EndLength,int& i); //EndLength in pixel coord
	void CalculateLaserPosition(CBeamGuiding *BeamGuiding);
	// Generated message map functions
protected:
	//{{AFX_MSG(CBeamStop)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

};

/////////////////////////////////////////////////////////////////////////////
// BeamPath window
class CBeamPath : public CDataPointObjectList 
{
// Construction
public:
	DECLARE_SERIAL(CBeamPath);
	CBeamPath();
	CBeamPath(CBeamPath* OrgPath);
// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBeamPath)
	virtual void CBeamPath::Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:
	int Binning;
	CPoint Offset;	
	CPoint Center;	
	void CalculateLaserPosition(CBeamGuiding *BeamGuiding);
	CBeamStop* GetBeamStop(unsigned short Nr);
	unsigned short MaxNumber;
	void AddBeamStop(CBeamStop* BeamStop);
	unsigned short GetNumberStops();
	bool BeamOn;	
	virtual ~CBeamPath();

	// Generated message map functions
protected:
	//{{AFX_MSG(CBeamPath)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

};

/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// CBeamGuiding window

class CBeamGuiding : public CObject
{
// Construction
public:
	CBeamGuiding(CNeuronApp *aparent);
	i16 iDevice;
	u32 ulCount;    	
    i16* piBuffer;    

// Attributes
private:
	i16 ParabolicHopEndDAC0,ParabolicHopEndDAC1,ParabolicHopEndDAC2,ParabolicHopEndDAC3;
	i16 ParabolicHopStartDAC0,ParabolicHopStartDAC1,ParabolicHopStartDAC2,ParabolicHopStartDAC3;
	bool AntiHysteresis;
	double DAC0Frequency;
	double DAC2Frequency;
	double DAC1Frequency;
	double DAC3Frequency;
	double DAC0Amplitude;
	double DAC1Amplitude;
	double DAC2Amplitude;
	double DAC3Amplitude;
	bool AntiHysteresisAfterHop;
	double DAC0FrequencyAfterHop;
	double DAC2FrequencyAfterHop;
	double DAC1FrequencyAfterHop;
	double DAC3FrequencyAfterHop;
	double DAC0AmplitudeAfterHop;
	double DAC1AmplitudeAfterHop;
	double DAC2AmplitudeAfterHop;
	double DAC3AmplitudeAfterHop;
	double AntiHysteresisDecayTime;
	BOOL	AntiHysteresisHopToCenter;
	BOOL	AntiHysteresisHopAlsoForOneStop;
	int		AntiHysteresisHopRestingTime;
	float	FixPointVoltageDAC0;
	float	FixPointVoltageDAC1;
	float	FixPointVoltageDAC2;
	float	FixPointVoltageDAC3;
	double InsertTime;
	long HopList[100];
	int AktHop;
// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBeamGuiding)
	//}}AFX_VIRTUAL

// Implementation
public:
	void InitialUpdate(bool aHardwareAccess);
	bool BeamPathChanged;
	void ConfigDlg();
	bool SinusSweep;
	u32 Starti;
	double LaserCellOverlap;
	double ScanSpeed;
	double ParabolicHopTime;
	CPoint GetLaserLongAxisVector(CPoint pos);
	void SetBeamIntense(bool Intensity);
	bool GetBeamIntensity() {return BeamIntensity;}
	bool BeamIntensity;
	bool HardwareAccess;
	double TotalTime;
	CNeuronApp* parent;
	bool BeamOn;
	CBeamPath* BeamPath;
	bool IsLaserOn();
	bool ToggleLaserOnOff();
	void SetNewPath(CBeamPath* NewPath, bool ForceCalculation=false);
	void SwitchBeam(bool onoff);	
	virtual ~CBeamGuiding();
	void SerializeParams(CArchive &ar);

	// Generated message map functions
protected:
	//{{AFX_MSG(CBeamGuiding)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

private:
	bool ScanningModeOn;
	bool ParabolicHopEndFound;
	u32 BeamPathBufferLength;
	i16* BeamPathBuffer;
	unsigned long ParabolicHopStarti;
	f64 Frequency;
	f64 WobbleXFrequency;
	f64 WobbleYFrequency;
	void ConvertPositionToVoltage(CPoint Pos, bool TopPoint, double &X2, double &Y2);	
	void CalcX1Y1Position(const double X2,const double Y2,double &X1, double &Y1);
	void SetVoltages();
	void SwitchScanningModeOn(u32 TransferBufferSize, i16 *TransferBuffer, bool NewBuffer=true);
	void SwitchScanningModeOff();
	void SwitchScanningModeOffAndSetVoltages(double aVoltage0,double aVoltage1,double aVoltage2,double aVoltage3);
	double Voltage0,Voltage1,Voltage2,Voltage3,OffVoltage0,OffVoltage2,OffVoltage1,OffVoltage3;	
	void AddToBuffer(int i,CPoint LaserPosition,bool LaserTop);
	void AddToBuffer(int i,i16 DAC0,i16 DAC1,i16 DAC2,i16 DAC3);
	void AddParabolicHop();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CBeamStopDlg dialog

class CBeamStopDlg : public CDialog
{
// Construction
public:
	void Serialize(CArchive &ar);
	int m_ModifyOthers;
	CBeamStopDlg(CWnd* pParent = NULL);   // standard constructor

// Dialog Data
	//{{AFX_DATA(CBeamStopDlg)
	enum { IDD = IDD_BEAMSTOP_DLG };
	long	m_DirectionX;
	long	m_DirectionY;
	long	m_DwellTime;
	long	m_PositionX;
	long	m_PositionY;
	long	m_SizeX;
	long	m_SizeY;
	long	m_Speed;
	long	m_Range;
	long	m_PolynomDegree;
	long	m_OffsetX;
	long	m_OffsetY;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBeamStopDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CBeamStopDlg)
		// NOTE: the ClassWizard will add member functions here
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};
//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BEAM_H__35CBA332_F7F9_4B32_AB36_CCDD37FCDFB0__INCLUDED_)
