// DataPointDoc.cpp : implementation of the CDataPointDoc class
//

#include "stdafx.h"
#include "Neuron.h"
#include "DataPointDoc.h"
#include "UnifiedBitmap.h"
#include "CommentDlg.h"
#include "MainFrm.h"
#include "Guiding.h"
#include "hint.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDataPointDoc

IMPLEMENT_SERIAL(CDataPointDoc, CDocument,1)

BEGIN_MESSAGE_MAP(CDataPointDoc, CDocument)
	//{{AFX_MSG_MAP(CDataPointDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_SEND_MAIL, OnFileSendMail)
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, OnUpdateFileSendMail)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDataPointDoc construction/destruction

CDataPointDoc::InitParams() {
    LaserImage=NULL;
	CellImage=NULL;
	BeamPath=NULL;
	time=CTime::GetCurrentTime();
	Active=false;
	RunDocParent=NULL;
	MyNumber=0;	
	ConeDetectionImage=NULL;
	ReferenceImageNumber=0;
	ImagesSaved=false;
	CellImageStored=false;
	LaserImageStored=false;
	ConeDetectionImageStored=false;
	ImageLookupTime=CTime::GetCurrentTime();	
	Comment="";
	RelativeTime=CTimeSpan::CTimeSpan(0,0,0,0);	
	Preview=NULL;
	PreviewBinning=1;
	Message="";
	CellImageFormat=LaserImageFormat=ConeDetectionImageFormat=16;
	CellContrastMin=CellContrastMax=0;
	LaserContrastMin=LaserContrastMax=0;
	m_ContrastRegion.SetRectEmpty();
}

CDataPointDoc::CDataPointDoc()
{	
	InitParams();
}

CDataPointDoc::CDataPointDoc(CAcquisitionParams &aParam) {		
	InitParams();
	MyNumber=0;
	Param=aParam;
	CellImage=new CImage(Param.Width/Param.CellBinning,Param.Height/Param.CellBinning,0); //no init 16 bit
	if (Param.SeparateLaserImage) LaserImage=new CImage(Param.Width/Param.LaserBinning,Param.Height/Param.LaserBinning,0); //no init 16bit
	else LaserImage=NULL;
}

CDataPointDoc::~CDataPointDoc()
{
	if (CellImage) delete CellImage;
	if (LaserImage) delete LaserImage;
	if (ConeDetectionImage) delete ConeDetectionImage;
	if (BeamPath) delete BeamPath;
	if (RunDocParent) RunDocParent->DataPointDocDelete(MyNumber);
	if (Preview) delete Preview;
}

BOOL CDataPointDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CDataPointDoc serialization

void CDataPointDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{	
		ar<<Directory;
		ar<<MyNumber;
		ar<<ReferenceImageNumber;
		ar<<time;
		ar<<RelativeTime;		
		ar<<Comment;	
		ar<<Message;
		if (ImagesSaved) ar<<'Y'; else ar<<'N';		
		ar<<PreviewBinning;		
		if (CellImageStored) {
			ar << 'Y';
			ar<<CellImageFormat;			
		} else ar << 'N';
		if (LaserImageStored) {
			ar << 'Y';
			ar<<LaserImageFormat; 
		} else ar<< 'N';
		if (ConeDetectionImageStored) {
			ar << 'Y';
			ar<<ConeDetectionImageFormat; 
		} else ar<<'N';				
		if (BeamPath) {				
			ar<<'Y';
			BeamPath->Serialize(ar);
		} else ar<<'N';
		if (Preview) {
			ar<<Preview->Width;
			ar<<Preview->Height;
			unsigned char *bm=Preview->bitmap8bit;
			for (long i=0;i<Preview->Height*Preview->Width;i++) {				
				ar<<(*bm);
				bm++;
			}
		} else {
			ar<<0;
			ar<<0;
		}
		ar<<CellContrastMin;
		ar<<CellContrastMax;		
		ar<<LaserContrastMin;
		ar<<LaserContrastMax;
		//ar<<m_ContrastRegion;
	}
	else
	{
		Active=false;
		ar>>Directory;
		ar>>MyNumber;
		ar>>ReferenceImageNumber;
		ar>>time;	
		ar>>RelativeTime;			
		ar>>Comment;		
		ar>>Message;
		char help;
		ar>>help;
		ImagesSaved=(help=='Y');		
		ar>>PreviewBinning;
		ar>>help;
		CellImageStored=(help=='Y');
		if (CellImageStored) ar>>CellImageFormat;
		ar>>help;
		LaserImageStored=(help=='Y');
		if (LaserImageStored) ar>>LaserImageFormat;
		ar>>help;
		ConeDetectionImageStored=(help=='Y');
		if (ConeDetectionImageStored) ar>>ConeDetectionImageFormat;
		CellImage=NULL;
		LaserImage=NULL;
		ConeDetectionImage=NULL;
		ar>>help;
		if (help=='Y') {
			if (BeamPath) delete BeamPath;
			BeamPath=new CBeamPath();		
			BeamPath->Serialize(ar);
		} else BeamPath=NULL;
				
		long width;
		ar>>width;
		long height;
		ar>>height;
		if (Preview) delete Preview;
		if ((width>0) && (height>0)) {			
			Preview=new CImage(width,height,0,8);
			unsigned char *bm=Preview->bitmap8bit;
			for (long i=0;i<Preview->Height*Preview->Width;i++) {				
				ar>>(*bm);										
				bm++;
			}			
		} else Preview=NULL;
		ar>>CellContrastMin;
		ar>>CellContrastMax;		
		ar>>LaserContrastMin;
		ar>>LaserContrastMax;		
		//ar>>m_ContrastRegion;
		//UpdateTitle();
	}	
	Param.Serialize(ar);
	if (Param.CellBinning > 4)
	{
		Param = CAcquisitionParams();
	}
}

void CDataPointDoc::Save(CString aDirectory,bool aSaveImages)
{
	//Here a CArchive with a filename generated by the directory stored in Name and MyNumber
	//has to be created and the Serialize(Carchive) function called.
	//all the serialization methods have to be implemented.
	if (((Directory!=aDirectory) || (!ImagesSaved)) && (aSaveImages)) SaveImages(aDirectory);
	Directory=aDirectory; 
	char pFileName[200];
	sprintf(pFileName,"%sData%04i.ndp",Directory,MyNumber);
	CFile f;	
	if( !f.Open( pFileName, CFile::modeCreate | CFile::modeWrite ) ) {
		#ifdef _DEBUG
		afxDump << "Unable to open file" << "\n";
		exit( 1 );
		#endif
	}
	CArchive ar( &f, CArchive::store);
	Serialize(ar);
	ar.Close();
	f.Close();
}

bool CDataPointDoc::Load(CString aDirectory, WORD aMyNumber)
{
	//Here a CArchive with a filename generated by the directory stored in Name and MyNumber
	//has to be created and the Serialize(Carchive) function called.
	//all the serialization methods have to be implemented.	
	Directory=aDirectory;
	MyNumber=aMyNumber;
	char pFileName[200];
	sprintf(pFileName,"%sData%04i.ndp",Directory,MyNumber);
	CFile f;	
	if( !f.Open( pFileName, CFile::modeRead ) ) {
		return false;
	}
	CArchive ar( &f, CArchive::load);
	Serialize(ar);
	ar.Close();
	f.Close();	
	return true;
}

void CDataPointDoc::SaveImages(CString aDirectory)
{
	ULARGE_INTEGER lpFreeBytesAvailableToCaller; // receives the number of bytes on
                                              // disk available to the caller
	ULARGE_INTEGER lpTotalNumberOfBytes;    // receives the number of bytes on disk
	ULARGE_INTEGER lpTotalNumberOfFreeBytes; // receives the free bytes on disk
	char dir[200];
	strcpy(dir,aDirectory);	
	bool error=GetDiskFreeSpaceEx( dir,&lpFreeBytesAvailableToCaller,&lpTotalNumberOfBytes,&lpTotalNumberOfFreeBytes);
	if (((lpFreeBytesAvailableToCaller.HighPart==0) && (lpFreeBytesAvailableToCaller.LowPart<100000000)) || (error==0)) {
		AddMessage("Out of disk space");
		return;
	}

	//unsigned int mbytes = 0; //bytes taken by images

	Directory=aDirectory;
	char name[200];
	//All Images have to be saved in TGA files.
	if (CellImage) {		
		if (CellImage->EffectiveBitPerPixel==8) {
			sprintf(name,"%sCell%04i.BMP",Directory,MyNumber);
			bool maydelete=false;
			CImage * help;
			if (Param.CellAutoContrast) {
				CellImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.CellContrastLevel,CellContrastMin,CellContrastMax);				
				help=CellImage;
			} else {
				help=new CImage(CellImage);
				help->ConvertTo8BitPerPixel();
				maydelete=true;
			}
			help->SaveAsBMP(name,CRect::CRect(0,0,0,0));
			//mbytes += sizeof(help->bitmap8bit) / 1048576;
			if (maydelete) delete help;
			CellImageStored=true;	
			CellImageFormat=8;	
		} else {
			sprintf(name,"%sCell%04i.TGA",Directory,MyNumber);
			CellImage->SaveAsTGA(name);
			CellImageStored=true;	
			CellImageFormat=16;
			//mbytes += sizeof(CellImage->bitmap)/ 1048576;
			//CellImage->ConvertTo8BitPerPixel();
		}
	}
	if (LaserImage) {
		if (LaserImage->EffectiveBitPerPixel==8) {
			sprintf(name,"%sLaser%04i.BMP",Directory,MyNumber);			
			if (Param.LaserAutoContrast) LaserImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.LaserContrastLevel,LaserContrastMin,LaserContrastMax);
			else LaserImage->ConvertTo8BitPerPixel();			
			LaserImage->SaveAsBMP(name,CRect::CRect(0,0,0,0));			
			LaserImageStored=true;	
			LaserImageFormat=8;		
			//mbytes += sizeof(LaserImage->bitmap8bit)/ 1048576;
		} else {
			sprintf(name,"%sLaser%04i.TGA",Directory,MyNumber);
			LaserImage->SaveAsTGA(name);
			LaserImageStored=true;	
			LaserImageFormat=16;		
			//mbytes += sizeof(LaserImage->bitmap)/ 1048576;
		}
		LaserImage->ConvertTo8BitPerPixel();
	}
	SaveDetectionImage();
	ImagesSaved=true;
	if (Param.CellAutoContrast && CellImage) CellImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.CellContrastLevel,CellContrastMin,CellContrastMax);
	if (Param.LaserAutoContrast && LaserImage) LaserImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.LaserContrastLevel,LaserContrastMin,LaserContrastMax);

		
	//subtract image size from rundoc disklimit ()
	//int megabytes = 0;
	//if (CellImage) megabytes += sizeof(CellImage->bit
	//RunDocParent->RunDiskLimit -= image size()
}

void CDataPointDoc::SaveDetectionImage()
{
	char name[200];
	if (ConeDetectionImage) {
		if (ConeDetectionImage->EffectiveBitPerPixel==1) {			
			ConeDetectionImage->ConvertTo1BitPerPixel();
			sprintf(name,"%sCone%04i.BMP",Directory,MyNumber);			
			ConeDetectionImage->SaveAsBMP(name,CRect::CRect(0,0,0,0));
			ConeDetectionImageStored=true;
			ConeDetectionImageFormat=1;			
		} else {
			sprintf(name,"%sCone%04i.TGA",Directory,MyNumber);
			ConeDetectionImage->SaveAsTGA(name);
			ConeDetectionImageStored=true;
			ConeDetectionImageFormat=16;
		}
	}
}

void CDataPointDoc::CompressImages()
{
	if (Param.LaserAutoContrast && LaserImage) LaserImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.LaserContrastLevel,LaserContrastMin,LaserContrastMax);
	else if (LaserImage) LaserImage->ConvertTo8BitPerPixel();
	//if (CellImage) CellImage->ConvertTo8BitPerPixel();
	//if (Param.CellAutoContrast && CellImage && !RunDocParent->Selection.IsRectNull()) 
	//	CellImage->OptimizeContrast(Param.CellContrastLevel,CellContrastMin,CellContrastMax, RunDocParent->Selection);
	///else if (Param.CellAutoContrast && CellImage) 
	//	CellImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.CellContrastLevel,CellContrastMin,CellContrastMax);
	if ((ConeDetectionImage) && (ConeDetectionImage->EffectiveBitPerPixel==1)) ConeDetectionImage->ConvertTo1BitPerPixel();
}

CImage* CDataPointDoc::LookupWithLoadCellImage(bool DoNotConvert)
{	
	//All Images have to be saved in TGA files.
	if (CellImageStored && (CellImage==NULL)) {		
		if (CellImageFormat==8) {
			char name[200];
			sprintf(name,"%sCell%04i.BMP",GetDirectory(),MyNumber);
			if (CellImage) delete CellImage;
			CellImage=new CImage;
			CellImage->bitperpixel=8;
			if (!CellImage->LoadFromBMP(name,0,false)) {
				delete CellImage;
				CellImage=NULL;
			} else CellImage->ConvertTo16BitPerPixel();
		} else {		
			char name[200];
			sprintf(name,"%sCell%04i.TGA",GetDirectory(),MyNumber);
			if (CellImage) delete CellImage;
			CellImage= new CImage();
			if (!CellImage->LoadFromTGA(name)) {
				delete CellImage;
				CellImage=NULL;
			}// else CellImage->ConvertTo8BitPerPixel();
			//if (Param.CellAutoContrast && CellImage && (!DoNotConvert)) 
			//	CellImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.CellContrastLevel,CellContrastMin,CellContrastMax);
			/*
			if (CellImage && !RunDocParent->m_ContrastRegion.IsRectNull())
					CellImage->OptimizeContrast(
						Param.CellContrastLevel,
						CellContrastMin,
						CellContrastMax,
						RunDocParent->m_ContrastRegion);
			*/
		}		
	}
	ImageLookupTime=CTime::GetCurrentTime();	
	return CellImage;
}

CImage* CDataPointDoc::LookupWithLoadLaserImage()
{		
	//All Images have to be saved in TGA files.	
	if (LaserImageStored && (LaserImage==NULL)) {		
		if (LaserImageFormat==8) {
			char name[200];
			sprintf(name,"%sLaser%04i.BMP",GetDirectory(),MyNumber);
			if (LaserImage) delete LaserImage;
			LaserImage=new CImage;
			LaserImage->bitperpixel=8;
			if (!LaserImage->LoadFromBMP(name,0,false)) {
				delete LaserImage;
				LaserImage=NULL;
			}// else LaserImage->ConvertTo16BitPerPixel();
		} else {
			char name[200];
			sprintf(name,"%sLaser%04i.TGA",GetDirectory(),MyNumber);
			if (LaserImage) delete LaserImage;
			LaserImage=new CImage();
			if (!LaserImage->LoadFromTGA(name)) {
				delete LaserImage;
				LaserImage=NULL;
			} else {
				if (Param.LaserAutoContrast) LaserImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.LaserContrastLevel,LaserContrastMin,LaserContrastMax);
				else LaserImage->ConvertTo8BitPerPixel();
			}
		}
	}
	ImageLookupTime=CTime::GetCurrentTime();
	return LaserImage;
}

CImage* CDataPointDoc::LookupWithLoadConeDetectionImage()
{		
	//All Images have to be saved in TGA files.	
	if (ConeDetectionImageStored && (ConeDetectionImage==NULL)) {
		if (ConeDetectionImageFormat==1) {
			char name[200];
			sprintf(name,"%sCone%04i.BMP",GetDirectory(),MyNumber);
			if (ConeDetectionImage) delete ConeDetectionImage;
			ConeDetectionImage=new CImage;
			ConeDetectionImage->bitperpixel=1;
			if (!ConeDetectionImage->LoadFromBMP(name,0,true)) {
				delete ConeDetectionImage;
				ConeDetectionImage=NULL;
			}// else ConeDetectionImage->ConvertTo16BitPerPixel();
		} else {
			char name[200];
			sprintf(name,"%sCone%04i.TGA",GetDirectory(),MyNumber);
			if (ConeDetectionImage) delete ConeDetectionImage;
			ConeDetectionImage=new CImage;
			if (!ConeDetectionImage->LoadFromTGA(name)) {
				delete ConeDetectionImage;
				ConeDetectionImage=NULL;
			}
		}
	}
	ImageLookupTime=CTime::GetCurrentTime();
	return ConeDetectionImage;
}

/////////////////////////////////////////////////////////////////////////////
// CDataPointDoc diagnostics

#ifdef _DEBUG
void CDataPointDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDataPointDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDataPointDoc commands

void CDataPointDoc::GetDocSize(long &width, long &height)
{
  width=Param.Width/Param.CellBinning;
  height=Param.Height/Param.CellBinning;  
}

void CDataPointDoc::ConvertCoordRealToPixel(CPoint &point)
{
	point.x=point.x/(Param.Calibration.x*Param.CellBinning);
	point.y=point.y/(Param.Calibration.y*Param.CellBinning);
}

void CDataPointDoc::ConvertCoordPixelToReal(CPoint &point)
{
	point.x=point.x*Param.Calibration.x*Param.CellBinning;
	point.y=point.y*Param.Calibration.y*Param.CellBinning;
}

void CDataPointDoc::ConvertCoordRealToPixel(double& point)
{
	point=point/(Param.Calibration.x*Param.CellBinning);	
}

void CDataPointDoc::ConvertCoordPixelToReal(double &point)
{
	point=point*Param.Calibration.x*Param.CellBinning;	
}

void CDataPointDoc::AddNewLaserFocus(CPoint Position, CPoint Direction)
{
	if (RunDocParent) {
		CBeamStop* NewFocus=new CBeamStop(Position,Direction,RunDocParent->GetNeuronApp()->LaserSize,RunDocParent->GetNeuronApp()->BeamStopParamsTemplate);
		if (BeamPath==NULL) BeamPath=new CBeamPath();
		BeamPath->AddBeamStop(NewFocus);
		RunDocParent->GetNeuronApp()->BeamGuiding->SetNewPath(BeamPath);	
		UpdateAllViews(NULL, 0L, NewFocus);
	}
}

void CDataPointDoc::DeleteLaserFocus(CBeamStop* BeamStop)
{	
	if ((BeamPath!=NULL) && (RunDocParent)) {		
		if (BeamPath->Delete(BeamStop)) {
			if (Active) RunDocParent->GetNeuronApp()->BeamGuiding->SetNewPath(BeamPath);	
			UpdateAllViews(NULL, 0L);
		}
	}
}

void CDataPointDoc::CalcPreviewWinBitmap(CBitmap* WinBMP,unsigned short CellImagePixelShift,unsigned short Zoom) {
	unsigned char *bm=Preview->bitmap8bit;
	unsigned long *hbitmap=new unsigned long[Preview->Height*Preview->Width*Zoom*Zoom];
	unsigned long *hb=hbitmap;
	unsigned char *bmz;
	for (long i=0;i<Preview->Height;i++) {
		bmz=bm;
		for (int k=1;k<=Zoom;k++) {
			bm=bmz;
			for (long j=0;j<Preview->Width;j++) {
				char col=(*bm) << CellImagePixelShift;
				COLORREF color=RGB(col,col,col);
				for (int l=1;l<=Zoom;l++) {
					*hb=color;			
					hb++;		
				}
				bm++;
			}			
		}		
	}
	WinBMP->SetBitmapBits(Preview->Height*Preview->Width*4*Zoom*Zoom,hbitmap);		
	delete hbitmap;	
}

void CDataPointDoc::CalcPreview(int aPreviewBinning) {
	if ((CellImage || CellImageStored) && (!Preview)) {
		if (CellImage==NULL) LookupWithLoadCellImage();
		PreviewBinning=aPreviewBinning;
		if (!Preview) {
			Preview=new CImage(CellImage->Width/PreviewBinning,CellImage->Height/PreviewBinning,0,8);//8bit grayscale
		}
		if (CellImage->bitperpixel==16) {
			long x=0;
			long y=0;
			unsigned char *bm=Preview->bitmap8bit;
			for (long i=0;i<Preview->Height;i++) {
				for (long j=0;j<Preview->Width;j++) {
					*bm=CellImage->bitmap[x+y*CellImage->Width]/256;
					bm++;
					x+=PreviewBinning;
					if (x>CellImage->Width) x=CellImage->Width;
				}	
				x=0;
				y+=PreviewBinning;
				if (y>CellImage->Height) y=CellImage->Height;
			}
		} else if (CellImage->bitperpixel==8) {
			long x=0;
			long y=0;
			unsigned char *bm=Preview->bitmap8bit;
			for (long i=0;i<Preview->Height;i++) {
				for (long j=0;j<Preview->Width;j++) {
					*bm=CellImage->bitmap8bit[x+y*CellImage->Width];
					bm++;
					x+=PreviewBinning;
					if (x>CellImage->Width) x=CellImage->Width;
				}	
				x=0;
				y+=PreviewBinning;
				if (y>CellImage->Height) y=CellImage->Height;
			}
		}
	}
};

void CDataPointDoc::CalcUnifiedBitmap(CDC* pDC,CUnifiedBitmap *&UnifiedBitmap,
								 bool aCellImage, COLORREF CellColor,
								 bool aLaserImage, COLORREF LaserColor,
								 bool aCompareImage, COLORREF CompareColor,
								 bool aConeDetectionImage, 
								 bool InverseConeDetectionImage, 
								 COLORREF ConeColor,
								 unsigned char aCellImagePixelShift,
								 bool ForceRecalculateUnifiedBitmap) {	
	if (!UnifiedBitmap) {
		LookupWithLoadCellImage();
		if (CellImage) {			
			UnifiedBitmap=new CUnifiedBitmap();  //UnifiedBitmap was NULL, tested above
		} else return;
	}
	bool HaveToCalc=UnifiedBitmap->Bitmap==NULL;
	HaveToCalc=HaveToCalc || ForceRecalculateUnifiedBitmap;
	HaveToCalc=HaveToCalc || (UnifiedBitmap->CellImageInBitmap!=aCellImage);
	HaveToCalc=HaveToCalc || (UnifiedBitmap->LaserImageInBitmap!=aLaserImage);
	HaveToCalc=HaveToCalc || (UnifiedBitmap->CompareImageInBitmap!=aCompareImage);
	HaveToCalc=HaveToCalc || (UnifiedBitmap->ConeDetectionImageInBitmap!=aConeDetectionImage);
	HaveToCalc=HaveToCalc || (UnifiedBitmap->CellImagePixelShift!=aCellImagePixelShift);
	UnifiedBitmap->CellImagePixelShift=aCellImagePixelShift;
	UnifiedBitmap->CellImageInBitmap=aCellImage;
	UnifiedBitmap->LaserImageInBitmap=aLaserImage;
	UnifiedBitmap->CompareImageInBitmap=aCompareImage;
	UnifiedBitmap->ConeDetectionImageInBitmap=aConeDetectionImage;
	if (HaveToCalc && (CellImage!=NULL)) {			
		unsigned long *hbitmap=new unsigned long[CellImage->Height*CellImage->Width];
		unsigned long *hbm=hbitmap;
		unsigned short *cbm=NULL;
		unsigned char *cbm8bit=NULL;
		unsigned short *lbm=NULL;
		unsigned char *lbm8bit=NULL;
		unsigned short *obm=NULL;
		unsigned char *obm8bit=NULL;
		unsigned short *odbm=NULL;
		CImage* odbm32bit=NULL;
		unsigned short *dbm=NULL;
		CImage* dbm32bit=NULL;
		if (UnifiedBitmap->CellImageInBitmap) {
			LookupWithLoadCellImage();
			OptimizeContrast();
		}
		if (UnifiedBitmap->LaserImageInBitmap) LookupWithLoadLaserImage();
		if (UnifiedBitmap->ConeDetectionImageInBitmap) LookupWithLoadConeDetectionImage();
		if ((CellImage) && UnifiedBitmap->CellImageInBitmap) {
			cbm=CellImage->bitmap; 
			cbm8bit=CellImage->bitmap8bit; 
		} else {
			cbm=NULL;
			cbm8bit=NULL;
		}
		if ((LaserImage) && UnifiedBitmap->LaserImageInBitmap) {
			lbm=LaserImage->bitmap;
			lbm8bit=LaserImage->bitmap8bit;
		} else {
			lbm=NULL;
			lbm8bit=NULL;
		}
		CImage *CompareImage=NULL;
		CImage *CompareConeDetectionImage=NULL;
		if (UnifiedBitmap->CompareImageInBitmap  && RunDocParent) {
			long mode=RunDocParent->GetNeuronApp()->ReferenceImageMode;
			long fixNr=RunDocParent->GetNeuronApp()->ReferenceImageFixNr;
			long relativeDistance=RunDocParent->GetNeuronApp()->ReferenceImageRelativeDistance;
			WORD ReferenceNumber=0;
			switch (mode) {
				case 0:{
					ReferenceNumber=fixNr;
					break;
					   }
				case 1:{
					if (relativeDistance<=MyNumber)	ReferenceNumber=MyNumber-relativeDistance;
					else ReferenceNumber=0;
					break;
					   }
			}
			CDataPointDoc* ReferenceDoc=NULL;
			do {
				if (RunDocParent->DataPointMap->LookupWithLoad(ReferenceNumber,ReferenceDoc)) {				
					CompareImage=ReferenceDoc->LookupWithLoadCellImage();					
					CompareConeDetectionImage=ReferenceDoc->LookupWithLoadConeDetectionImage();					
				}
				if (CompareImage==NULL) ReferenceNumber--;
			} while	((CompareImage==NULL) && (ReferenceNumber>0));
		}
		if ((CompareImage) && UnifiedBitmap->CompareImageInBitmap) {
			if (CompareImage) {
				obm=CompareImage->bitmap; 
				obm8bit=CompareImage->bitmap8bit; 
			} else {
				obm=NULL;		
				obm8bit=NULL;		
			}
			if (CompareConeDetectionImage) {
				if (CompareConeDetectionImage->bitperpixel==1) odbm32bit=CompareConeDetectionImage;
				else odbm=CompareConeDetectionImage->bitmap;
			} else {
				odbm=NULL;
				odbm32bit=NULL;
			}
		} else {
			obm=NULL;
			obm8bit=NULL;
			odbm=NULL;
			odbm32bit=NULL;
		}
		if ((ConeDetectionImage) && UnifiedBitmap->ConeDetectionImageInBitmap) {
			if (ConeDetectionImage->bitperpixel==1) {
				dbm32bit=ConeDetectionImage; 
			} else dbm=ConeDetectionImage->bitmap;			
		} else {
			dbm=NULL;
			dbm32bit=NULL;
		}
		//We assume that the CellImage has the highest resolution and that the other images are produced by
		//binning from this imagesize. This can be written much more general and with error detection. 
		//So, get binning factors...
		int lbin,lbinaktx=0,lbinakty=0;
		int obin,obinaktx=0,obinakty=0;
		int dbin,dbinaktx=0,dbinakty=0;
		if (lbm || lbm8bit) lbin=CellImage->Height/LaserImage->Height;
		if (obm || obm8bit) obin=CellImage->Height/CompareImage->Height;
		if (dbm || dbm32bit) dbin=CellImage->Height/ConeDetectionImage->Height;
		//Values at start of line;
		unsigned short *lslbm=lbm;
		unsigned short *oslbm=obm;
		unsigned char *lslbm8bit=lbm8bit;
		unsigned char *oslbm8bit=obm8bit;		
		unsigned short *dslbm=dbm;		
		unsigned short cr=GetRValue(CellColor);
		unsigned short cg=GetGValue(CellColor);
		unsigned short cb=GetBValue(CellColor);
		unsigned short lr=GetRValue(LaserColor);
		unsigned short lg=GetGValue(LaserColor);
		unsigned short lb=GetBValue(LaserColor);
		unsigned short or=GetRValue(CompareColor);
		unsigned short og=GetGValue(CompareColor);
		unsigned short ob=GetBValue(CompareColor);
		unsigned short dr=GetRValue(ConeColor);
		unsigned short dg=GetGValue(ConeColor);
		unsigned short db=GetBValue(ConeColor);		
		for (long i=0;i<CellImage->Height;i++) {
			lbm8bit=lslbm8bit;
			obm8bit=oslbm8bit;
			lbm=lslbm;
			obm=oslbm;
			dbm=dslbm;					
			for (long j=0;j<CellImage->Width;j++) {
				lbinaktx++;obinaktx++;dbinaktx++;
				unsigned short red=0,green=0,blue=0;
				if (cbm || cbm8bit) {  //cellimage
					unsigned char help;
					if (cbm) help=(((*cbm)<<aCellImagePixelShift)/256);
					else help=(*cbm8bit)<<aCellImagePixelShift;
					if (dbm) {   //detection image																		
						bool enhancedcolor=false;
						if (InverseConeDetectionImage) {
							if ((*dbm)<32768) enhancedcolor=true;
						} else {
							if ((*dbm)>32768) enhancedcolor=true;
						}
						if (enhancedcolor) {
							red+=dr*help/256;
							green+=dg*help/256;
							blue+=db*help/256;					
						} else {
							red+=cr*help/256;
							green+=cg*help/256;
							blue+=cb*help/256;
						}
						if (dbinaktx>=dbin) {
							dbm++;
							dbinaktx=0;
						}
					} if (dbm32bit) {						
						bool enhancedcolor=dbm32bit->GetPixel1Bit(j,i);
						if (InverseConeDetectionImage) enhancedcolor=!enhancedcolor;						
						if (enhancedcolor) {
							red+=dr*help/256;
							green+=dg*help/256;
							blue+=db*help/256;					
						} else {
							red+=cr*help/256;
							green+=cg*help/256;
							blue+=cb*help/256;
						}						
					} else {  //only cell image						
						red+=cr*help/256;
						green+=cg*help/256;
						blue+=cb*help/256;
					}
					if (cbm) cbm++;
					else cbm8bit++;
				}				
				if (lbm || lbm8bit) {  //laser image
					unsigned char help;
					if (lbm) help=(*lbm/256);
					else help=(*lbm8bit);
					red+=lr*help/256;
					green+=lg*help/256;
					blue+=lb*help/256;
					if (lbinaktx>=lbin) {
						if (lbm) lbm++;
						else lbm8bit++;
						lbinaktx=0;
					}
				}
				if (obm || obm8bit) {  //compare image
					unsigned char help;
					if (obm) help=(((*obm)<<aCellImagePixelShift)/256);
					else help=(*obm8bit)<<aCellImagePixelShift;
					if (odbm) {			//compare detection image			
						if (InverseConeDetectionImage) {
							if ((*odbm)>32768) help=0;					
						} else {
							if ((*odbm)<32768) help=0;					
						}
					} else if (odbm32bit) {			//compare detection image			
						bool help2=odbm32bit->GetPixel1Bit(j,i);
						if (!InverseConeDetectionImage) help2=!help2;							
						if (help2) help=0;
					}
					red+=or*help/256;
					green+=og*help/256;
					blue+=ob*help/256;					
					if (obinaktx>=obin) {
						if (odbm) odbm++;
						if (obm) obm++;
						else obm8bit++;
						obinaktx=0;
					}
				}
				if ((dbm) && (cbm==NULL) && (cbm8bit==NULL)) {  //detection image but no cell image
					unsigned char help=(*dbm/256);
					if (InverseConeDetectionImage) help=255-help;
					red+=dr*help/256;
					green+=dg*help/256;
					blue+=db*help/256;					
					if (dbinaktx>=dbin) {
						dbm++;
						dbinaktx=0;
					}
				}
				if ((dbm32bit) && (cbm==NULL) && (cbm8bit==NULL)) {  //detection image but no cell image
					bool help=dbm32bit->GetPixel1Bit(j,i);
					if (InverseConeDetectionImage) help=!help;
					if (help) {
						red+=dr;
						green+=dg;
						blue+=db;					
					}					
				}
				if (red>255) red=255;
				if (green>255) green=255;
				if (blue>255) blue=255;
				*hbm=RGB(blue,green,red); //want color in CBitmap format, red and blue exchanged
				hbm++;
			}
			lbinakty++;obinakty++;dbinakty++;				
			if ((lbm) && (lbinakty>=lbin)) {
				lslbm=lbm;
				lbinakty=0;
			} else if ((lbm8bit) && (lbinakty>=lbin)) {
				lslbm8bit=lbm8bit;
				lbinakty=0;
			}
			if ((obm) && (obinakty>=obin)) {
				oslbm=obm;
				obinakty=0;
			} else if ((obm8bit) && (obinakty>=obin)) {
				oslbm8bit=obm8bit;
				obinakty=0;
			}
			if ((dbm) && (dbinakty>=dbin)) {
				dslbm=dbm;
				dbinakty=0;
			}
		}					
		if (UnifiedBitmap->Bitmap) delete UnifiedBitmap->Bitmap;
		UnifiedBitmap->Bitmap=new CBitmap();
		UnifiedBitmap->Bitmap->CreateCompatibleBitmap(pDC,CellImage->Width,CellImage->Height);
		ASSERT(UnifiedBitmap->Bitmap->m_hObject != NULL);
		UnifiedBitmap->Bitmap->SetBitmapBits(CellImage->Width*CellImage->Height*4,hbitmap);						
		delete hbitmap;
	}
}

CImage* CDataPointDoc::CalcUnifiedBitmap( 
								 CRect region,
								 bool aCellImage, COLORREF CellColor,
								 bool aLaserImage, COLORREF LaserColor,
								 bool aCompareImage, COLORREF CompareColor,
								 bool aConeDetectionImage, 
								 bool InverseConeDetectionImage, COLORREF ConeColor,
								 unsigned char aCellImagePixelShift) 
{	
	//unsigned long *hbitmap = new unsigned long[region.Width() * region.Height()];
	unsigned long *hbitmap = new unsigned long[CellImage->Height*CellImage->Width];
	unsigned long *hbm=hbitmap;
	unsigned short *cbm=NULL;
	unsigned char *cbm8bit=NULL;
	unsigned short *lbm=NULL;
	unsigned char *lbm8bit=NULL;
	unsigned short *obm=NULL;
	unsigned char *obm8bit=NULL;
	unsigned short *odbm=NULL;
	CImage* odbm32bit=NULL;
	unsigned short *dbm=NULL;
	CImage* dbm32bit=NULL;
	
	if (aCellImage)
	{
		LookupWithLoadCellImage();
		OptimizeContrast();
		if (CellImage)
		{
			cbm=CellImage->bitmap; 
			cbm8bit=CellImage->bitmap8bit;
		}
	}
	if (aLaserImage)
	{
		LookupWithLoadLaserImage();
		if (LaserImage)
		{
			lbm=LaserImage->bitmap;
			lbm8bit=LaserImage->bitmap8bit;
		}
	}
	if (aConeDetectionImage)
	{
		LookupWithLoadConeDetectionImage();
		if (ConeDetectionImage->bitperpixel==1)
		{
			dbm32bit=ConeDetectionImage; 
		} else 
			dbm=ConeDetectionImage->bitmap;		
	}

	CImage *CompareImage=NULL;
	CImage *CompareConeDetectionImage=NULL;
	if (aCompareImage  && RunDocParent) 
	{
		long mode=RunDocParent->GetNeuronApp()->ReferenceImageMode;
		long fixNr=RunDocParent->GetNeuronApp()->ReferenceImageFixNr;
		long relativeDistance=RunDocParent->GetNeuronApp()->ReferenceImageRelativeDistance;
		WORD ReferenceNumber=0;
		switch (mode) 
		{
			case 0:{
				ReferenceNumber=fixNr;
				break;
				   }
			case 1:{
				if (relativeDistance<=MyNumber)	ReferenceNumber=MyNumber-relativeDistance;
				else ReferenceNumber=0;
				break;
				   }
		}
		CDataPointDoc* ReferenceDoc=NULL;
		do 
		{
			if (RunDocParent->DataPointMap->LookupWithLoad(ReferenceNumber,ReferenceDoc)) 
			{				
				CompareImage=ReferenceDoc->LookupWithLoadCellImage();					
				CompareConeDetectionImage=ReferenceDoc->LookupWithLoadConeDetectionImage();					
			}
			if (CompareImage==NULL) ReferenceNumber--;
		} while	((CompareImage==NULL) && (ReferenceNumber>0));

		if (CompareImage) {
			obm=CompareImage->bitmap; 
			obm8bit=CompareImage->bitmap8bit; 
		}
		if (CompareConeDetectionImage) 
		{
			if (CompareConeDetectionImage->bitperpixel==1) 
				odbm32bit=CompareConeDetectionImage;
			else 
				odbm=CompareConeDetectionImage->bitmap;
		} 
	}
	//We assume that the CellImage has the highest resolution and that the other images are produced by
	//binning from this imagesize. This can be written much more general and with error detection. 
	//So, get binning factors...
	int lbin,lbinaktx=0,lbinakty=0;
	int obin,obinaktx=0,obinakty=0;
	int dbin,dbinaktx=0,dbinakty=0;
	if (lbm || lbm8bit) lbin=CellImage->Height/LaserImage->Height;
	if (obm || obm8bit) obin=CellImage->Height/CompareImage->Height;
	if (dbm || dbm32bit) dbin=CellImage->Height/ConeDetectionImage->Height;
	//Values at start of line;
	unsigned short *lslbm=lbm;
	unsigned short *oslbm=obm;
	unsigned char *lslbm8bit=lbm8bit;
	unsigned char *oslbm8bit=obm8bit;		
	unsigned short *dslbm=dbm;		
	unsigned short cr=GetRValue(CellColor);
	unsigned short cg=GetGValue(CellColor);
	unsigned short cb=GetBValue(CellColor);
	unsigned short lr=GetRValue(LaserColor);
	unsigned short lg=GetGValue(LaserColor);
	unsigned short lb=GetBValue(LaserColor);
	unsigned short or=GetRValue(CompareColor);
	unsigned short og=GetGValue(CompareColor);
	unsigned short ob=GetBValue(CompareColor);
	unsigned short dr=GetRValue(ConeColor);
	unsigned short dg=GetGValue(ConeColor);
	unsigned short db=GetBValue(ConeColor);		
	for (long i=0;i<CellImage->Height;i++) {
		lbm8bit=lslbm8bit;
		obm8bit=oslbm8bit;
		lbm=lslbm;
		obm=oslbm;
		dbm=dslbm;					
		for (long j=0;j<CellImage->Width;j++) {
			lbinaktx++;obinaktx++;dbinaktx++;
			unsigned short red=0,green=0,blue=0;
			if (cbm || cbm8bit) {  //cellimage
				unsigned char help;
				if (cbm) 
					help=(((*cbm)<<aCellImagePixelShift)/256);
				else 
					help=(*cbm8bit)<<aCellImagePixelShift;
				if (dbm) {   //detection image																		
					bool enhancedcolor=false;
					if (InverseConeDetectionImage) {
						if ((*dbm)<32768) enhancedcolor=true;
					} else {
						if ((*dbm)>32768) enhancedcolor=true;
					}
					if (enhancedcolor) {
						red+=dr*help/256;
						green+=dg*help/256;
						blue+=db*help/256;					
					} else {
						red+=cr*help/256;
						green+=cg*help/256;
						blue+=cb*help/256;
					}
					if (dbinaktx>=dbin) {
						dbm++;
						dbinaktx=0;
					}
				} if (dbm32bit) {						
					bool enhancedcolor=dbm32bit->GetPixel1Bit(j,i);
					if (InverseConeDetectionImage) enhancedcolor=!enhancedcolor;						
					if (enhancedcolor) {
						red+=dr*help/256;
						green+=dg*help/256;
						blue+=db*help/256;					
					} else {
						red+=cr*help/256;
						green+=cg*help/256;
						blue+=cb*help/256;
					}						
				} else {  //only cell image						
					red+=cr*help/256;
					green+=cg*help/256;
					blue+=cb*help/256;
				}
				if (cbm) cbm++;
				else cbm8bit++;
			}				
			if (lbm || lbm8bit) {  //laser image
				unsigned char help;
				if (lbm) help=(*lbm/256);
				else help=(*lbm8bit);
				red+=lr*help/256;
				green+=lg*help/256;
				blue+=lb*help/256;
				if (lbinaktx>=lbin) {
					if (lbm) lbm++;
					else lbm8bit++;
					lbinaktx=0;
				}
			}
			if (obm || obm8bit) {  //compare image
				unsigned char help;
				if (obm) help=(((*obm)<<aCellImagePixelShift)/256);
				else help=(*obm8bit)<<aCellImagePixelShift;
				if (odbm) {			//compare detection image			
					if (InverseConeDetectionImage) {
						if ((*odbm)>32768) help=0;					
					} else {
						if ((*odbm)<32768) help=0;					
					}
				} else if (odbm32bit) {			//compare detection image			
					bool help2=odbm32bit->GetPixel1Bit(j,i);
					if (!InverseConeDetectionImage) help2=!help2;							
					if (help2) help=0;
				}
				red+=or*help/256;
				green+=og*help/256;
				blue+=ob*help/256;					
				if (obinaktx>=obin) {
					if (odbm) odbm++;
					if (obm) obm++;
					else obm8bit++;
					obinaktx=0;
				}
			}
			if ((dbm) && (cbm==NULL) && (cbm8bit==NULL)) {  //detection image but no cell image
				unsigned char help=(*dbm/256);
				if (InverseConeDetectionImage) help=255-help;
				red+=dr*help/256;
				green+=dg*help/256;
				blue+=db*help/256;					
				if (dbinaktx>=dbin) {
					dbm++;
					dbinaktx=0;
				}
			}
			if ((dbm32bit) && (cbm==NULL) && (cbm8bit==NULL)) {  //detection image but no cell image
				bool help=dbm32bit->GetPixel1Bit(j,i);
				if (InverseConeDetectionImage) help=!help;
				if (help) {
					red+=dr;
					green+=dg;
					blue+=db;					
				}					
			}
			if (red>255) red=255;
			if (green>255) green=255;
			if (blue>255) blue=255;
			*hbm=RGB(blue,green,red); //want color in CBitmap format, red and blue exchanged
			hbm++;
		}
		lbinakty++;obinakty++;dbinakty++;				
		if ((lbm) && (lbinakty>=lbin)) {
			lslbm=lbm;
			lbinakty=0;
		} else if ((lbm8bit) && (lbinakty>=lbin)) {
			lslbm8bit=lbm8bit;
			lbinakty=0;
		}
		if ((obm) && (obinakty>=obin)) {
			oslbm=obm;
			obinakty=0;
		} else if ((obm8bit) && (obinakty>=obin)) {
			oslbm8bit=obm8bit;
			obinakty=0;
		}
		if ((dbm) && (dbinakty>=dbin)) {
			dslbm=dbm;
			dbinakty=0;
		}
	}					
						
	CImage* result = new CImage();
	result->bitmap32bit = hbitmap;
	result->bitperpixel = 16;
	result->Height = CellImage->Height;
	result->Width = CellImage->Width;
	return result;
}

void CDataPointDoc::DeleteImages()
{
	if (CellImage) delete CellImage;
	CellImage=NULL;
	if (LaserImage) delete LaserImage;
	LaserImage=NULL;
	if (ConeDetectionImage) delete ConeDetectionImage;
	ConeDetectionImage=NULL;	
	if (Preview && (!CellImageStored)) {
		delete Preview;
		Preview=NULL;
	}
/*	char buf[200];
	sprintf(buf,"Delete Images of DataPoint Nr %u",MyNumber);
	AddMessage(buf);*/
}

CString CDataPointDoc::GetTimeStamp()
{
	long h=RelativeTime.GetTotalHours();
	double m=RelativeTime.GetMinutes();
	double s=RelativeTime.GetSeconds();
	char buf[200];
	sprintf(buf,"%i:%02.0f:%02.0f",h,m,s);
	return buf;
}


CString CDataPointDoc::UpdateTitle()
{
	CString title;
	title.Format("Picture %i/%i",MyNumber, RunDocParent->LastPictureNumber);
	title += "   " + GetTimeStamp();
	if (Active) title+="   active";
	SetTitle(title);	
	return title;
}

void CDataPointDoc::AddSpline(CPoint point)  //point expected in pixel coords
{
	int subPoint;
	CBeamStop* BeamStop=FindNearestBeamStop(point,subPoint);	
	if (BeamStop) {
		ConvertCoordPixelToReal(point);
		BeamStop->AddSpline(point,Param.Calibration,Param.CellBinning);		
		if (Active) RunDocParent->GetNeuronApp()->BeamGuiding->SetNewPath(BeamPath);
		UpdateAllViews(NULL, 0L);
	}	
}

void CDataPointDoc::Move(CPoint MoveStartPoint,CPoint point,CBeamStop*& MovedStop, int &MovedSubPoint) {  //points expected in pixel coords
	if (MovedStop==NULL) {
		int subPoint;
		CBeamStop* BeamStop=FindNearestBeamStop(MoveStartPoint,subPoint);
		if (BeamStop) {
			MovedStop=BeamStop;
			MovedSubPoint=subPoint;
		}
	}
	if ((MovedStop) && (CellImage)) {
		CRect Limit(0,0,CellImage->Width,CellImage->Height);
		ConvertCoordPixelToReal(Limit.TopLeft());
		ConvertCoordPixelToReal(Limit.BottomRight());
		CPoint Move=point-MoveStartPoint;								
		ConvertCoordPixelToReal(Move);
		CRect rec=MovedStop->GetBoundingRect();
		MovedStop->Move(Move,Limit,MovedSubPoint,this);
		if (Active) RunDocParent->GetNeuronApp()->BeamGuiding->SetNewPath(BeamPath);	
		CRect rec2=MovedStop->GetBoundingRect();
		rec.UnionRect(rec,rec2);
		MovedStop->SetBoundingRect(rec);					
		UpdateAllViews(NULL, 0L, MovedStop);
		MovedStop->CalcBoundingRect();	
	}
}

CBeamStop* CDataPointDoc::FindNearestBeamStop(CPoint point,int &subPoint) //point expected in pixel coords
{
	if (BeamPath) {
		CPoint hpoint=point;
		ConvertCoordPixelToReal(hpoint);
		unsigned long dist;
		CPoint pos;
		CBeamStop* BeamStop=((CBeamStop*)BeamPath->GetNearest(hpoint,pos,dist,subPoint));
		if (BeamStop) {
			ConvertCoordRealToPixel(pos);			
			pos-=point;
			if ((pos.x*pos.x+pos.y*pos.y)<100000) {
				return BeamStop;			
			}
		}	
	}
	return NULL;	
}

void CDataPointDoc::SaveAsBMPParam(CRect Selection,bool m_CellImage,bool m_LaserImage,bool m_DetectionImage)
{
    if (RunDocParent) {		
		if (RunDocParent->GetRunDirectory(true).GetLength()!=0) {		
			char name[200];
			//Selected Images have to be saved in BMP files.
			if (CellImageStored && m_CellImage && LaserImageStored && m_LaserImage)
			{
				OptimizeContrast();
				sprintf(name,"%sCellandLaser%04i.BMP",RunDocParent->GetRunDirectory(true),MyNumber);
				
				COLORREF CellCol,LaserCol;
				CellCol=RGB(128+64,128+64,128+64);
				LaserCol=RGB(128,0,0);

				CImage* image = CalcUnifiedBitmap(Selection, true, CellCol, true, CellCol, false, NULL,
					false, false, NULL, 1);
				
				image->SaveAsBMP(name, Selection);
			}
			else if ((CellImageStored) && (m_CellImage)) {		
				if (CellImage==NULL) LookupWithLoadCellImage();
				OptimizeContrast();
				sprintf(name,"%sSCell%04i.BMP",RunDocParent->GetRunDirectory(true),MyNumber);
				CellImage->SaveAsBMP(name,Selection);			
			}						
			else if ((LaserImageStored) && (m_LaserImage)) {	
				if (LaserImage==NULL) LookupWithLoadLaserImage();
				CRect Sel=Selection;
				if (CellImage) {
					double lbin=CellImage->Height/LaserImage->Height;
					Sel.left*=lbin;
					Sel.right*=lbin;
					Sel.top*=lbin;
					Sel.bottom*=lbin;
				}
				sprintf(name,"%sSLaser%04i.BMP",RunDocParent->GetRunDirectory(true),MyNumber);
				LaserImage->SaveAsBMP(name,Sel);			
			}
			if ((ConeDetectionImageStored) && (m_DetectionImage)) {	
				if (ConeDetectionImage==NULL) LookupWithLoadConeDetectionImage();
				sprintf(name,"%sSDetect%04i.BMP",RunDocParent->GetRunDirectory(true),MyNumber);
				ConeDetectionImage->SaveAsBMP(name,Selection);			
			}
		}
	}
}

void CDataPointDoc::ChangeComment()
{
	CCommentDlg dlg;
	dlg.m_Comment=Comment;	
	if (dlg.DoModal() == IDOK) {
		Comment=dlg.m_Comment;					
		if (RunDocParent) RunDocParent->CommentChanged();
	}		
}

void CDataPointDoc::AddMessage(CString aMessage)
{
	Message+=aMessage;
	CString strCurPos;
	strCurPos.Format(ID_INDICATOR_MESSAGE, aMessage);
	((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->SetPaneText(((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->CommandToIndex(ID_INDICATOR_MESSAGE),strCurPos);
	if (RunDocParent) RunDocParent->MessageChanged();
}

CString CDataPointDoc::GetDirectory()
{
	if (RunDocParent) return RunDocParent->GetRunDirectory(false);
	else return Directory;
}

void CDataPointDoc::CalcDetectionImage() {

	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_WAIT));
	if ((!CellImage) || (CellImage->bitperpixel!=16)) {
		delete CellImage;
		CellImage=NULL;
		LookupWithLoadCellImage(true);
	}
	if ((!CellImage) || (CellImage->bitperpixel!=16)) return;

	if (! m_DetectionRegion.IsRectEmpty())
		RunDocParent->GetNeuronApp()->Guiding->CalcDetectionImage(this, m_DetectionRegion);
	else
		RunDocParent->GetNeuronApp()->Guiding->CalcDetectionImage(this);
	SaveDetectionImage();
	if (Param.CellAutoContrast && ConeDetectionImage) {
		ConeDetectionImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.CellContrastLevel,CellContrastMin,CellContrastMax);
		ConeDetectionImage->ConvertTo16BitPerPixel();
	}
	//if (Param.CellAutoContrast && CellImage) CellImage->OptimizeContrastAndConvertTo8BitPerPixel(Param.CellContrastLevel,CellContrastMin,CellContrastMax);		
	CHint Help(4);
	UpdateAllViews(NULL,0,&Help);
	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
}

void CDataPointDoc::OptimizeContrast()
{
	if (CellImage && !m_ContrastRegion.IsRectNull())
		CellImage->OptimizeContrast(
			Param.CellContrastLevel,
			CellContrastMin,
			CellContrastMax,
			m_ContrastRegion);
	else if (CellImage && Param.CellAutoContrast)
		CellImage->OptimizeContrastAndConvertTo8BitPerPixel(
			Param.CellContrastLevel,
			CellContrastMin,
			CellContrastMax);
}
