// Guiding.cpp : implementation file
//

#include "stdafx.h"
#include "Neuron.h"
#include "Guiding.h"
#include "GuidingDlg.h"
#include <fstream.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGuiding

CGuiding::CGuiding()
{
	IsGuiding=false;	
	m_BeamAdvance=0;
	GuidingMode=2;
	LastGuidingTime=CTime::GetCurrentTime();
	m_MaxStepsize=1000;
	m_BeamAdvance=0;	
	m_CropBorders=10;
	m_EdgeFilterEnd=4;
	m_EdgeFilterStart=4;
	m_EraseSmaller1=0;
	m_EraseSmaller2=10000;
	m_FillGapStart1=0;
	m_FillGapStop1=0;
	m_FillGapStart2=2;
	m_FillGapStop2=2;
	m_MedianLevel=100;
	m_StopAfterStep=8;	
	m_MedianSize=3;
	m_FloodFillOutsideSize=10000;
	m_ExaminedSurrounding=0;
	ExtractCellArea=true;
	ExtractAxonArea=true;
}

CGuiding::~CGuiding()
{

}

/////////////////////////////////////////////////////////////////////////////
// CGuiding message handlers
void CGuiding::OnGuidingDialog() 
{
	CGuidingDlg dlg;	
	dlg.m_GuidingMode=GuidingMode;	
	dlg.m_BeamAdvance=m_BeamAdvance;
	dlg.m_MaxStepsize=m_MaxStepsize;
	dlg.m_CropBorders=m_CropBorders;
	dlg.m_EdgeFilterEnd=m_EdgeFilterEnd;
	dlg.m_EdgeFilterStart=m_EdgeFilterStart;
	dlg.m_EraseSmaller1=m_EraseSmaller1;
	dlg.m_EraseSmaller2=m_EraseSmaller2;
	dlg.m_FillGapStart1=m_FillGapStart1;
	dlg.m_FillGapStop1=m_FillGapStop1;
	dlg.m_FillGapStart2=m_FillGapStart2;
	dlg.m_FillGapStop2=m_FillGapStop2;
	dlg.m_MedianLevel=m_MedianLevel;
	dlg.m_StopAfterStep=m_StopAfterStep;
	dlg.m_MedianSize=m_MedianSize;
	dlg.m_FloodFillOutsideSize=m_FloodFillOutsideSize;
	dlg.m_ExaminedSurrounding=m_ExaminedSurrounding;
	if (dlg.DoModal() == IDOK) {		
		m_BeamAdvance=dlg.m_BeamAdvance;
		m_MaxStepsize=dlg.m_MaxStepsize;
		m_CropBorders=dlg.m_CropBorders;
		m_EdgeFilterEnd=dlg.m_EdgeFilterEnd;
		m_EdgeFilterStart=dlg.m_EdgeFilterStart;
		m_EraseSmaller1=dlg.m_EraseSmaller1;
		m_EraseSmaller2=dlg.m_EraseSmaller2;
		m_FillGapStart1=dlg.m_FillGapStart1;
		m_FillGapStop1=dlg.m_FillGapStop1;
		m_FillGapStart2=dlg.m_FillGapStart2;
		m_FillGapStop2=dlg.m_FillGapStop2;
		m_MedianLevel=dlg.m_MedianLevel;
		m_StopAfterStep=dlg.m_StopAfterStep;
		GuidingMode=dlg.m_GuidingMode;
		m_MedianSize=dlg.m_MedianSize;
		m_FloodFillOutsideSize=dlg.m_FloodFillOutsideSize;
		m_ExaminedSurrounding=dlg.m_ExaminedSurrounding;
	}
}

void CGuiding::CalcNewBeamPath(CDataPointDoc *aDataPoint, CBeamGuiding *aBeamGuiding,long Timelaps)
{
	CTime Now=CTime::GetCurrentTime();
	CTimeSpan Delay=Now-LastGuidingTime;
	LastGuidingTime=Now;
	double DelaySec=Delay.GetTotalSeconds( );
	CBeamPath* Path=new CBeamPath(aBeamGuiding->BeamPath);
	switch (GuidingMode) {
		case 1: {			
			POSITION pos=Path->GetHeadPosition();	
			while (pos!=NULL) {
				CBeamStop* BeamStop=(CBeamStop*)Path->GetNext(pos);
				CPoint OldPos=BeamStop->Position;
				PrepareAdvanceBeamStop(aDataPoint, BeamStop,false);
				aDataPoint->ConvertCoordPixelToReal(InitT);
				InitT+=BeamStop->Speed*(DelaySec/600.0);
				aDataPoint->ConvertCoordRealToPixel(InitT);
				CPoint pos;
				bool error=GetPos(BeamStop,InitT,pos);   //t in pixel coord
				if (!error) {
					aDataPoint->ConvertCoordPixelToReal(pos);			
					BeamStop->Position=pos;										
					if (Spline) {  //correct direction
						BeamStop->SetLaserPositionOnSpline(CurvePoints,BeamStop->GetLaserPositionOnSpline(CurvePoints));
						BeamStop->Position=pos;	
					}										
				} else BeamStop->Position=OldPos;
			}		
			break;
		}
		case 2: {
			if (aDataPoint->m_DetectionRegion.IsRectNull())
				CalcDetectionImage(aDataPoint,aBeamGuiding);
			else
				CalcDetectionImage(aDataPoint,aDataPoint->m_DetectionRegion, aBeamGuiding);

			POSITION pos=Path->GetHeadPosition();	
			while (pos!=NULL) {
				CBeamStop* BeamStop=(CBeamStop*)Path->GetNext(pos);
				AdvanceBeamStop(aDataPoint,BeamStop,Timelaps);				
			}
			if (aDataPoint->ConeDetectionImage->EffectiveBitPerPixel==1) aDataPoint->ConeDetectionImage->ConvertTo1BitPerPixel();
			break;
		}
	} 
	if (aBeamGuiding) Path->CalculateLaserPosition(aBeamGuiding); 
	Path->Offset=CPoint::CPoint((aDataPoint->Param.XOrigin)*(aDataPoint->Param.Calibration.x),(aDataPoint->Param.YOrigin)*(aDataPoint->Param.Calibration.y));
	Path->Center=CPoint::CPoint((aDataPoint->Param.Width)*(aDataPoint->Param.Calibration.x)/2,(aDataPoint->Param.Height)*(aDataPoint->Param.Calibration.y)/2);
	Path->Binning=aDataPoint->Param.CellBinning;
	if (GuidingMode!=0) {
		aBeamGuiding->SetNewPath(Path);		
	} else if (aBeamGuiding->BeamPath) {
		aBeamGuiding->BeamPath->Offset=CPoint::CPoint((aDataPoint->Param.XOrigin)*(aDataPoint->Param.Calibration.x),(aDataPoint->Param.YOrigin)*(aDataPoint->Param.Calibration.y));
		aBeamGuiding->BeamPath->Center=CPoint::CPoint((aDataPoint->Param.Width)*(aDataPoint->Param.Calibration.x)/2,(aDataPoint->Param.Height)*(aDataPoint->Param.Calibration.y)/2);
		aBeamGuiding->BeamPath->Binning=aDataPoint->Param.CellBinning;		
	}
	if (aDataPoint->BeamPath!=NULL) delete aDataPoint->BeamPath;
	aDataPoint->BeamPath=new CBeamPath(Path);
	aDataPoint->BeamPath->BeamOn=aBeamGuiding->BeamOn;
	delete Path;
}

void CGuiding::StartStop(bool start)
{
	if (IsGuiding!=start) {
		IsGuiding=start;
		if (IsGuiding) {
			LastGuidingTime=CTime::GetCurrentTime();
		}
	}
}

void CGuiding::SerializeParams(CArchive &ar)
{
	if (ar.IsStoring())
	{	
		if (ExtractAxonArea) ar<<'Y'; else ar<<'N';
		if (ExtractCellArea) ar<<'Y'; else ar<<'N';		
		ar<<GuidingMode;
		ar<<m_FloodFillOutsideSize;
		ar<<m_MedianSize;
		ar<<m_BeamAdvance;
		ar<<m_MaxStepsize;
		ar<<m_CropBorders;
		ar<<m_EdgeFilterEnd;
		ar<<m_EdgeFilterStart;
		ar<<m_EraseSmaller1;
		ar<<m_EraseSmaller2;
		ar<<m_FillGapStart1;
		ar<<m_FillGapStart2;
		ar<<m_FillGapStop1;
		ar<<m_FillGapStop2;
		ar<<m_MedianLevel;
		ar<<m_StopAfterStep;
		ar<<m_ExaminedSurrounding;
	} else {
		char help;
		ar>>help;
		ExtractAxonArea=help=='Y';	
		ar>>help;
		ExtractCellArea=help=='Y';
		ar>>GuidingMode;
		ar>>m_FloodFillOutsideSize;
		ar>>m_MedianSize;
		ar>>m_BeamAdvance;
		ar>>m_MaxStepsize;
		ar>>m_CropBorders;
		ar>>m_EdgeFilterEnd;
		ar>>m_EdgeFilterStart;
		ar>>m_EraseSmaller1;
		ar>>m_EraseSmaller2;
		ar>>m_FillGapStart1;
		ar>>m_FillGapStart2;
		ar>>m_FillGapStop1;
		ar>>m_FillGapStop2;
		ar>>m_MedianLevel;
		ar>>m_StopAfterStep;
		ar>>m_ExaminedSurrounding;
	}
}

long D1Gradient2[4]={-1,0,0,1};
long D2Gradient2[4]={0,-1,1,0};
long XGradient2[4]={1,-1,0,0};
long YGradient2[4]={1,0,-1,0};
long D2Gradient3[9]={-2,0,0,
					  0,0,0,
					  0,0,2};
long D1Gradient3[9]={0,0,-2,
					 0,0,0,
					 2,0,0};
long XGradient3[9]={0,0,0,
					2,0,-2,
					0,0,0};
long YGradient3[9]={0,2,0,
				    0,0,0,
					0,-2,0};
long D1Gradient4[16]={-2,0,0,0,
					  0,-1,0,0,
					  0,0,1,0,
					  0,0,0,2};
long D2Gradient4[16]={0,0,0,2,
					 0,0,1,0,
					 0,-1,0,0,
					 -2,0,0,0};
long XGradient4[16]={0,0,0,0,
					-2,-1,1,2,
					0,0,0,0,
					0,0,0,0,};
long YGradient4[16]={0,2,0,0,
					0,1,0,0,
					0,-1,0,0,
					0,-2,0,0,};
long D1Gradient5[25]={-3,0,0,0,0,
					  0,-2,0,0,0,
					  0,0,0,0,0,
					  0,0,0,2,0,
					  0,0,0,0,3};
long D2Gradient5[25]={0,0,0,0,-3,
					  0,0,0,-2,0,
					  0,0,0,0,0,
					  0,2,0,0,0,
					  3,0,0,0,0};
long XGradient5[25]={0,0,0,0,0,
					  0,0,0,0,0,
					  3,2,0,-2,-3,
					  0,0,0,0,0,
					  0,0,0,0,0};
long YGradient5[25]={0,0,3,0,0,
					  0,0,2,0,0,
					  0,0,0,0,0,
					  0,0,-2,0,0,
					  0,0,-3,0,0};

void CGuiding::CalcDetectionImage(CDataPointDoc *aDataPoint, CBeamGuiding *aBeamGuiding)
{
	if (!aDataPoint) return;
	if (m_StopAfterStep<=0) return;
	const unsigned char CalcBitPerPixel=8;
	if (aDataPoint->ConeDetectionImage!=NULL) delete aDataPoint->ConeDetectionImage;
	aDataPoint->ConeDetectionImage=new CImage(aDataPoint->CellImage->Width,aDataPoint->CellImage->Height,1,(m_StopAfterStep<=1) ? 16 : CalcBitPerPixel);  //init and 1bit or 16bit image		
	long j=0;
	double jd=0;
	double Median=0;
	CImage* help=NULL;

	bool UseSubRegion=true;
	CPoint center;
	POSITION pos=NULL;
	CImage* PartialDetection=NULL;
	if ((m_ExaminedSurrounding>0) && (aBeamGuiding!=NULL)) pos=aBeamGuiding->BeamPath->GetHeadPosition();				
	while (UseSubRegion) {
		long x0,y0,x1,y1;		
		if ((m_ExaminedSurrounding>0) && (aBeamGuiding)) {						
			if (pos!=NULL) {				
				CBeamStop* BeamStop=(CBeamStop*)aBeamGuiding->BeamPath->GetNext(pos);
				CPoint Center=BeamStop->Position;
				aDataPoint->ConvertCoordRealToPixel(Center);
				x0=Center.x-m_ExaminedSurrounding;
				y0=Center.y-m_ExaminedSurrounding;
				x1=Center.x+m_ExaminedSurrounding;
				y1=Center.y+m_ExaminedSurrounding;				
				if (CalcBitPerPixel==1) {
					//x1&=0x1f;  //guarantees bitalignement for easier copying
					//x0&=0x1f;
					x1 = (x1 % 32) * 32;
					x0 = (x0 % 32) * 32;
				}
				if (x0<0) x0=0;
				if (x1<0) x1=0;
				if (y0<0) y0=0;
				if (y1<0) y1=0;
				if (x0>aDataPoint->ConeDetectionImage->Width) x0=aDataPoint->ConeDetectionImage->Width;
				if (x1>aDataPoint->ConeDetectionImage->Width) x1=aDataPoint->ConeDetectionImage->Width;
				if (y0>aDataPoint->ConeDetectionImage->Height) y0=aDataPoint->ConeDetectionImage->Height;
				if (y1>aDataPoint->ConeDetectionImage->Height) y1=aDataPoint->ConeDetectionImage->Height;				
				UseSubRegion=true;
			} else goto CleanUpAndReturn;						
		} else {
			x0=0;
			y0=0;
			UseSubRegion=false;			
		}
	
		CImage* SubRegion;
		if (UseSubRegion) {
			SubRegion=new CImage(x1-x0,y1-y0,0,16); //no init 16 bit
			SubRegion->CopyRegion(x0,y0,x1,y1,aDataPoint->CellImage);
		} else SubRegion=aDataPoint->CellImage;

		CImage* sum=new CImage(SubRegion->Width,SubRegion->Height,1,16);
		if (help) delete help;
		help=new CImage(SubRegion->Width,SubRegion->Height,0,16);	
		if (m_EdgeFilterStart>m_EdgeFilterEnd) {
			long help=m_EdgeFilterStart;
			m_EdgeFilterStart=m_EdgeFilterEnd;
			m_EdgeFilterEnd=help;
		}
		unsigned short NrFilters=m_EdgeFilterEnd-m_EdgeFilterStart+1;
	
		if ((2>=m_EdgeFilterStart) && (2<=m_EdgeFilterEnd)) {
			SubRegion->Filter(2,D1Gradient2,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(2,D2Gradient2,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(2,XGradient2,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(2,YGradient2,1*NrFilters,help);
			sum->AddImage(help);
		}

		if ((3>=m_EdgeFilterStart) && (3<=m_EdgeFilterEnd)) {
			SubRegion->Filter(3,D1Gradient3,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(3,D2Gradient3,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(3,XGradient3,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(3,YGradient3,1*NrFilters,help);
			sum->AddImage(help);
		}
	
		if ((4>=m_EdgeFilterStart) && (4<=m_EdgeFilterEnd)) {
			SubRegion->Filter(4,D1Gradient4,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(4,D2Gradient4,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(4,XGradient4,1*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(4,YGradient4,1*NrFilters,help);
			sum->AddImage(help);
		}
	
		if ((5>=m_EdgeFilterStart) && (5<=m_EdgeFilterEnd)) {
			SubRegion->Filter(5,D1Gradient5,2*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(5,D2Gradient5,2*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(5,XGradient5,2*NrFilters,help);
			sum->AddImage(help);
			SubRegion->Filter(5,YGradient5,2*NrFilters,help);
			sum->AddImage(help);
		}
		delete help;
		help=NULL;
		if (UseSubRegion) delete SubRegion;

		if (m_StopAfterStep<=1) {			
			if (PartialDetection) delete PartialDetection;
			PartialDetection=new CImage(sum->Width,sum->Height,0,16);  //no init 16bit
			PartialDetection->Copy(sum);		
			goto EndOfLoop; 
		}

		Median=sum->CalcMedian();
		jd=m_MedianLevel;
		jd/=100;
		Median*=jd;
		if (PartialDetection) delete PartialDetection;
		PartialDetection=new CImage(sum->Width,sum->Height,0,CalcBitPerPixel);  //no init 1bit
		PartialDetection->CopyMedian(m_MedianSize,Median,sum);	
		delete sum;
		sum=NULL;	

		if (m_StopAfterStep<=2) goto EndOfLoop;	

		PartialDetection->CropBorders(m_CropBorders,0);
	
		if (m_StopAfterStep<=3) goto EndOfLoop;	
	
		if ((m_FillGapStart1!=0)|| (m_FillGapStop1!=0)) {
			if (m_FillGapStart1<m_FillGapStop1) {
				j=m_FillGapStart1;
				m_FillGapStart1=m_FillGapStop1;
				m_FillGapStop1=j;
			}
			for (int k=m_FillGapStart1;k>=m_FillGapStop1;k--) PartialDetection->FillGaps(k);
		}
	
		if (m_StopAfterStep<=4) goto EndOfLoop;	

		PartialDetection->EraseSmallerThan(m_EraseSmaller1);
		
		if (m_StopAfterStep<=5) goto EndOfLoop;	
	
		if ((m_FillGapStart2!=0)|| (m_FillGapStop2!=0)) {
			if (m_FillGapStart2<m_FillGapStop2) {
				j=m_FillGapStart2;
				m_FillGapStart2=m_FillGapStop2;
				m_FillGapStop2=j;
			}
			for (int k=m_FillGapStart2;k>=m_FillGapStop2;k--) PartialDetection->FillGaps(k);
		}

		if (m_StopAfterStep<=6) goto EndOfLoop;		

		PartialDetection->EraseSmallerThan(m_EraseSmaller2);	
	
		if (m_StopAfterStep<=7) goto EndOfLoop;				
		
		PartialDetection->KeepOutside(m_FloodFillOutsideSize);				

EndOfLoop:
		if (UseSubRegion) {
			aDataPoint->ConeDetectionImage->CopyOrAt(x0,y0,PartialDetection);
			delete PartialDetection;
			PartialDetection=NULL;
		} else {
			if (aDataPoint->ConeDetectionImage) delete aDataPoint->ConeDetectionImage;
			aDataPoint->ConeDetectionImage=PartialDetection;
			PartialDetection=NULL;
		}
	}  //while loop

CleanUpAndReturn:	
	if (m_StopAfterStep>=2) aDataPoint->ConeDetectionImage->EffectiveBitPerPixel=1;
	if (m_StopAfterStep>7) aDataPoint->ConeDetectionImage->FillBorder(m_CropBorders);
	//aDataPoint->ConeDetectionImage->ConvertTo8BitPerPixel();	
	if (PartialDetection) delete PartialDetection;
}

void CGuiding::CalcDetectionImage(CDataPointDoc *aDataPoint, CRect region, CBeamGuiding *aBeamGuiding)
{
	if (!aDataPoint) return;
	if (m_StopAfterStep<=0) return;
	const unsigned char CalcBitPerPixel=8;
	if (aDataPoint->ConeDetectionImage!=NULL) delete aDataPoint->ConeDetectionImage;
	aDataPoint->ConeDetectionImage=new CImage(aDataPoint->CellImage->Width,aDataPoint->CellImage->Height,0,(m_StopAfterStep<=1) ? 16 : CalcBitPerPixel);  //init and 1bit or 16bit image		
	aDataPoint->ConeDetectionImage->ClearWhite();
	long j=0;
	double jd=0;
	double Median=0;
	CImage* help=NULL;

	
	bool UseSubRegion=true;
	CImage* PartialDetection=NULL;
	
	int x0 = region.left;
	int y0 = region.top;
	int x1 = region.right;
	int y1 = region.bottom;

	CImage* SubRegion;
	if (UseSubRegion) {
		SubRegion=new CImage(x1-x0,y1-y0,0,16); //no init 16 bit
		SubRegion->CopyRegion(x0,y0,x1,y1,aDataPoint->CellImage);
	} else SubRegion=aDataPoint->CellImage;

	CImage* sum=new CImage(SubRegion->Width,SubRegion->Height,1,16);
	if (help) delete help;
	help=new CImage(SubRegion->Width,SubRegion->Height,0,16);	
	if (m_EdgeFilterStart>m_EdgeFilterEnd) {
		long help=m_EdgeFilterStart;
		m_EdgeFilterStart=m_EdgeFilterEnd;
		m_EdgeFilterEnd=help;
	}
	unsigned short NrFilters=m_EdgeFilterEnd-m_EdgeFilterStart+1;

	if ((2>=m_EdgeFilterStart) && (2<=m_EdgeFilterEnd)) {
		SubRegion->Filter(2,D1Gradient2,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(2,D2Gradient2,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(2,XGradient2,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(2,YGradient2,1*NrFilters,help);
		sum->AddImage(help);
	}

	if ((3>=m_EdgeFilterStart) && (3<=m_EdgeFilterEnd)) {
		SubRegion->Filter(3,D1Gradient3,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(3,D2Gradient3,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(3,XGradient3,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(3,YGradient3,1*NrFilters,help);
		sum->AddImage(help);
	}

	if ((4>=m_EdgeFilterStart) && (4<=m_EdgeFilterEnd)) {
		SubRegion->Filter(4,D1Gradient4,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(4,D2Gradient4,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(4,XGradient4,1*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(4,YGradient4,1*NrFilters,help);
		sum->AddImage(help);
	}

	if ((5>=m_EdgeFilterStart) && (5<=m_EdgeFilterEnd)) {
		SubRegion->Filter(5,D1Gradient5,2*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(5,D2Gradient5,2*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(5,XGradient5,2*NrFilters,help);
		sum->AddImage(help);
		SubRegion->Filter(5,YGradient5,2*NrFilters,help);
		sum->AddImage(help);
	}
	delete help;
	help=NULL;
	if (UseSubRegion) delete SubRegion;

	if (m_StopAfterStep<=1) {			
		if (PartialDetection) delete PartialDetection;
		PartialDetection=new CImage(sum->Width,sum->Height,0,16);  //no init 16bit
		PartialDetection->Copy(sum);		
		goto EndOfLoop; 
	}

	Median=sum->CalcMedian();
	jd=m_MedianLevel;
	jd/=100;
	Median*=jd;
	if (PartialDetection) delete PartialDetection;
	PartialDetection=new CImage(sum->Width,sum->Height,0,CalcBitPerPixel);  //no init 1bit
	PartialDetection->CopyMedian(m_MedianSize,Median,sum);	
	delete sum;
	sum=NULL;	

	if (m_StopAfterStep<=2) goto EndOfLoop;	

	PartialDetection->CropBorders(m_CropBorders,0);

	if (m_StopAfterStep<=3) goto EndOfLoop;	

	if ((m_FillGapStart1!=0)|| (m_FillGapStop1!=0)) {
		if (m_FillGapStart1<m_FillGapStop1) {
			j=m_FillGapStart1;
			m_FillGapStart1=m_FillGapStop1;
			m_FillGapStop1=j;
		}
		for (int k=m_FillGapStart1;k>=m_FillGapStop1;k--) PartialDetection->FillGaps(k);
	}

	if (m_StopAfterStep<=4) goto EndOfLoop;	

	PartialDetection->EraseSmallerThan(m_EraseSmaller1);
	
	if (m_StopAfterStep<=5) goto EndOfLoop;	

	if ((m_FillGapStart2!=0)|| (m_FillGapStop2!=0)) {
		if (m_FillGapStart2<m_FillGapStop2) {
			j=m_FillGapStart2;
			m_FillGapStart2=m_FillGapStop2;
			m_FillGapStop2=j;
		}
		for (int k=m_FillGapStart2;k>=m_FillGapStop2;k--) PartialDetection->FillGaps(k);
	}

	if (m_StopAfterStep<=6) goto EndOfLoop;		

	PartialDetection->EraseSmallerThan(m_EraseSmaller2);	

	if (m_StopAfterStep<=7) goto EndOfLoop;				
	
	PartialDetection->KeepOutside(m_FloodFillOutsideSize);				

EndOfLoop:
	if (UseSubRegion) {
		//aDataPoint->ConeDetectionImage->CopyOrAt(x0,y0,PartialDetection);
		aDataPoint->ConeDetectionImage->CopyAt(x0,y0,PartialDetection);
		delete PartialDetection;
		PartialDetection=NULL;
	} else {
		if (aDataPoint->ConeDetectionImage) delete aDataPoint->ConeDetectionImage;
		aDataPoint->ConeDetectionImage=PartialDetection;
		PartialDetection=NULL;
	}


CleanUpAndReturn:	
	if (m_StopAfterStep>=2) aDataPoint->ConeDetectionImage->EffectiveBitPerPixel=1;
	if (m_StopAfterStep>7) aDataPoint->ConeDetectionImage->FillBorder(m_CropBorders);
	//aDataPoint->ConeDetectionImage->ConvertTo8BitPerPixel();	
	if (PartialDetection) delete PartialDetection;
}

bool CGuiding::GetPos(CBeamStop *BeamStop,double t,CPoint &pos) {   //t in pixel coord
	if (Spline) {
		if (t<0) return true;  //endreached
		if (t>SplineLength) return true;
		int i;
		pos=BeamStop->GetIntermediateSplinePos(CurvePoints,t,i);	
	} else {
		pos.x=xp+t*xd;
		pos.y=yp+t*yd;		
	}
	return (!((pos.x>0) && (pos.x<width) && (pos.y>0) && (pos.y<width)));
}

void CGuiding::PrepareAdvanceBeamStop(CDataPointDoc *aDataPoint, CBeamStop *BeamStop, bool Substractm_BeamAdvance)
{		
	height=aDataPoint->CellImage->Height;
	width=aDataPoint->CellImage->Width;	
	if (BeamStop->SplineControlPoints.GetSize()>1) {		
		BeamStop->GetSpline(CurvePoints);  //in pixel coord
		InitT=BeamStop->GetLaserPositionOnSpline(CurvePoints); //in 0..1
		SplineLength=BeamStop->GetSplineLength(CurvePoints); //in real coords
		InitT*=SplineLength;  //in real coords		
		if (Substractm_BeamAdvance) InitT-=m_BeamAdvance;   //in real coords
		//convert to pixel coord
		aDataPoint->ConvertCoordRealToPixel(InitT);  
		SplineLength=BeamStop->GetSplineLength(CurvePoints,-1,false); //in pixel coords
		Spline=true;
	} else {
		Spline=false;
		CPoint pos=BeamStop->Position;	
		CPoint dir=BeamStop->Direction;
		double l;
		xd=dir.x;
		yd=dir.y;
		l=sqrt(xd*xd+yd*yd);
		xd/=l;
		yd/=l;
		CPoint helpdir;
		helpdir.x=xd/m_BeamAdvance;
		helpdir.y=yd/m_BeamAdvance;
		if (Substractm_BeamAdvance) pos-=helpdir;
		aDataPoint->ConvertCoordRealToPixel(pos);
		xp=pos.x;
		yp=pos.y;		
		InitT=0;
	}		
}

void CGuiding::AdvanceBeamStop(CDataPointDoc *aDataPoint, CBeamStop *BeamStop,long Timelaps)
{
	if (BeamStop->FrontierPoints) delete BeamStop->FrontierPoints;
	BeamStop->FrontierPoints=new CArray<POINT, POINT>;						
			
	CPoint OldPos=BeamStop->Position;
	PrepareAdvanceBeamStop(aDataPoint, BeamStop,true);
	CPoint pos;
	CPoint CellPos;
	bool error=GetPos(BeamStop,InitT,pos);
	if ((pos.x<0) || (pos.x>width) || (pos.y<0) || (pos.y>width) || error) return;	
	unsigned short initcolor;
	aDataPoint->ConeDetectionImage->GetPixel(pos.x,pos.y,initcolor);
	if (initcolor==0) CellPos=pos; //we are inside the cell
	double t=InitT;
	bool found=false;
	bool ende=GetPos(BeamStop,t,pos);
	CPoint ForwardPos,BackwardPos,FoundPos;
	while ((!ende) && (!found)) {
		t+=0.1;
		ende=GetPos(BeamStop,t,pos);		
		if ((pos.x>=aDataPoint->ConeDetectionImage->Width) || (pos.y>=aDataPoint->ConeDetectionImage->Height) ||
			(pos.x<0) || (pos.y<0)) ende=true;
		if (!ende) {
			unsigned short aktcolor;
			aDataPoint->ConeDetectionImage->GetPixel(pos.x,pos.y,aktcolor);
			found=initcolor!=aktcolor;
	//		BeamStop->FrontierPoints->Add(pos);	//for debug
			if (found) FoundPos=ForwardPos=pos;
		}		
	}
	double foundforward=found;
	found=false;
	double tr=InitT;
	ende=GetPos(BeamStop,tr,pos);
	while ((!ende) && (!found)) {
		tr-=0.1;
		ende=GetPos(BeamStop,tr,pos);		
		if ((pos.x>=aDataPoint->ConeDetectionImage->Width) || (pos.y>=aDataPoint->ConeDetectionImage->Height) ||
			(pos.x<0) || (pos.y<0)) ende=true;
		if (!ende) {
			unsigned short aktcolor;
			aDataPoint->ConeDetectionImage->GetPixel(pos.x,pos.y,aktcolor);
			found=initcolor!=aktcolor;
	//		BeamStop->FrontierPoints->Add(pos);	//for debug
			if (found) FoundPos=BackwardPos=pos;
		}		
	}
	BeamStop->Position=OldPos;
	if (foundforward && found) {
		if (fabs(t-InitT)>fabs(tr-InitT)) {
			t=tr;
			FoundPos=BackwardPos;
		} else FoundPos=ForwardPos;
	}
	if ((!foundforward) && found) {
		t=tr;
		FoundPos=BackwardPos;
	}

	if (foundforward || found) {
		BeamStop->NormalDirection=BeamStop->GetDirection(CurvePoints,t,false); //t in pixel coords	
		double l=BeamStop->NormalDirection.x*BeamStop->NormalDirection.x+BeamStop->NormalDirection.y*BeamStop->NormalDirection.y;
		if (l>0) l=sqrt(l); else l=0.0001;				
		double dl=1000/l;  //length normalized to 1000
		BeamStop->NormalDirection.x*=dl;
		BeamStop->NormalDirection.y*=dl;
	}

	double dt=fabs(t-InitT);		
	double g=m_MaxStepsize;
	aDataPoint->ConvertCoordRealToPixel(g);
	if (dt<g)  {		//Maximum movement in pixel
		if (foundforward || found) {
			double ha=m_BeamAdvance;
			aDataPoint->ConvertCoordRealToPixel(ha);  						
			double th=t+ha;			
			GetPos(BeamStop,th,pos);
			//pos=FoundPos;
			aDataPoint->ConvertCoordPixelToReal(pos);			
			CPoint OldPos=BeamStop->Position;
			BeamStop->Position=pos;			
			CPoint Dir=pos-OldPos;
			double l=Dir.x*Dir.x+Dir.y*Dir.y;
			if (l>0) l=sqrt(l); else l=0.0001;			
			double tl=Timelaps;
			double dl=(600.0/tl)/l;  //length in ten minutes
			Dir.x*=dl;
			Dir.y*=dl;
			BeamStop->Direction=Dir;
		} 
	} else {
		char buf[200];
		aDataPoint->ConvertCoordPixelToReal(dt);
		double hdt=dt;
		hdt/=1000;
		double max=m_MaxStepsize;
		max/=1000;		
		sprintf(buf,"Advance %.1f bigger than allowed (%.1f)",hdt,max);
		aDataPoint->AddMessage(buf);
	}
	if (foundforward || found) {
		double th=t;
		aDataPoint->ConvertCoordPixelToReal(th);
		BeamStop->NeuronPositionOnSpline=th;
		unsigned short endcolor;
		aDataPoint->ConeDetectionImage->GetPixel(FoundPos.x,FoundPos.y,endcolor);
		if (endcolor==0) CellPos=FoundPos; //we are inside the cell now, if not we were before and CellPos is already initialized		
		if (BeamStop->Range>0) {			
			CImage *help=new CImage(aDataPoint->ConeDetectionImage);					
		//	BeamStop->FrontierPoints->Add(FoundPos);	//for debug
			//have range/2 to left and right of spline, fit 50% more to smoothen ends
			double fitrange=0.75*BeamStop->Range;
			aDataPoint->ConvertCoordRealToPixel(fitrange);
			unsigned long count;
			CPoint Left,Right;
			help->GetFrontier(FoundPos,Left,Right,fitrange,BeamStop->FrontierPoints,count);
			delete help;						
			if (count>0) {
				if (BeamStop->ScanCurve) delete BeamStop->ScanCurve;
				BeamStop->ScanCurve=NULL;
				double Range;
				BeamStop->ScanCurve=new CSmoothCurve(FoundPos,Left,Right,BeamStop->FrontierPoints,BeamStop->PolynomDegree,Range);

				//calculate vector orthogonal scancurve at midpoint in direction of propagation normalized to 1000
				CPoint Normal=BeamStop->ScanCurve->GetNormal(0); //t in pixel coords // normalized to 1000
				double help=Normal.x*BeamStop->NormalDirection.x+Normal.y*BeamStop->NormalDirection.y; //scalar product
				if (help<0) { //Normal in opposite direction than NormalDirection
					Normal.x*=-1;
					Normal.y*=-1;
				}
				BeamStop->NormalDirection.x=Normal.x;
				BeamStop->NormalDirection.y=Normal.y;

				/*if (BeamStop->FrontierPoints->GetSize()>0) {  //debug
					ofstream out("c:\\Neuron\\data\\frontier.dat");
					for (int l=0;l<BeamStop->FrontierPoints->GetSize();l++) {
						out<<(*BeamStop->FrontierPoints)[l].x<<" "<<(*BeamStop->FrontierPoints)[l].y<<endl;
					}
				}*/
			}
		}
		if (ExtractCellArea) {			
			//determine cellarea
			CImage* help=new CImage(aDataPoint->ConeDetectionImage);
			unsigned long count;
			help->FloodFill(CellPos.x,CellPos.y,65535,count);
			BeamStop->CellArea=count;
			aDataPoint->ConvertCoordPixelToReal(BeamStop->CellArea);//twice since it is an area
			aDataPoint->ConvertCoordPixelToReal(BeamStop->CellArea);
			delete help;
		}
		if (BeamStop->SplineControlPoints.GetSize()>1) {
			if (ExtractAxonArea) {			
				//determine axonarea
				CImage* help=new CImage(aDataPoint->ConeDetectionImage);
				//Draw line to border
				GetPos(BeamStop,0,pos);
				CPoint SplineDirection=BeamStop->GetDirection(CurvePoints,0,false);
				CPoint Direction;
				Direction.x=-SplineDirection.y;
				Direction.y=SplineDirection.x;				
				help->DrawLineTillBorder(pos,Direction,BeamStop->AxonStartLeft,BeamStop->AxonStartPos,BeamStop->AxonStartRight);
				if ((BeamStop->AxonStartPos.x!=0) || (BeamStop->AxonStartPos.y!=0)) {					
					aDataPoint->ConvertCoordPixelToReal(BeamStop->AxonStartPos);
					aDataPoint->ConvertCoordPixelToReal(BeamStop->AxonStartLeft);
					aDataPoint->ConvertCoordPixelToReal(BeamStop->AxonStartRight);
					BeamStop->SplineControlPoints[0]=BeamStop->AxonStartPos;
				}				
				unsigned long count;
				help->FloodFill(CellPos.x,CellPos.y,65535,count);
				BeamStop->AxonArea=count;
				aDataPoint->ConvertCoordPixelToReal(BeamStop->AxonArea);//twice since it is an area
				aDataPoint->ConvertCoordPixelToReal(BeamStop->AxonArea);
				delete help;
			}
	/*		 help=new CImage(aDataPoint->ConeDetectionImage);  //for debug
			//Draw line to border
			GetPos(BeamStop,0,pos);
			SplineDirection=BeamStop->GetDirection(CurvePoints,0,false);
			Direction;
			Direction.x=-SplineDirection.y;
			Direction.y=SplineDirection.x;
			help->DrawLineTillBorder(pos,Direction);							
			delete  aDataPoint->ConeDetectionImage;
			aDataPoint->ConeDetectionImage=help;*/
		}
	}
}
