// Neuron.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "Neuron.h"
#include <direct.h>
#include <stdio.h>
#include <io.h>
#include "MainFrm.h"
#include "ChildFrm.h"
#include "RunPreview.h"
#include "RunView.h"
#include "RunDlg.h"
#include "RunForm.h"
#include "DataPointDoc.h"
#include "DataPointView.h"
#include "Acquisition.h"
#include "Beam.h"
#include "Microscope.h"
#include "Guiding.h"
#include "NeuronAppDlg.h"
#include "ReferenceImageDlg.h"
#include "hint.h"
#include "runformframe.h"
#include "MyEditView.h"
#include "pictureexportdlg.h"
#include "ExtractDataDlg.h"
#include <fstream.h>

//{{AFX_INCLUDES()
#include "_idevice.h"
//}}AFX_INCLUDES

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: HANDLE CreateOneAppMutex(LPCTSTR lpName)
//
//  PURPOSE: Creates a mutex only for one application
//
//  COMMENTS:
//
//    This function tries to create a mutex, but if specified mutex exits
//    this function returns NULL.
//
HANDLE CreateOneAppMutex(LPCTSTR lpName)
{
    HANDLE hMutex;

    hMutex = CreateMutex(NULL, TRUE, lpName);   // Create mutex

    switch(GetLastError())
    {
    case ERROR_SUCCESS:
        //
        // Mutex created successfully. There is no instances running
        //
        break;

    case ERROR_ALREADY_EXISTS:
        //
        // Mutex already exists so there is a running instance of our app.
        //
        hMutex = NULL;
        break;

    default:
        //
        // Failed to create mutex by unkniown reason
        //
        break;
    }

    return hMutex;
}

//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage is only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
/*
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDR_MAINFRAME);
	wcex.hCursor		= LoadCursor(NULL, IDC_NEURON);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR)IDC_ONLYONE;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_NEURON_SMALL);

	return RegisterClassEx(&wcex);
} */

// CNeuronApp

BEGIN_MESSAGE_MAP(CNeuronApp, CWinApp)
	//{{AFX_MSG_MAP(CNeuronApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_ACQUISITIONDIALOG, OnAcquisitiondialog)
	ON_COMMAND(ID_PLAY, OnPlay)
	ON_UPDATE_COMMAND_UI(ID_PLAY, OnUpdatePlay)
	ON_COMMAND(ID_LASER_ON_OFF, OnLaserOnOff)
	ON_UPDATE_COMMAND_UI(ID_LASER_ON_OFF, OnUpdateLaserOnOff)
	ON_COMMAND(ID_SAVE, OnSave)
	ON_UPDATE_COMMAND_UI(ID_SAVE, OnUpdateSave)
	ON_MESSAGE(IDM_GET_NEURON_APP, OnGetNeuronApp)	
	ON_COMMAND(ID_MICROSCOPE_LIGHT_ON_OFF, OnMicroscopeLightOnOff)
	ON_UPDATE_COMMAND_UI(ID_MICROSCOPE_LIGHT_ON_OFF, OnUpdateMicroscopeLightOnOff)
	ON_COMMAND(ID_GUIDING_DIALOG, OnGuidingDialog)
	ON_COMMAND(ID_CONFIG_GENERAL_PROPERTIES, OnConfigGeneralProperties)	
	ON_COMMAND(IDC_REFERENCE_IMAGE_DLG, OnReferenceImageDlg)		
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_UPDATE_COMMAND_UI(ID_ZOOM_IN, OnUpdateZoomIn)	
	ON_UPDATE_COMMAND_UI(ID_ZOOM_OUT, OnUpdateZoomOut)
	ON_COMMAND(ID_ZOOM_OUT, OnZoomOut)
	ON_COMMAND(ID_ZOOM_IN, OnZoomIn)
	ON_COMMAND(ID_TAKEONEPICTURE, OnTakeonepicture)
	ON_UPDATE_COMMAND_UI(ID_TAKEONEPICTURE, OnUpdateTakeonepicture)
	ON_COMMAND(IDC_DATAEXTRACTION_DLG, OnDataextractionDlg)
	ON_COMMAND(IDC_BEAMSTOPTEMPLATE, OnBeamstoptemplate)
	ON_COMMAND(IDC_BEAMGUIDINGDLG, OnBeamguidingdlg)
	ON_COMMAND(ID_CALIBRATION, OnCalibration)
	ON_UPDATE_COMMAND_UI(ID_SELECT_REGION, OnUpdateSelectRegion)
	ON_COMMAND(ID_FAKE_RUN, OnFakeRun)
	ON_UPDATE_COMMAND_UI(ID_FAKE_RUN, OnUpdateFakeRun)
	ON_UPDATE_COMMAND_UI(ID_CELL_IMAGE_VISIBLE, OnUpdateCellImageVisible)
	ON_UPDATE_COMMAND_UI(ID_LASER_PATH_VISIBLE, OnUpdateLaserPathVisible)
	ON_UPDATE_COMMAND_UI(ID_LASER_IMAGE_VISIBLE, OnUpdateLaserImageVisible)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// The one and only CNeuronApp object

CNeuronApp theApp;
#define MAX_LOADSTRING 100
TCHAR szMutexName[MAX_LOADSTRING];              // Mutex name

/////////////////////////////////////////////////////////////////////////////
// CNeuronApp construction

CNeuronApp::CNeuronApp()
{	
	Mutex=NULL;
	//ParamFileName="c:\\Neuron\\Params.dat";
	char dir[100];
	_getcwd(dir, 100);
	ParamFileName = dir;
	ParamFileName += "\\Params.dat";
	PreviewBinning=10;
	Acquisition=NULL;
	BeamGuiding=NULL;
	ActiveRunDocAssigned=false;
	RootDirectory="c:\\Neuron\\";
	DataDirectory="Data\\";
	Saving=true;
	Microscope=NULL;
	Acquisition=NULL;
	BeamGuiding=NULL;
	Guiding=new CGuiding();
	ImageCacheSize=1;
	LaserSize.x=1000;
	LaserSize.y=1000;	
	ReferenceImageFixNr=1;
	ReferenceImageRelativeDistance=10;
	ReferenceImageMode=0;
	OnFileNewCalled=false;
	OnlyOnePicture=false;	
	UserAllowsHardwareAccess=true;
	PreviewPlaybackSpeed=10;
	AutoSaveIntervallMinutes=10;
	Fakerun = false;
}

CNeuronApp::~CNeuronApp() {
	SaveParams(ParamFileName);
	delete Acquisition;
	delete BeamGuiding;
	delete Microscope;
	delete Guiding;
	CoUninitialize();
	if (Mutex) ReleaseMutex(Mutex);
}

/////////////////////////////////////////////////////////////////////////////
// CNeuronApp initialization

BOOL CNeuronApp::InitInstance()
{	
	//MyRegisterClass(m_hInstance);
	LoadString(m_hInstance, IDC_MUTEX, szMutexName, MAX_LOADSTRING);
	Mutex=CreateOneAppMutex(szMutexName);
	if (Mutex==NULL) {
		//another copy of Neuron is already running
		//->disable hardware access
		AfxMessageBox("CNeuronApp::InitInstance: Another instance of Neuron is running. Disabling hardware access. ");
	}	
	AfxEnableControlContainer();
	if (CoInitialize(NULL)/*,COINIT_APARTMENTTHREADED)*/!=S_OK) {
		AfxMessageBox("CNeuronApp::InitInstance: COM library initialization failed");
	};
	Microscope=new CMicroscope(); 
	Acquisition=new CAcquisition(this,Microscope);
	BeamGuiding=new CBeamGuiding(this);		
	{	// BLOCK: doc template registration
		// Register the document template.  Document templates serve
		// as the connection between documents, frame windows and views.
		// Attach this form to another document or frame window by changing
		// the document or frame class in the constructor below.
		RunDocTemplate = new CMultiDocTemplate(			
			IDR_RUNFORM_TMPL,
			RUNTIME_CLASS(CRunDoc),		// document class
			RUNTIME_CLASS(CRunFormFrame),		// frame class
			RUNTIME_CLASS(CRunForm));		// view class
		AddDocTemplate(RunDocTemplate);
		
		RunDocCommentViewTemplate=new CMultiDocTemplate(IDR_TEXTTYPE,
		RUNTIME_CLASS(CRunDoc), RUNTIME_CLASS(CMDIChildWnd),
		RUNTIME_CLASS(CMyEditView));
		AddDocTemplate(RunDocCommentViewTemplate);

	}

	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

    /*CRunDlg dlg;
	m_pMainWnd = &dlg;
	int nResponse = dlg.DoModal(); */

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(_T("BONG"));
	LoadIcon(IDR_MAINFRAME );
	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

/*	CMultiDocTemplate* pRunDocTemplate;
	pRunDocTemplate = new CMultiDocTemplate(
		IDR_RUNTYPE,
		RUNTIME_CLASS(CRunDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CRunView));
	AddDocTemplate(pRunDocTemplate); */
	
	DataPointDocTemplate = new CMultiDocTemplate(
		IDR_DATAPOINT_TYPE,
		RUNTIME_CLASS(CDataPointDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CDataPointView));
	AddDocTemplate(DataPointDocTemplate);	

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE); 

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// The main window has been OnFileNewCalled, so show and update it.
	pMainFrame->ShowWindow(SW_SHOWMAXIMIZED);
	pMainFrame->UpdateWindow();
	TemperatureController.AttachControlSite(pMainFrame);
	LoadParams(ParamFileName);
	Acquisition->InitialUpdate((this->UserAllowsHardwareAccess) && (Mutex!=NULL));
	Microscope->InitialUpdate((this->UserAllowsHardwareAccess) && (Mutex!=NULL));
	BeamGuiding->InitialUpdate((this->UserAllowsHardwareAccess) && (Mutex!=NULL));
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CNeuronApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CNeuronApp message handlers

#include "RunForm.h"

void CNeuronApp::OnAcquisitiondialog() 
{
   if (Acquisition)	Acquisition->ConfigDlg();
}

void CNeuronApp::OnPlay() 
{
//	TemperatureController.ShowAboutBox();//PropertyPages();
	if (Acquisition) {
		Acquisition->Play();
		Guiding->StartStop(Acquisition->IsPlaying());
	}
}

void CNeuronApp::OnUpdatePlay(CCmdUI* pCmdUI) 
{
	if (Acquisition!=NULL) {
		pCmdUI->SetCheck( Acquisition->IsPlaying() && (!OnlyOnePicture));		
		pCmdUI->Enable( true );
	} else pCmdUI->Enable( true );
}

void CNeuronApp::OnSave() 
{
	Saving=!Saving;
	CRunDoc* ActiveRunDoc=GetActiveRunDoc();
	if (ActiveRunDoc && Saving) ActiveRunDoc->SaveActivated();
}

void CNeuronApp::OnUpdateSave(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( Saving );
	pCmdUI->Enable( true );
}

void CNeuronApp::OnLaserOnOff() 
{
	if (BeamGuiding) BeamGuiding->ToggleLaserOnOff();
}

void CNeuronApp::OnUpdateLaserOnOff(CCmdUI* pCmdUI) 
{
	if (BeamGuiding) {
		pCmdUI->SetCheck( BeamGuiding->IsLaserOn() );
		pCmdUI->Enable( true );
	} else pCmdUI->Enable( false );
}

BOOL CNeuronApp::OnIdle(LONG lCount) 
{	
	if (Acquisition) Acquisition->OnIdle();
	bool found=false;
	CRunDoc* ActRunDoc=NULL;
	POSITION pos=RunDocTemplate->GetFirstDocPosition();	

	while (pos!=NULL) {
		ActRunDoc=(CRunDoc*)RunDocTemplate->GetNextDoc(pos);		
		if (ActRunDoc) {
			ActRunDoc->OnIdle();
			CView* AktForm;
			POSITION ViewPos=ActRunDoc->GetFirstViewPosition( );
			while (ViewPos!=NULL) {
				AktForm=(CView*)ActRunDoc->GetNextView(ViewPos);
				if ((AktForm) && (AktForm->IsKindOf(RUNTIME_CLASS(CRunForm)))) {
					((CRunForm*)AktForm)->OnIdle();
				}
			}		  
		}
	}

	return 1;
}

void CNeuronApp::NewDataPoint(CDataPointDoc *aDataPoint, long Timelaps)
{	
	CRunDoc* ActiveRunDoc=GetActiveRunDoc();
	aDataPoint->m_DetectionRegion = ActiveRunDoc->m_DetectionRegion;
	if (aDataPoint->BeamPath) delete aDataPoint->BeamPath;
	aDataPoint->BeamPath=NULL;	
	if (BeamGuiding && BeamGuiding->BeamPath) {
		Guiding->CalcNewBeamPath(aDataPoint,BeamGuiding,Timelaps);		
	} 
	//DataPointDocTemplate->AddDocument( aDataPoint );
	if (ActiveRunDoc) ActiveRunDoc->NewDataPoint(aDataPoint);	
	if (OnlyOnePicture) {
		OnlyOnePicture=false;
		OnPlay();
		((CMainFrame*)m_pMainWnd)->GetToolBar()->Invalidate();//CheckDlgButton(ID_TAKEONEPICTURE,0);
	}
}

CRunDoc* CNeuronApp::GetActiveRunDoc()
{
	bool found=false;
	CRunDoc* ActRunDoc=NULL;
	POSITION pos=RunDocTemplate->GetFirstDocPosition();	
	while ((pos!=NULL) && (!found)) {
	  ActRunDoc=(CRunDoc*)RunDocTemplate->GetNextDoc(pos);
	  if (ActRunDoc) found=ActRunDoc->IsActive();
	} 
	if (!found) {
		ActRunDoc=(CRunDoc*)RunDocTemplate->CreateNewDocument();	
		ActRunDoc->SetActive();
		CFrameWnd* NewFrame=RunDocTemplate->CreateNewFrame(ActRunDoc,NULL);	
		RunDocTemplate->InitialUpdateFrame( NewFrame, ActRunDoc);
	}
	if ((!ActRunDoc) || (!ActRunDoc->IsActive())) {
		AfxMessageBox("CNeuronApp : GetActiveRunDoc : Active Run Doc not found");
	}
    return ActRunDoc;
}

void CNeuronApp::GetRunDirectory(CString &aRunDirectory,bool Create,int RunNr)
{		
	_chdrive(3); //disk C:
	aRunDirectory="";
	if ((_chdir(RootDirectory)!=0) && (!Create)) return;
	_mkdir(RootDirectory);
	_chdir(RootDirectory);
	char buf[200];
	sprintf(buf,"%s%s",RootDirectory,DataDirectory);
	if ((_chdir(buf)!=0) && (!Create)) return;
	_mkdir(buf);
	_chdir(buf);

	CTime time=CTime::GetCurrentTime();
	CString TimeName=time.Format("%y%m%d");
	CString help=buf;
	help+=TimeName+'\\';
	if ((_chdir(help)!=0) && (!Create)) return;
	_mkdir(help);
	_chdir(help);
	if ((RunNr==-1) && (Create)) {
		struct _finddata_t ffblk;
		long done = _findfirst("*.",&ffblk);
		int max=0;
		bool found=(done!= -1L);
		while (found) {
			char buf[20];
			if (ffblk.name[0]=='R') {
				int i;
				for (i=1;(i<7) && (ffblk.name[i]!='.');i++) buf[i-1]=ffblk.name[i];
				buf[i]=0;
				int n=atoi(buf);
				if (n>max) max=n;
			}
			found = _findnext(done,&ffblk)==0;
		}
		_findclose( done );   
		RunNr=max+1;
	}
	if (RunNr==-1) return;
	char RunDirectory[200];
	//if (RunNr < 10)
	//	sprintf(RunDirectory,"%sR0%i\\",help, RunNr);
	//else
		sprintf(RunDirectory,"%sR%02i\\",help, RunNr);
	if ((_chdir(RunDirectory)!=0) && (!Create)) return;	
	_mkdir(RunDirectory);
	_chdir(RunDirectory);
	aRunDirectory=RunDirectory;
}

LRESULT CNeuronApp::OnGetNeuronApp() 
{
	return 0;//&theApp;
}


// RunDoc.cpp : implementation of the CRunDoc class
//

#include "stdafx.h"
#include "Neuron.h"
#include "DataPointDoc.h"
#include "DataPointView.h"
#include "UnifiedBitmap.h"


//IMPLEMENT_SERIAL(CDataPointMap, CDataPointMapHelp,1)

CDataPointMap::CDataPointMap(CRunDoc* aparent) {
	parent=aparent;
}

CDataPointMap::~CDataPointMap() {

}

BOOL CDataPointMap::LookupWithLoad(WORD key, CDataPointDoc*& aDataPointDoc ) {	
//	if (aDataPointDoc) delete aDataPointDoc;
	aDataPointDoc=NULL;
	if (Lookup(key,aDataPointDoc)==0) {
		CString RunDirectory=parent->GetRunDirectory(false);
		if (RunDirectory.GetLength()==0) return false;
		aDataPointDoc=new CDataPointDoc();
		if (aDataPointDoc->Load(RunDirectory,key)) {
			SetAtRemoveOld(key,aDataPointDoc);
			aDataPointDoc->RunDocParent=parent;
			return true;
		} else {
			delete aDataPointDoc;
			aDataPointDoc=NULL;
			return false;
		}
	} else return true;
}

void CDataPointMap::Serialize(CArchive& ar,WORD LastDataPoint) {
	if (ar.IsStoring())
	{
		CDataPointDoc *aktDoc=NULL;		
		ar<<LastDataPoint;		
		for (int i=1;i<=LastDataPoint;i++) {
			if (LookupWithLoad( i, (CDataPointDoc*&) aktDoc )) {
				ar<<'Y';					
				ASSERT(aktDoc);
				aktDoc->Serialize(ar);
			} else ar<<'N';			
		}
	} else {
		WORD number;
		ar>>number;		
		for (int i=1;i<=number;i++) {
			char h;
			ar>>h;
			if (h=='Y') {
				CDataPointDoc* aktDoc=new CDataPointDoc();			
				aktDoc->Serialize(ar);
				SetAt(i,aktDoc);
			}
		}

	}
}


/////////////////////////////////////////////////////////////////////////////
// CRunDoc

IMPLEMENT_SERIAL(CRunDoc, CDocument,1)

BEGIN_MESSAGE_MAP(CRunDoc, CDocument)
	//{{AFX_MSG_MAP(CRunDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
		ON_COMMAND(ID_VIEW_COMMENT, OnViewComment)
		ON_UPDATE_COMMAND_UI(ID_VIEW_COMMENT, OnUpdateViewComment)
		ON_COMMAND(ID_VIEW_MESSAGE, OnViewMessage)
		ON_UPDATE_COMMAND_UI(ID_VIEW_MESSAGE, OnUpdateViewMessage)		
		ON_COMMAND(ID_LASERPATHTEXT, OnLaserPathText)
		ON_UPDATE_COMMAND_UI(ID_LASERPATHTEXT, OnUpdateLaserPathText)
		ON_COMMAND(ID_PREVIEW, OnPreview)
		ON_UPDATE_COMMAND_UI(ID_PREVIEW, OnUpdatePreview)
		ON_COMMAND(ID_SAVE_LASER_PATH, OnSaveLaserPath)
		ON_COMMAND(ID_SAVE_COMMENT, OnExportComment)		
		ON_COMMAND(ID_CALIBRATION, OnCalibration)			
		ON_COMMAND(ID_CALC_ALL_DETECTION_IMAGES, CalcAllDetectionImages)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_SEND_MAIL, OnFileSendMail)
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, OnUpdateFileSendMail)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRunDoc construction/destruction

bool ActiveRunDocAssigned=false;

CRunDoc::CRunDoc()
{	
	RunNr=-1;
	PreviewVisible=true;
	StartTime=CTime::GetCurrentTime();
	SaveTime=StartTime;
	SetModifiedFlag(false);
	LastLoadedDataPoint=0;
	if (ActiveRunDocAssigned) Active=false;
	else {
		Active=true;
		ActiveRunDocAssigned=true;
	}
	LastPictureNumber=0;
	RunDirectory="";
	SaveCycle=0;
	DataPointMap=new CDataPointMap(this);
	SaveWasActivated=false;
	Selection.left=0;
	Selection.right=0;
	Selection.bottom=0;
	Selection.top=0;
	m_CellImage=true;		
	m_DetectionImage=false;
	m_LaserImage=false;
	m_StartPicture=1;
	m_StopPicture=1;
	SetTitle();	
	ViewComment=true;
	ViewMessage=true;
	m_ContrastRegion.SetRectEmpty();
	RunDiskLimit = 2000; //megabytes, probably
	b_takingscreenshots = false;
}

CRunDoc::~CRunDoc()
{
	if (Active && SaveWasActivated) {
		Save();
		ActiveRunDocAssigned=false;
	}
	POSITION pos=DataPointMap->CMapWordToOb::GetStartPosition();
	WORD key;		
	CDataPointDoc *help=NULL;
	while (pos) {
		DataPointMap->GetNextAssoc(pos,key,help);
		help->RunDocParent=NULL;
		//if (help->GetFirstViewPosition()==NULL) delete help;
		help->m_bAutoDelete=TRUE;
		POSITION viewpos = help->GetFirstViewPosition();
		while (viewpos)
		{
			CDataPointView* view = (CDataPointView*)help->GetNextView(viewpos);
			view->GetParentFrame()->DestroyWindow();
		}
		delete help;
	}	
	delete DataPointMap;


}

BOOL CRunDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CRunDoc diagnostics

#ifdef _DEBUG
void CRunDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CRunDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRunDoc commands

bool CRunDoc::IsActive()
{
  return Active;
}

bool CRunDoc::ExchangeDataPointInView(CDataPointDoc *NewDataPoint,CDataPointDoc *OldDataPoint,bool ChangeAllViews)
{
	bool ViewFound=false;
	CDataPointView* View;
	POSITION ViewPos=OldDataPoint->GetFirstViewPosition( );
	while ((ViewPos!=NULL) && ((!ViewFound) || (ChangeAllViews))) {
		View=(CDataPointView*)OldDataPoint->GetNextView(ViewPos);
		if ((View) && (!View->IsPreview)) {
			ViewFound=true;
			OldDataPoint->RemoveView(View);
			NewDataPoint->AddView(View);
			if (View->UnifiedBitmap) {
				if (View->UnifiedBitmap->Bitmap) {  //this is weird but if I dont put it the Debugger will crash
					delete View->UnifiedBitmap->Bitmap;
					View->UnifiedBitmap->Bitmap=NULL;
				}
				delete View->UnifiedBitmap;
				View->UnifiedBitmap=NULL;
			}
			if (b_takingscreenshots)
			{
				View->Invalidate();
				char name[200];
				sprintf(name,"%sNeuron%04i.BMP",GetRunDirectory(true),NewDataPoint->MyNumber);
				WriteWindowToDIB(name, View);
			}
		}
	}	
	if (ViewFound){
		NewDataPoint->UpdateTitle();
	}
	return ViewFound;
}

void CRunDoc::NewDataPoint(CDataPointDoc *aDataPoint)
{
	if (IsRunFull())
	{
		AfxMessageBox("Run size limit exceeded");
		return;
	}
	SetModifiedFlag(true);
	aDataPoint->RunDocParent=this;
	CDataPointDoc* LastDoc=NULL;
	unsigned short oldLastPictureNumber=LastPictureNumber;
	if (LastPictureNumber>0) {
		if (DataPointMap->Lookup( LastPictureNumber, (CDataPointDoc*&) LastDoc )==0) LastDoc=NULL; 
	}
	LastPictureNumber++;  //first real datapoint has number 1, 0 does not correspond to a datapoint
	if (LastPictureNumber==1) StartTime=CTime::GetCurrentTime();
	aDataPoint->RelativeTime=aDataPoint->time-StartTime;
	aDataPoint->MyNumber=LastPictureNumber;	
	DataPointMap->SetAtRemoveOld(LastPictureNumber,aDataPoint);
	aDataPoint->m_ContrastRegion = m_ContrastRegion;
	//aDataPoint->m_DetectionRegion = m_DetectionRegion;
//	aDataPoint->OptimizeContrast();
	if (LastDoc) {
		LastDoc->Active=false;		
		ExchangeDataPointInView(aDataPoint,LastDoc,true);		
		LastDoc->UpdateTitle();
	} else if (GetNeuronApp()) {		
		CFrameWnd* NewFrame=GetNeuronApp()->DataPointDocTemplate->CreateNewFrame(aDataPoint,NULL);	
		GetNeuronApp()->DataPointDocTemplate->InitialUpdateFrame( NewFrame, aDataPoint);
	}		
	aDataPoint->UpdateAllViews(NULL);
	bool ImagesSaved=false;
	if (GetNeuronApp()) {		
		if (GetNeuronApp()->Saving) {	
			SaveWasActivated=true;
			GetRunDirectory(true);				
			if (RunDirectory.GetLength()!=0) {
				SaveCycle++;
				if (SaveCycle>aDataPoint->Param.StoreCycle) {
					SaveCycle=0;
					aDataPoint->SaveImages(RunDirectory);				
					ImagesSaved=true;
				}		
				aDataPoint->CalcPreview(GetNeuronApp()->PreviewBinning);
				aDataPoint->Save(RunDirectory,false);	
			}
		}		
	}	
	if (!ImagesSaved) aDataPoint->CompressImages();	
	aDataPoint->CalcPreview(GetNeuronApp()->PreviewBinning);
	aDataPoint->UpdateTitle();
	SetTitle();
	CRunForm* Form=GetRunForm();
	if (Form!=NULL) Form->ChangeMaxDataPoint();	
}

void CRunDoc::SetActive()
{
  Active=true;
  ActiveRunDocAssigned=true;
}

void CRunDoc::Save()
{
	GetRunDirectory(true);
	if (RunDirectory.GetLength()!=0) {
		char h=RunDirectory[RunDirectory.GetLength()-2];
		char pFileName[200];
		sprintf(pFileName,"%sR%c.run",RunDirectory,h);		
		CFile f;	
		if( !f.Open( pFileName, CFile::modeCreate | CFile::modeWrite ) ) {
			#ifdef _DEBUG
			afxDump << "Unable to open file" << "\n";
			exit( 1 );
			#endif
		}
		CArchive ar( &f, CArchive::store);
		Serialize(ar);
		ar.Close();
		f.Close();				
		sprintf(pFileName,"%sParams.dat",RunDirectory);		
		GetNeuronApp()->SaveParams(pFileName);
		SetModifiedFlag(false);		
		SaveTime=CTime::GetCurrentTime();
		SetTitle();
	}	
}

/////////////////////////////////////////////////////////////////////////////
// CRunDoc serialization

void CRunDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		if (PreviewVisible) ar<<'Y'; else ar<<'N';
		ar<<StartTime;
		ar<<RunNr;
		ar<<RunDirectory;
		ar<<SaveCycle;
		ar<<LastPictureNumber;	
		DataPointMap->Serialize(ar,LastPictureNumber);							
	}
	else
	{
		CFile* fp = ar.GetFile();
		CString Path=fp->GetFilePath();
		CString Name=fp->GetFileName();		
		Path=Path.Left(Path.GetLength()-Name.GetLength());
		Active=false;		
		char help;
		ar>>help;
		PreviewVisible=help=='Y';
		ar>>StartTime;
		ar>>RunNr;
		ar>>RunDirectory;		
		RunDirectory=Path;
		ar>>SaveCycle;
		ar>>LastPictureNumber;		
		DataPointMap->Serialize(ar,LastPictureNumber);			
		CDataPointDoc *LastDoc;
		bool TextFound=false;
		for (int i=1;i<=LastPictureNumber;i++) {
			if (DataPointMap->Lookup( i, (CDataPointDoc*&) LastDoc )!=0) {
				LastDoc->RunDocParent=this;
				TextFound=TextFound || (LastDoc->Message.GetLength()>0) || (LastDoc->Comment.GetLength()>0);
			}
		}
		if (TextFound) {
			ViewComment=true;
			ViewMessage=true;
			CEditView* view=SearchTextView();
			UpdateCommentViewText(view);
		}

		//if (LastPictureNumber>0 && GetNextWithImage(LastLoadedDataPoint)) {	
			
			//LoadDatapoint(LastLoadedDataPoint);
			//UpdateAllViews(NULL, 1L);
			//SearchPreview(LastDoc);
			/*
			if (DataPointMap->Lookup( LastLoadedDataPoint, (CDataPointDoc*&) LastDoc )!=0) {
				CFrameWnd* NewFrame=GetNeuronApp()->DataPointDocTemplate->CreateNewFrame(LastDoc,NULL);	
				GetNeuronApp()->DataPointDocTemplate->InitialUpdateFrame( NewFrame, LastDoc);
				UpdateAllViews(NULL);
				SearchPreview(LastDoc);
			}
			*/
		//}		
		SetTitle();
	}
}

CNeuronApp* CRunDoc::GetNeuronApp()
{
	return (CNeuronApp*)AfxGetApp();//&theApp;
}

void CNeuronApp::OnMicroscopeLightOnOff() 
{
	Microscope->ToggleLight();
}

void CNeuronApp::OnUpdateMicroscopeLightOnOff(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( Microscope->IsLightOn() );
	pCmdUI->Enable( true );
}

void CNeuronApp::OnGuidingDialog() {
	Guiding->OnGuidingDialog();
}

void CRunDoc::DataPointDocDelete(WORD DeletedDataPointNumber)
{
	DataPointMap->RemoveKey(DeletedDataPointNumber);
}

void CNeuronApp::OnConfigGeneralProperties() 
{
	CNeuronAppDlg dlg;
// Initialize dialog data	
	dlg.m_LaserSizeX=LaserSize.x;
	dlg.m_LaserSizeY=LaserSize.y;
	dlg.m_ImageCacheSize=ImageCacheSize;	
	dlg.m_DataDirectory=DataDirectory;
	dlg.m_RootDirectory=RootDirectory;
	dlg.m_PreviewBinning=PreviewBinning;
	dlg.m_TestPictureName=Acquisition->GetTestPictureName();
	dlg.m_ForceTestPicture=Acquisition->ForceTestPicture;
	dlg.m_HardwareAccess=UserAllowsHardwareAccess;
	dlg.m_PlaybackSpeed=PreviewPlaybackSpeed;
	dlg.m_AutoSaveIntervallMinutes=AutoSaveIntervallMinutes;
	// Invoke the dialog box
	if (dlg.DoModal() == IDOK) { 
		LaserSize.x=dlg.m_LaserSizeX;
		LaserSize.y=dlg.m_LaserSizeY;
		ImageCacheSize=dlg.m_ImageCacheSize;		
		DataDirectory=dlg.m_DataDirectory;
		RootDirectory=dlg.m_RootDirectory;
		PreviewBinning=dlg.m_PreviewBinning;
		UserAllowsHardwareAccess=dlg.m_HardwareAccess;
		Acquisition->SetTestPictureName(dlg.m_TestPictureName);
		Acquisition->ForceTestPicture=dlg.m_ForceTestPicture;
		PreviewPlaybackSpeed=dlg.m_PlaybackSpeed;
		AutoSaveIntervallMinutes=dlg.m_AutoSaveIntervallMinutes;
	}	
}


void CNeuronApp::OnReferenceImageDlg() 
{
	CReferenceImageDlg dlg;
// Initialize dialog data	
	dlg.m_ReferenceImageFixNr=ReferenceImageFixNr;
	dlg.m_ReferenceImageRelativeDistance=ReferenceImageRelativeDistance;
	dlg.m_ReferenceImageMode=ReferenceImageMode;	
	// Invoke the dialog box
	if (dlg.DoModal() == IDOK) { 
		ReferenceImageFixNr=dlg.m_ReferenceImageFixNr;
		ReferenceImageRelativeDistance=dlg.m_ReferenceImageRelativeDistance;
		ReferenceImageMode=dlg.m_ReferenceImageMode;			
	}	
}

CString CRunDoc::GetRunDirectory(bool Create)
{
	if (Active) {
		if (RunDirectory.GetLength()==0) GetNeuronApp()->GetRunDirectory(RunDirectory,Create,RunNr);
	}
	return RunDirectory;
}

void CDataPointMap::DeleteOldImages()
{
	long CacheSize=parent->GetNeuronApp()->ImageCacheSize;
	if (CacheSize<=1) CacheSize=1;	
	int NrPoints=NumberOfDataPointsWithImage();
	if (NrPoints>CacheSize) {
		for (int i=0;i<NrPoints-CacheSize;i++) {
			CDataPointDoc* OldPoint=GetOldestDataPointWithImage();
			if (OldPoint) OldPoint->DeleteImages();
		}
	}	
}

long CDataPointMap::NumberOfDataPointsWithImage()
{
	long Nr=0;
	POSITION pos=CMapWordToOb::GetStartPosition();
	WORD key;
	CDataPointDoc* value;
	while (pos!=NULL) {		
		GetNextAssoc(pos,key,value);
		if (value && value->CellImage) Nr++;
	}
	return Nr;
}

long CDataPointMap::NumberOfDataPoints()
{
	long Nr=0;
	POSITION pos=CMapWordToOb::GetStartPosition();
	WORD key;
	CDataPointDoc* value;
	while (pos!=NULL) {		
		GetNextAssoc(pos,key,value);
		Nr++;
	}
	return Nr;
}

CDataPointDoc* CDataPointMap::GetOldestDataPoint()
{
	CTime oldestTime=CTime::GetCurrentTime();	
	POSITION pos=CMapWordToOb::GetStartPosition();
	WORD key;	
	CDataPointDoc *oldest=NULL;
	CDataPointDoc *help=NULL;
	while (pos!=NULL) {
		GetNextAssoc(pos,key,help);		
		if ((help->ImageLookupTime)<oldestTime) {
			oldestTime=help->ImageLookupTime;
			oldest=help;
		}
	}
	return oldest;
}

CDataPointDoc* CDataPointMap::GetOldestDataPointWithImage()
{
	CTime oldestTime=CTime::GetCurrentTime();	
	POSITION pos=CMapWordToOb::GetStartPosition();
	WORD key;	
	CDataPointDoc *oldest=NULL;
	CDataPointDoc *help=NULL;
	while (pos!=NULL) {
		GetNextAssoc(pos,key,help);		
		if (((help->ImageLookupTime)<oldestTime) && (help->CellImage)) {
			oldestTime=help->ImageLookupTime;
			oldest=help;
		}
	}
	return oldest;
}

void CDataPointMap::SetAtRemoveOld(WORD key, CDataPointDoc *newValue)
{
	SetAt(key,newValue);
	DeleteOldImages();
}


void CRunDoc::LoadDatapointInNewView(long Number)
{
	if ((Number>0) && (Number<=LastPictureNumber)) {
		CDataPointDoc* Data=NULL;
		if (DataPointMap->LookupWithLoad(Number,Data) && GetNeuronApp()) {
			CFrameWnd* NewFrame=GetNeuronApp()->DataPointDocTemplate->CreateNewFrame(Data,NULL);	
			GetNeuronApp()->DataPointDocTemplate->InitialUpdateFrame( NewFrame, Data);					
			LastLoadedDataPoint=Number;
		}
	}
}

void CRunDoc::LoadDatapoint(long Number)
{
	if ((Number>0) && (Number<=LastPictureNumber)) {
		DataPointMap->DeleteOldImages();
		if (LastLoadedDataPoint==0)
			LoadDatapointInNewView(Number);
		else 
		{
			CDataPointDoc* LastDoc=NULL;			
			DataPointMap->Lookup( LastLoadedDataPoint, (CDataPointDoc*&) LastDoc ); 
			//if (DataPointMap->Lookup( LastLoadedDataPoint, (CDataPointDoc*&) LastDoc )==0) 
			//	LoadDatapointInNewView(Number);
			//else 
			{
				CDataPointDoc* Data=NULL;
				if (DataPointMap->LookupWithLoad(Number,Data) && GetNeuronApp())
				{
					if (!ExchangeDataPointInView(Data,LastDoc,true))
						LoadDatapointInNewView(Number);	
					else 
						Data->UpdateAllViews(NULL);
					LastLoadedDataPoint = Number;
				}
			}
		}
	}
}

CRunForm* CRunDoc::GetRunForm()
{
	CRunForm* View=NULL;
	POSITION ViewPos=GetFirstViewPosition( );
	if (ViewPos!=NULL) View=(CRunForm*)GetNextView(ViewPos);
	return View;
}

int CRunDoc::FindNextImage(int number, int increment)
{
	//int oldNumber = number;
	bool Found=false;
	number += increment;
	CDataPointDoc *LastDoc=NULL;
	if (DataPointMap->LookupWithLoad(number, (CDataPointDoc*&) LastDoc ))
		Found = LastDoc->CellImageStored || LastDoc->CellImage;
	if (increment >= 0)
		increment = 1;
	else
		increment = -1;
	while (!Found) 
	{
		if (number < 1)
		{
			number = 0;
			increment = 1;
		}
		else if (number > LastPictureNumber)
		{
			number = LastPictureNumber + 1;
			increment = -1;
		}
		number += increment;
		if (DataPointMap->LookupWithLoad(number, (CDataPointDoc*&) LastDoc ))
			Found = LastDoc->CellImageStored || LastDoc->CellImage;
	} 
	//if (!Found) 
	//	number = oldNumber;
	
	return number;
}

CDataPointDoc* CRunDoc::GetDataPoint(WORD Number)
{
	CDataPointDoc* Doc=NULL;
	if (DataPointMap->LookupWithLoad(Number, (CDataPointDoc*&) Doc )==0) Doc=NULL;
	return Doc;
}

CString CRunDoc::GetCommentText(bool ViewComment, bool ViewMessage) {
	CString Text="";			
	for (WORD i=1;i<=LastPictureNumber;i++) {
		CDataPointDoc* doc;
		if (DataPointMap->Lookup( i, (CDataPointDoc*&) doc )==0) doc=NULL; 
		if (doc) {				
			if ((ViewMessage && (doc->Message!="")) || (ViewComment && (doc->Comment!=""))) {
				Text+=doc->UpdateTitle()+"\r\n";
				if (ViewMessage && (doc->Message!="")) Text+=doc->Message+"\r\n";
				if (ViewComment && (doc->Comment!="")) Text+=doc->Comment+"\r\n";				
				Text+="\r\n";
			}
		}
	}
	return Text;
}

void CRunDoc::UpdateCommentViewText(CEditView* CommentView) {
	if (CommentView!=NULL) {
		if ((!ViewComment) && (!ViewMessage)) ViewComment=true;
		CEdit& Edit=CommentView->GetEditCtrl( );
		Edit.SetSel(-1 << 16,true);
		Edit.ReplaceSel( GetCommentText(ViewComment,ViewMessage));
	}
}

CEditView* CRunDoc::SearchTextView() {
	POSITION Pos=GetFirstViewPosition( );
	while (Pos!=NULL) {
		CView* View=GetNextView(Pos);
		if ((View) && (View->IsKindOf( RUNTIME_CLASS( CEditView )))) return (CEditView*)View;
	}
	CFrameWnd* NewFrame=GetNeuronApp()->RunDocCommentViewTemplate->CreateNewFrame(this,NULL);	
	GetNeuronApp()->RunDocCommentViewTemplate->InitialUpdateFrame( NewFrame, this);
//	GetNeuronApp()->RunDocCommentViewTemplate->AddDocument( this );
	CEditView* View=(CEditView*)NewFrame->GetActiveView();	
	CEdit& Edit=View->GetEditCtrl( );
	Edit.SetReadOnly();	
	return View;
}

CDataPointView* CRunDoc::SearchPreview(CDataPointDoc* pDoc) {
	if ((!PreviewVisible) || ((pDoc->Preview==NULL) && (!pDoc->CellImageStored))) return NULL;
	POSITION Pos=GetNeuronApp()->DataPointDocTemplate->GetFirstDocPosition( );
	while (Pos!=NULL) {
		CDataPointDoc* Doc=(CDataPointDoc*)GetNeuronApp()->DataPointDocTemplate->GetNextDoc(Pos);
		if (Doc) {
			POSITION ViewPos=Doc->GetFirstViewPosition( );
			while (ViewPos!=NULL) {
				CDataPointView* View=(CDataPointView*)Doc->GetNextView(ViewPos);
				if ((View) && (View->IsPreview)) {
					Doc->RemoveView(View);
					pDoc->AddView(View);	
					GetNeuronApp()->DataPointDocTemplate->RemoveDocument( Doc );
					GetNeuronApp()->DataPointDocTemplate->AddDocument( pDoc );
					CHint *hint=new CHint(3,CRect());
					pDoc->UpdateAllViews(NULL,0L, hint);
					pDoc->UpdateTitle();
					delete hint;					
					return (CDataPointView*)View;
				}
			}
		}
	}
	CFrameWnd* NewFrame=GetNeuronApp()->DataPointDocTemplate->CreateNewFrame(pDoc,NULL);		
	GetNeuronApp()->DataPointDocTemplate->InitialUpdateFrame( NewFrame, pDoc);
	GetNeuronApp()->DataPointDocTemplate->AddDocument( pDoc );
	CDataPointView* View=(CDataPointView*)NewFrame->GetActiveView();
	View->SetAsPreview();
	CHint *hint=new CHint(3,CRect());
	pDoc->UpdateAllViews(NULL,0L, hint);
	delete hint;	
	return View;
}

void CRunDoc::OnViewComment() 
{
	ViewComment=!ViewComment;
	CEditView* view=SearchTextView();
	UpdateCommentViewText(view);
}

void CRunDoc::OnUpdateViewComment(CCmdUI* pCmdUI) 
{	
	pCmdUI->SetCheck(ViewComment);
	pCmdUI->Enable(true);			
}

void CRunDoc::OnViewMessage() 
{
	ViewMessage=!ViewMessage;
	CEditView* view=SearchTextView();
	UpdateCommentViewText(view);
}

void CRunDoc::OnUpdateViewMessage(CCmdUI* pCmdUI) 
{	
	pCmdUI->SetCheck(ViewMessage);
	pCmdUI->Enable(true);			
}

void CRunDoc::MessageChanged()
{
	ViewMessage=false;
	OnViewMessage();	
}

void CRunDoc::CommentChanged()
{
	ViewComment=false;
	OnViewComment();	
}

CString CRunDoc::GetLaserPathText() {
	unsigned short MaxBeamStops=0;
	for (WORD i=1;i<=LastPictureNumber;i++) {
		CDataPointDoc* doc;
		if (DataPointMap->Lookup( i, (CDataPointDoc*&) doc )==0) doc=NULL; 
		if ((doc) && (doc->BeamPath) && (doc->BeamPath->MaxNumber>MaxBeamStops)) MaxBeamStops=doc->BeamPath->MaxNumber;			
	}	
	CString Text="PictureNr Time BeamOn";
	for (unsigned short j=1;j<=MaxBeamStops;j++) {
		char buf[200];
		sprintf(buf," X%u Y%u l%u C%u A%u ASX%u ASY%u",j,j,j,j,j,j,j);
		Text+=buf;			
	}
	Text+="\r\n";			
	for (i=1;i<=LastPictureNumber;i++) {
		CDataPointDoc* doc;
		if (DataPointMap->Lookup( i, (CDataPointDoc*&) doc )==0) doc=NULL; 
		if (doc) {
			char buf[100];					
			sprintf(buf,"%i %i %i ",doc->MyNumber,doc->RelativeTime.GetTotalSeconds(), ((doc->BeamPath) && (doc->BeamPath->BeamOn)) ? 1 : 0);
			CString help=buf;
			Text+=help;
			if (doc->BeamPath) {
				CBeamStop* BeamStop;
				for (j=1;j<=MaxBeamStops;j++) {
					BeamStop=doc->BeamPath->GetBeamStop(j);
					if (BeamStop!=NULL) {						
						//conversion from nm to mikrometer
						sprintf(buf," %3.3f %3.3f %3.3f %3.3f %3.3f %3.3f %3.3f",0.001*BeamStop->Position.x,
							0.001*BeamStop->Position.y, 0.001*BeamStop->NeuronPositionOnSpline,0.000001*BeamStop->CellArea,
							0.000001*BeamStop->AxonArea,0.001*BeamStop->AxonStartPos.x,0.001*BeamStop->AxonStartPos.y); 
						help=buf;
						Text+=help;						
					} else Text+=" 0.0 0.0 0.0 0.0 0.0 0.0 0.0";
				}					
			} else {
				for (int k=0;k<MaxBeamStops*7;k++) Text+=" 0.0";
			}
			Text+="\r\n";
		}
	}
	return Text;
}

void CRunDoc::UpdateLaserPathText(CEditView *LaserPathText) {
	if (LaserPathText!=NULL) {
		CString Text=GetLaserPathText();
		CEdit& Edit=LaserPathText->GetEditCtrl( );
		Edit.SetSel(-1 << 16,true);
		Edit.ReplaceSel( Text);
	}
}

void CRunDoc::OnLaserPathText() 
{
	CEditView* view=SearchTextView();
	UpdateLaserPathText(view);
}

void CRunDoc::OnUpdateLaserPathText(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(true);			
}

void CRunDoc::OnPreview() 
{
	PreviewVisible=!PreviewVisible;
	if ((LastPictureNumber>0) && (PreviewVisible)) {
		CDataPointDoc* LastDoc;
		if (DataPointMap->Lookup( LastPictureNumber, (CDataPointDoc*&) LastDoc )==0) LastDoc=NULL; 
		if (LastDoc) SearchPreview(LastDoc);	
	}	
}

void CRunDoc::OnUpdatePreview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(true);			
	pCmdUI->SetCheck( PreviewVisible);
}

void CRunDoc::SaveActivated()
{
	SaveWasActivated=true;
}

void CRunDoc::SaveAsBMP(long number)
{	
	CPictureExportDlg dlg;
	dlg.m_CellImage=m_CellImage;		
	dlg.m_DetectionImage=m_DetectionImage;
	dlg.m_LaserImage=m_LaserImage;
	if (number==0) {
		dlg.m_StartPicture=1;
		dlg.m_StopPicture=LastPictureNumber;
	} else {
		dlg.m_StartPicture=number;
		dlg.m_StopPicture=number;
	}
	dlg.m_x1=Selection.left;
	dlg.m_y1=Selection.top;
	dlg.m_x2=Selection.right;
	dlg.m_y2=Selection.bottom;
	//dlg.Selection = Selection;
	dlg.LastPictureNumber=LastPictureNumber;
	//dlg.datapointmap = DataPointMap;
	dlg.rundoc = this;
	b_takingscreenshots = true;
	if (dlg.DoModal() == 1) 
	{		
		m_CellImage=dlg.m_CellImage;		
		m_DetectionImage=dlg.m_DetectionImage;
		m_LaserImage=dlg.m_LaserImage;
		m_StartPicture=dlg.m_StartPicture;
		m_StopPicture=dlg.m_StopPicture;
		Selection.left=dlg.m_x1;
		Selection.top=dlg.m_y1;
		Selection.right=dlg.m_x2;
		Selection.bottom=dlg.m_y2;
	}
}

//DEL void CRunDoc::OnExportPictureDlg(long number)
//DEL {	
//DEL 	CPictureExportDlg dlg;
//DEL 	dlg.m_CellImage=m_CellImage;		
//DEL 	dlg.m_DetectionImage=m_DetectionImage;
//DEL 	dlg.m_LaserImage=m_LaserImage;
//DEL 	if (number==0) {
//DEL 		dlg.m_StartPicture=1;
//DEL 		dlg.m_StopPicture=LastPictureNumber;
//DEL 	} else {
//DEL 		dlg.m_StartPicture=number;
//DEL 		dlg.m_StopPicture=number;
//DEL 	}
//DEL 	dlg.m_x1=Selection.left;
//DEL 	dlg.m_y1=Selection.top;
//DEL 	dlg.m_x2=Selection.right;
//DEL 	dlg.m_y2=Selection.bottom;
//DEL 	if (dlg.m_x1>dlg.m_x2) {
//DEL 		int h=dlg.m_x2;
//DEL 		dlg.m_x2=dlg.m_x1;
//DEL 		dlg.m_x1=h;
//DEL 	}
//DEL 	if (dlg.m_y1>dlg.m_y2) {
//DEL 		int h=dlg.m_y2;
//DEL 		dlg.m_y2=dlg.m_y1;
//DEL 		dlg.m_y1=h;
//DEL 	}
//DEL 	dlg.LastPictureNumber=LastPictureNumber;
//DEL 	if (dlg.DoModal() == IDOK) {		
//DEL 		m_CellImage=dlg.m_CellImage;		
//DEL 		m_DetectionImage=dlg.m_DetectionImage;
//DEL 		m_LaserImage=dlg.m_LaserImage;
//DEL 		m_StartPicture=dlg.m_StartPicture;
//DEL 		m_StopPicture=dlg.m_StopPicture;
//DEL 		Selection.left=dlg.m_x1;
//DEL 		Selection.top=dlg.m_y1;
//DEL 		Selection.right=dlg.m_x2;
//DEL 		Selection.bottom=dlg.m_y2;
//DEL 	}
//DEL }

void CRunDoc::OnSaveLaserPath() 
{	
	if (GetRunDirectory(true).GetLength()!=0) {		
		char name[200];
		sprintf(name,"%sPath.txt",GetRunDirectory(true));
		CString text=GetLaserPathText();
		ofstream out(name);
		out<<text;
		SetTitle();
	}	
}

void CRunDoc::OnExportComment() 
{	
	if (GetRunDirectory(true).GetLength()!=0) {		
		char name[200];
		sprintf(name,"%sComment.txt",GetRunDirectory(true));
		CString text=GetCommentText();
		ofstream out(name);
		out<<text;
		SetTitle();
	}	
}

void CRunDoc::OnCalibration() 
{
	GetNeuronApp()->OnCalibration();	
}

void CNeuronApp::OnFileNew() 
{
	if (OnFileNewCalled) RunDocTemplate->OpenDocumentFile(NULL);
	OnFileNewCalled=true;
}

void CNeuronApp::OnUpdateZoomIn(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( true );
}

void CNeuronApp::OnUpdateZoomOut(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( true );
}

void CNeuronApp::OnZoomOut() 
{
	m_pMainWnd->SendMessage(IDM_ZOOM_OUT,0,0);	
}

void CNeuronApp::OnZoomIn() 
{
	m_pMainWnd->SendMessage(IDM_ZOOM_IN,0,0);	
}

//DEL void CNeuronApp::OnSelectRegion() 
//DEL {
//DEL 	//m_pMainWnd->SendMessage(IDM_SELECT_REGION,0,0);		
//DEL }

//DEL void CNeuronApp::OnUpdateSelectRegion(CCmdUI* pCmdUI) 
//DEL {
//DEL 	pCmdUI->Enable( true );
//DEL }


void CNeuronApp::OnTakeonepicture() 
{
	OnlyOnePicture=true;
	OnPlay();
}

void CNeuronApp::OnUpdateTakeonepicture(CCmdUI* pCmdUI) 
{
	/*
	if (Acquisition) {
		pCmdUI->SetCheck( false);//Acquisition->IsPlaying() && OnlyOnePicture);		
		pCmdUI->Enable( true );
	} //else pCmdUI->Enable( true );
	*/
}

void CNeuronApp::OnDataextractionDlg() 
{
	CExtractDataDlg dlg;
	dlg.m_CellArea=Guiding->ExtractCellArea;
	dlg.m_AxonArea=Guiding->ExtractAxonArea;
	if (dlg.DoModal()==IDOK) {
		Guiding->ExtractCellArea=dlg.m_CellArea;
		Guiding->ExtractAxonArea=dlg.m_AxonArea;
	}
}

void CNeuronApp::LoadParams(CString name)
{
	CFile f;	
	if( !f.Open( name, CFile::modeRead ) ) return;
	CArchive ar( &f, CArchive::load);
	SerializeParams(ar);
	ar.Close();
	f.Close();
}

void CNeuronApp::SaveParams(CString name)
{
	CFile f;	
	if( !f.Open( name, CFile::modeCreate | CFile::modeWrite ) ) {
		#ifdef _DEBUG
		afxDump << "Unable to open file" << "\n";
		exit( 1 );
		#endif
	}
	CArchive ar( &f, CArchive::store);
	SerializeParams(ar);
	ar.Close();
	f.Close();
}


void CNeuronApp::SerializeParams(CArchive &ar)
{
	if (ar.IsStoring())
	{	
		Acquisition->SerializeParams(ar);
		BeamGuiding->SerializeParams(ar);
		Guiding->SerializeParams(ar);
		BeamStopParamsTemplate.Serialize(ar);
		if (CommentViewVisible) ar<<'Y'; else ar<<'N';
		if (UserAllowsHardwareAccess) ar<<'Y'; else ar<<'N';
		ar<<DataDirectory;
		ar<<ImageCacheSize;
		ar<<LaserSize;
		ar<<PreviewBinning;
		ar<<ReferenceImageFixNr;
		ar<<ReferenceImageMode;
		ar<<ReferenceImageRelativeDistance;
		ar<<RootDirectory;
		ar<<PreviewPlaybackSpeed;
		ar<<AutoSaveIntervallMinutes;
	} else {
		Acquisition->SerializeParams(ar);		
		BeamGuiding->SerializeParams(ar);
		Guiding->SerializeParams(ar);
		BeamStopParamsTemplate.Serialize(ar);
		char help;
		ar>>help;
		CommentViewVisible=help=='Y';
		ar>>help;
		UserAllowsHardwareAccess=help=='Y';		
		ar>>DataDirectory;
		ar>>ImageCacheSize;
		ar>>LaserSize;
		ar>>PreviewBinning;
		ar>>ReferenceImageFixNr;
		ar>>ReferenceImageMode;
		ar>>ReferenceImageRelativeDistance;
		ar>>RootDirectory;		
		ar>>PreviewPlaybackSpeed;
		ar>>AutoSaveIntervallMinutes;
	}
}

void CNeuronApp::OnBeamstoptemplate() 
{
	BeamStopParamsTemplate.DoModal();	
}

void CNeuronApp::OnBeamguidingdlg() 
{
	if (BeamGuiding) BeamGuiding->ConfigDlg();	
}

void Delay(DWORD TickCounts) //Delay in ms
{	
	DWORD StartTickCount=GetTickCount();
	while (GetTickCount()<(StartTickCount+TickCounts)) ;
}

void CNeuronApp::OnCalibration() 
{
	CRunDoc* ActiveRunDoc=GetActiveRunDoc();	
	if (!ActiveRunDoc) return;
	if (!BeamGuiding) return;
	if (!BeamGuiding->BeamPath) return;
	if (!Acquisition) return;
	POSITION pos=BeamGuiding->BeamPath->GetHeadPosition();	
	if (pos==NULL) return;
	CBeamStop* Stop=(CBeamStop*)BeamGuiding->BeamPath->GetNext(pos);	
	if (!Stop) return;
	int nx=6;
	int ny=4;
	long dwidth=0.9*(BeamGuiding->BeamPath->Center.x*2)/(nx-1);
	long dheight=0.9*(BeamGuiding->BeamPath->Center.y*2)/(ny-1);
	long xoffset=0.05*(BeamGuiding->BeamPath->Center.x*2);
	long yoffset=0.05*(BeamGuiding->BeamPath->Center.y*2);
	int n=0;
	for (int x=0;x<nx;x++) for (int y=0;y<ny;y++) {
		POSITION pos=BeamGuiding->BeamPath->GetTailPosition();	
		if (pos==NULL) return;
		CBeamStop* Stop=(CBeamStop*)BeamGuiding->BeamPath->GetNext(pos);		
		if (Stop) {
			n++;
			Stop->Position=CPoint::CPoint(xoffset+dwidth*x,yoffset+dheight*y);
			CBeamPath *help=new CBeamPath(BeamGuiding->BeamPath);
			BeamGuiding->SetNewPath(help);
			delete help;
			OnTakeonepicture();
			CString buf;
			buf.Format("(%i/24) x=%i y=%i", n,xoffset+dwidth*x,yoffset+dheight*y);			
			((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->SetPaneText(((CMainFrame*)AfxGetApp()->m_pMainWnd)->GetStatusBar()->CommandToIndex(ID_INDICATOR_MESSAGE),buf);
			long Timeout=0;
			while (OnlyOnePicture && (Timeout<10000)) {
				Acquisition->OnIdle();
				Timeout++;
				Delay(1);
			}			
		}		
	}
}

void CRunDoc::OnIdle()
{
	int saveintervall=GetNeuronApp()->AutoSaveIntervallMinutes;
	if (Active && IsModified() && (saveintervall>0) && SaveWasActivated) {
		CTimeSpan span=CTime::GetCurrentTime()-SaveTime;
		if (span.GetTotalMinutes()>=saveintervall) Save();
	}
}

void CRunDoc::SetTitle() { 
	CString title;
	//title.Format("(%i/%i)   ", LastLoadedDataPoint, LastPictureNumber);
	if (GetRunDirectory(false).GetLength()!=0) {
		char h=RunDirectory[RunDirectory.GetLength()-2];
		char pFileName[200];				
		sprintf(pFileName,"%sR%c.run",RunDirectory,h);
		title = pFileName;
		if (IsModified()) title+=' *';	
	} else title="Unsaved experimental run";	
	CDocument::SetTitle(title);
	//UpdateAllViews(NULL);
}

void CRunDoc::CalcAllDetectionImages() {
	/*
	if (AfxMessageBox("Do you really want to calculate all detection images ?",MB_OKCANCEL)!=IDOK) return;
	int number=1;
	bool Success=true;
	while (Success) {
		Success=GetNextWithImage(number);
		if (Success) {
			LoadDatapoint(number);
			CDataPointDoc *AktDataPoint=GetDataPoint(number);
			if (AktDataPoint) AktDataPoint->CalcDetectionImage();
		}
	}
	*/
}

//DEL void CNeuronApp::OnUpdateAutocontrast(CCmdUI* pCmdUI) 
//DEL {
//DEL 	pCmdUI->Enable(true);
//DEL }

void CNeuronApp::OnUpdateSelectRegion(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(true);	
}

bool CRunDoc::IsRunFull()
{	
	CString rundir = GetRunDirectory(false);
	CFileFind finder;
	BOOL working = finder.FindFile("*");
	DWORD mtotal = 0; //total in megabytes
	while (working)
	{
		working = finder.FindNextFile();
		if (!finder.IsDots())
			mtotal += finder.GetLength() / 1024 / 1024;
	}
	return mtotal >= RunDiskLimit;
		
}



void CRunDoc::SetTitle(int number)
{
	CString title;
	title.Format("Picture %i/%i   ", number, LastPictureNumber);
	if (GetRunDirectory(false).GetLength()!=0) {
		char h=RunDirectory[RunDirectory.GetLength()-2];
		char pFileName[200];				
		sprintf(pFileName,"%sR%c.run",RunDirectory,h);
		title += pFileName;
		if (IsModified()) title+=' *';	
	} else title="Unsaved experimental run";	
	CDocument::SetTitle(title);
}

void CNeuronApp::OnFakeRun() 
{
	if (Fakerun)
		Fakerun = false;
	else
	{

		CString dir = CSelectDirDialog();
		if (!dir.IsEmpty())
		{
			if (Acquisition) Acquisition->FakeRunDir = dir;
			Fakerun = true;
		}
	}
}

void CNeuronApp::OnUpdateFakeRun(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(Fakerun);	
}

CString CNeuronApp::CSelectDirDialog()
{
	BROWSEINFO  Bi;
	LPITEMIDLIST  lpItemLst;
	char lpName[255];
	HRESULT  rc;
	LPMALLOC pMalloc;

	Bi.hwndOwner = m_pMainWnd->m_hWnd;
	Bi.pidlRoot = NULL;
	Bi.pszDisplayName = lpName;
	Bi.lpszTitle = "Local Directory";
	Bi.ulFlags = NULL;
	Bi.lpfn = NULL;
	Bi.lParam = NULL;
	Bi.iImage = 0;

	lpItemLst = SHBrowseForFolder (&Bi);
	SHGetPathFromIDList (lpItemLst, lpName);
	if(SUCCEEDED(SHGetMalloc (&pMalloc)))

	pMalloc->Free ((void *)lpItemLst);
	rc = pMalloc->Release ();
	return CString(lpName);
}

void CNeuronApp::OnUpdateCellImageVisible(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable();	
}

void CNeuronApp::OnUpdateLaserPathVisible(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable();	
}

void CNeuronApp::OnUpdateLaserImageVisible(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable();	
}
