// Neuron.h : main header file for the NEURON application
//

#if !defined(AFX_NEURON_H__AD7880BA_5B31_4D5A_882A_4F160ADFE827__INCLUDED_)
#define AFX_NEURON_H__AD7880BA_5B31_4D5A_882A_4F160ADFE827__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols
#include "DataPointDoc.h"
#include "_idevice.h"	// Added by ClassView
#include "Beam.h"	// Added by ClassView

/////////////////////////////////////////////////////////////////////////////
// CNeuronApp:
// See Neuron.cpp for the implementation of this class
//



class CAcquisition;
class CBeamGuiding;
class CDataPointDoc;
class CRunDoc;
class CGuiding;

class CNeuronApp : public CWinApp
{
public:			
	CString CSelectDirDialog();
	BOOL Fakerun;
	int AutoSaveIntervallMinutes;
	long PreviewPlaybackSpeed;
	CBeamStopDlg BeamStopParamsTemplate;
	bool UserAllowsHardwareAccess;
	HANDLE Mutex;
	CString ParamFileName;
	void SerializeParams(CArchive &ar);	
	void LoadParams(CString name);
	void SaveParams(CString name);
	C_iDevice TemperatureController;
	bool OnlyOnePicture;
	bool OnFileNewCalled;
	int PreviewBinning;
	bool CommentViewVisible;
	CPoint LaserSize;
	long ImageCacheSize;
	long ReferenceImageFixNr;
	long ReferenceImageRelativeDistance;
	long ReferenceImageMode;
	void GetRunDirectory(CString &aRunDirectory,bool Create,int RunNr);
	bool Saving;
	CString RootDirectory;
	CString DataDirectory;

	CMicroscope* Microscope;
	CGuiding* Guiding;
	bool ActiveRunDocAssigned;
	CRunDoc* GetActiveRunDoc();
	void NewDataPoint(CDataPointDoc* aDataPoint, long Timelaps);
	CBeamGuiding* BeamGuiding;
	CAcquisition* Acquisition;	
	CMultiDocTemplate* RunDocTemplate;
	CMultiDocTemplate* RunDocCommentViewTemplate;
    CMultiDocTemplate* DataPointDocTemplate;	
	CNeuronApp();
	virtual ~CNeuronApp();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CNeuronApp)
	public:
	virtual BOOL InitInstance();
	virtual BOOL OnIdle(LONG lCount);
	//}}AFX_VIRTUAL

// Implementation
	//{{AFX_MSG(CNeuronApp)
	afx_msg void OnAppAbout();
	afx_msg void OnAcquisitiondialog();
	afx_msg void OnPlay();
	afx_msg void OnUpdatePlay(CCmdUI* pCmdUI);
	afx_msg void OnLaserOnOff();
	afx_msg void OnUpdateLaserOnOff(CCmdUI* pCmdUI);
	afx_msg void OnSave();
	afx_msg void OnUpdateSave(CCmdUI* pCmdUI);
	afx_msg LRESULT  OnGetNeuronApp();
	afx_msg void OnMicroscopeLightOnOff();
	afx_msg void OnUpdateMicroscopeLightOnOff(CCmdUI* pCmdUI);	
	afx_msg void OnGuidingDialog();
	afx_msg void OnConfigGeneralProperties();	
	afx_msg void OnReferenceImageDlg();	
	afx_msg void OnFileNew();
	afx_msg void OnUpdateZoomIn(CCmdUI* pCmdUI);
	afx_msg void OnUpdateZoomOut(CCmdUI* pCmdUI);
	afx_msg void OnZoomOut();
	afx_msg void OnZoomIn();
	afx_msg void OnTakeonepicture();
	afx_msg void OnUpdateTakeonepicture(CCmdUI* pCmdUI);
	afx_msg void OnDataextractionDlg();
	afx_msg void OnBeamstoptemplate();
	afx_msg void OnBeamguidingdlg();
	afx_msg void OnCalibration();
	afx_msg void OnUpdateSelectRegion(CCmdUI* pCmdUI);
	afx_msg void OnFakeRun();
	afx_msg void OnUpdateFakeRun(CCmdUI* pCmdUI);
	afx_msg void OnUpdateCellImageVisible(CCmdUI* pCmdUI);
	afx_msg void OnUpdateLaserPathVisible(CCmdUI* pCmdUI);
	afx_msg void OnUpdateLaserImageVisible(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
protected:
};

class CDataPointMap : public CTypedPtrMap<CMapWordToOb,WORD,CDataPointDoc*>
{
public:
	CDataPointMap(CRunDoc* aparent);
//	DECLARE_SERIAL(CDataPointMap)
public:
	void SetCacheSize(long aCacheSize);
	void SetAtRemoveOld( WORD key, CDataPointDoc* newValue );
	CDataPointDoc* GetOldestDataPoint();
	CDataPointDoc* GetOldestDataPointWithImage();
	long NumberOfDataPoints();
	long NumberOfDataPointsWithImage();
	void DeleteOldImages();
	CRunDoc* parent;
	virtual ~CDataPointMap();
	virtual void Serialize(CArchive& ar,WORD LastDataPoint);	
	BOOL LookupWithLoad(WORD key, CDataPointDoc*& aDataPointDoc );
};

class CRunForm;
// RunDoc.h : interface of the CRunDoc class
//
/////////////////////////////////////////////////////////////////////////////

class CRunDoc : public CDocument
{
protected: // create from serialization only
	CRunDoc();
	DECLARE_SERIAL(CRunDoc)

// Attributes
public:		
	CDataPointMap* DataPointMap;
private:
	CString RunDirectory;
// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CRunDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:		
	BOOL b_takingscreenshots;
	CRect m_DetectionRegion;
	CRect m_ContrastRegion;
	BOOL	m_CellImage;
	BOOL	m_DetectionImage;
	BOOL	m_LaserImage;
	long	m_StartPicture;
	long	m_StopPicture;
	void MessageChanged();
	CRect Selection;
	int RunNr;
	void SaveAsBMP(long number);
	CString GetLaserPathText();
	CString GetCommentText(bool ViewComment=true, bool ViewMessage=true);
	bool PreviewVisible;
	void SaveActivated();
	bool SaveWasActivated;
	unsigned short LastPictureNumber;
	void CommentChanged();	
	CDataPointDoc* GetDataPoint(WORD Number);
	CTime StartTime;
	int FindNextImage(int number, int increment);
	CRunForm* GetRunForm();
	long LastLoadedDataPoint;
	void LoadDatapointInNewView(long Number);
	void LoadDatapoint(long Number);
	CString GetRunDirectory(bool Create);
	void DataPointDocDelete(WORD DeletedDataPointNumber);
	CNeuronApp* GetNeuronApp();
	void Save();
	unsigned short SaveCycle;
	void SetActive();
	bool ExchangeDataPointInView(CDataPointDoc *NewDataPoint,CDataPointDoc *OldDataPoint,bool ChangeAllViews);
	void NewDataPoint(CDataPointDoc* aDataPoint);
	bool IsActive();
	virtual ~CRunDoc();
	void UpdateCommentViewText(CEditView* CommentView);
//	void UpdateCommentView();
	void UpdateLaserPathText(CEditView *LaserPathText);
	void UpdateLaserPathTextView();
	CDataPointView* SearchPreview(CDataPointDoc* pDoc);
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;	
#endif

protected:

// Generated message map functions
public:
	void SetTitle(int number);
	bool IsRunFull();
	//the max size for the run in megabytes
	unsigned int RunDiskLimit;
	virtual void SetTitle();
	CTime SaveTime;
	void OnIdle();
	bool ViewMessage;
	bool ViewComment;
	//{{AFX_MSG(CRunDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
		afx_msg void OnViewComment();
		afx_msg void OnLaserPathText();
		afx_msg void OnPreview();
		afx_msg void OnUpdateViewComment(CCmdUI* pCmdUI);		
		afx_msg void OnUpdateLaserPathText(CCmdUI* pCmdUI);		
		afx_msg void OnUpdatePreview(CCmdUI* pCmdUI);
		afx_msg void OnViewMessage();
		afx_msg void OnUpdateViewMessage(CCmdUI* pCmdUI);
		afx_msg void OnSaveLaserPath();
		afx_msg void OnExportComment();
		afx_msg void OnCalibration();		
		afx_msg void CalcAllDetectionImages();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	bool Active;	
	CEditView* SearchTextView();
};

void Delay(DWORD TickCounts);
/////////////////////////////////////////////////////////////////////////////


//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.


#endif // !defined(AFX_NEURON_H__AD7880BA_5B31_4D5A_882A_4F160ADFE827__INCLUDED_)


