// SmoothCurve.cpp: implementation of the CSmoothCurve class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Neuron.h"
#include "SmoothCurve.h"
#include "fit.h"
#include "Beam.h"
#include "DataPointView.h"
#include <fstream.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/*
bool fit(double x[],	//x and y datapoints from 0 to ndata-1   //returns matherror
		 double y[], 
		 int ndata,		//number of datapoints
		 double a[],	//coefficients 0 to ma-1
		 double v[],	//should coefficient be fitted v=1 or treated as constant v=0  from 0 to ma-1
  		 int ma,		//number of arguments
		 double *chisq, //chisqu
		 int maxn,		//maximum number of fitting rounds
		 int minn,		//number of fitting rounds after last that changed something in coefficients		 
		 double minchisq,	//stop if chisqu<minchisqu		 
		 int &nend,			//return value: number of fitting rounds
		 double &changeend,	//return value: value of change during last round
		 void (*funcs)(  //fitting function
			double,			//point at which to evaluate
			double [],		//coefficients
			double *,		//return value
			double [],		//return derivatives
			int)			//number of coefficients
	) 

double (*FitFunc1D)(double, double []);  //set my fit function: func(x,a): x=f(a);

void SetFitFunc1D(double (*func)(double, double [])
*/

int h_PolynomDegree=3;
double h_Range=1.0;
double SmoothCurveFitFunction(double t, double a[]) {
	double f=0;
	if (t<2*h_Range) {
		t-=h_Range;
		for (int i=0;i<=h_PolynomDegree;i++) f=f*t+a[i+1]; //+1 because of numerical recipes standard
	} else {
		t-=3*h_Range;
		for (int i=h_PolynomDegree+1;i<=2*h_PolynomDegree+1;i++) f=f*t+a[i+1];
	}
	return f;
}

void GetSmoothCurvePointDouble(double t, double a[],double &x, double &y) {
	x=0;
	y=0;
	for (int i=0;i<=h_PolynomDegree;i++) x=x*t+a[i];
	for (i=h_PolynomDegree+1;i<=2*h_PolynomDegree+1;i++) y=y*t+a[i];	
}

CPoint GetSmoothCurvePoint(double t, double a[]) {
	double x=0;
	double y=0;
	for (int i=0;i<=h_PolynomDegree;i++) x=x*t+a[i];
	for (i=h_PolynomDegree+1;i<=2*h_PolynomDegree+1;i++) y=y*t+a[i];
	return CPoint(x,y);
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CSmoothCurve::CSmoothCurve()
{	
	Parameters=NULL;
	Range=1;
	PolynomDegree=0;
}

CSmoothCurve::CSmoothCurve(CSmoothCurve *Org)
{
	Range=Org->Range;
	PolynomDegree=Org->PolynomDegree;
	if (Org->Parameters) {
		Parameters=new double[2*(PolynomDegree+1)];
		for (int i=0;i<2*(PolynomDegree+1);i++) Parameters[i]=Org->Parameters[i];
	} else Parameters=NULL;	
}

CSmoothCurve::CSmoothCurve(CPoint Origin,CPoint Left, CPoint Right, CArray<POINT, POINT> *FrontierPoints,int aPolynomDegree,double &aRange)
//everything in Pixel coord
{
	SetFitFunc1D(&SmoothCurveFitFunction);
	PolynomDegree=aPolynomDegree;
	if (PolynomDegree<1) PolynomDegree=1;
	Range=0;
	double *datax;
	datax=new double [2*FrontierPoints->GetSize()];
	double *datay;
	datay=new double[2*FrontierPoints->GetSize()];		
	for (int i=0;i<FrontierPoints->GetSize();i++) {
		double x=(*FrontierPoints)[i].x;
		double y=(*FrontierPoints)[i].y;
		double dx=x-Origin.x;
		double dy=y-Origin.y;
		double d=dx*dx+dy*dy;
		if (d>0.0001) d=sqrt(d);
		if (d>Range) Range=d;
		dx=x-Left.x;
		dy=y-Left.y;
		double dl=dx*dx+dy*dy;
		if (dl>0.0001) dl=sqrt(dl);
		dx=x-Right.x;
		dy=y-Right.y;
		double dr=dx*dx+dy*dy;
		if (dr>0.0001) dr=sqrt(dr);						
		if (dl<dr) d*=-1;
		datax[i]=d;
		datay[i]=x;
		datax[i+FrontierPoints->GetSize()]=d;
		datay[i+FrontierPoints->GetSize()]=y;
	}
	
	for (i=FrontierPoints->GetSize();i<2*FrontierPoints->GetSize();i++) datax[i]+=3*Range; 
	for (i=0;i<FrontierPoints->GetSize();i++) datax[i]+=Range; 	
	double *a;
	a = new double[2*(PolynomDegree+1)];	
	for (i=0;i<2*(PolynomDegree+1);i++) a[i]=0;
	a[PolynomDegree]=Origin.x;
	a[2*PolynomDegree+1]=Origin.y;
	double mx=Right.x-Left.x;
	mx/=2*Range;
	double my=Right.y-Left.y;
	my/=2*Range;
	a[PolynomDegree-1]=mx;
	a[2*PolynomDegree]=my;	
	Parameters=new double[2*(PolynomDegree+1)];
	for (i=0;i<2*(PolynomDegree+1);i++) Parameters[i]=a[i];	
	
	/*
	
	ofstream out("c:\\Neuron\\data\\frontier.dat");
	for (int l=0;l<2*FrontierPoints->GetSize();l++) {
		out<<datax[l]<<" "<<datay[l]<<endl;
	}
	
	ofstream out2("c:\\Neuron\\data\\frontier2.dat");
	double t=-Range;
	double dt=Range/100;
	while (t<Range) {
		CPoint help=GetSmoothCurvePoint(t,Parameters);
		out2<<t<<" "<<help.x<<" "<<help.y<<endl;
		t+=dt;
	}

	ofstream out3("c:\\Neuron\\data\\frontier3.dat");
	for (l=0;l<2*FrontierPoints->GetSize();l++) {
		out3<<datax[l]<<" "<<SmoothCurveFitFunction(datax[l],Parameters)<<endl;
	} */

	double chisq;
	int nend;
	double changeend;
	h_PolynomDegree=PolynomDegree;
	h_Range=Range;
	double *v;
	v=new double[2*(PolynomDegree+1)];
	for (i=0;i<2*(PolynomDegree+1);i++) v[i]=1;
	bool matherror=fit(datax,	//x and y datapoints from 0 to ndata-1   //returns matherror
		 datay, 
		 FrontierPoints->GetSize()*2,		//number of datapoints
		 a,	//coefficients 0 to ma-1
		 v,	//should coefficient be fitted v=1 or treated as constant v=0  from 0 to ma-1
  		 2*(PolynomDegree+1),		//number of arguments
		 &chisq, //chisqu
		 50,		//maximum number of fitting rounds
		 2,		//number of fitting rounds after last that changed something in coefficients		 
		 0.0001,	//stop if chisqu<minchisqu		 
		 nend,			//return value: number of fitting rounds
		 changeend,	//return value: value of change during last round
		 &GeneralFitFunc1D  //fitting function			
	);
	if (!matherror) {
		for (int i=0;i<2*(PolynomDegree+1);i++) Parameters[i]=a[i];
	}
	delete a;
	delete datax;
	delete datay;
	delete v;
}

CSmoothCurve::~CSmoothCurve()
{
	if (Parameters) delete Parameters;
}

void CSmoothCurve::Draw(CDC *pDC,CBeamStop* BeamStop, CDataPointView *pView)
{
	double hRange=BeamStop->Range/2;
	BeamStop->ConvertCoordRealToPixel(hRange);
	if (hRange>Range) {
		double help=Range*2;
		BeamStop->ConvertCoordPixelToReal(help);
		BeamStop->Range=help;
		hRange=Range;
	}
	CPen penStroke;		
	penStroke.CreatePen(PS_SOLID, 2, RGB(255,255,0));		
	CPen* pOldPen = pDC->SelectObject(&penStroke);
	double t=-hRange;
	double dt=2.0*hRange/50.0;
	CPoint help=GetPoint(t);
//	BeamStop->ConvertCoordRealToPixel(help);
	pView->ConvertCoordsDocToView(help);
	pDC->MoveTo(help);
	while (t<=hRange) {		
		help=GetPoint(t);
	//	BeamStop->ConvertCoordRealToPixel(help);
		pView->ConvertCoordsDocToView(help);
		pDC->LineTo(help);
		t+=dt;
	}
	pDC->SelectObject(pOldPen);
}

CPoint CSmoothCurve::GetPoint(double t)  //t from -Range/2 to Range/2 in pixelcoord //return in pixelcoords
{
	h_PolynomDegree=PolynomDegree;
	h_Range=Range;
	return GetSmoothCurvePoint(t,Parameters);
}

void CSmoothCurve::Serialize(CArchive &ar)
{
	if (ar.IsStoring())
	{		
		ar<<Range;
		ar<<PolynomDegree;
		for (int i=0;i<2*(PolynomDegree+1);i++) ar<<Parameters[i];
	} else {
		ar>>Range;
		ar>>PolynomDegree;
		if (Parameters) delete Parameters;
		Parameters=new double[2*(PolynomDegree+1)];
		for (int i=0;i<2*(PolynomDegree+1);i++) ar>>Parameters[i];
	}
}

CPoint CSmoothCurve::GetTangent(double t) //t in nm  // returns tangent normalized to 1000
{
	double x1,x2,y1,y2;
	GetSmoothCurvePointDouble(t,Parameters,x1,y1);
	GetSmoothCurvePointDouble(t+1.0,Parameters,x2,y2);
	double tx=x2-x1;
	double ty=y2-y1;
	double l=tx*tx+ty*ty;
	if (l<0.00000000001) l=1;
	l=sqrt(l);
	tx=1000*tx/l;
	ty=1000*ty/l;
	return CPoint::CPoint(tx,ty);
}

CPoint CSmoothCurve::GetNormal(double t) //t in nm  // returns normal normalized to 1000
{
	double x1,x2,y1,y2;
	GetSmoothCurvePointDouble(t,Parameters,x1,y1);
	GetSmoothCurvePointDouble(t+1.0,Parameters,x2,y2);
	double tx=x2-x1;
	double ty=y2-y1;
	double l=tx*tx+ty*ty;
	if (l<0.00000000001) l=1;
	l=sqrt(l);
	tx=1000*tx/l;
	ty=1000*ty/l;
	return CPoint::CPoint(ty,-tx);
}
