#include "fit.h"
//#include <stdio.h>
//#include <window.h>
#include <math.h>
#include "stdafx.h"
#define NRANSI
//#include <stdlib.h >
#define FREE_ARG char*
#define NR_END 1

#define SWAP(a,b) {swap=(a);(a)=(b);(b)=swap;}

bool matherror=false;
bool TrapMathError=false;
bool MathErrorMessage=false;

double fabs(double a) { 
	if (a>0) return a; else return -1;
}

void SetMathErrorMessage( bool b) {
  MathErrorMessage=b;
}

int _matherr (struct exception *a)
{
  if (!TrapMathError) return 0;
  matherror=true;
  if (MathErrorMessage) return 0; else return 1;
}

int _matherrl(struct exceptionl *a)
{
  if (!TrapMathError) return 0;
  matherror=true;
  if (MathErrorMessage) return 0; else return 1;
}

void nrerror(char error_text[])
{
  if (MathErrorMessage) {
    char buf[300];
    sprintf(buf,"Numerical Recipes run-time error...\n%s\n",error_text);
    AfxMessageBox(buf);
  }
  matherror=true;
}

double **matrix(long nrl, long nrh, long ncl, long nch)
/* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	double **m;

	/* allocate pointers to rows */
	m=(double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m += NR_END;
	m -= nrl;

	/* allocate rows and set pointers to them */
	m[nrl]=(double *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double)));
	if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	/* return pointer to array of pointers to rows */
	return m;
}

void free_matrix(double **m, long nrl, long nrh, long ncl, long nch)
/* free a double matrix allocated by matrix() */
{
	free((FREE_ARG) (m[nrl]+ncl-NR_END));
	free((FREE_ARG) (m+nrl-NR_END));
}

double *vector(long nl, long nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
	double *v;

	v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl+NR_END;
}

void free_vector(double *v, long nl, long nh)
/* free a double vector allocated with vector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

int *ivector(long nl, long nh)
/* allocate an int vector with subscript range v[nl..nh] */
{
	int *v;

	v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
	if (!v) nrerror("allocation failure in ivector()");
	return v-nl+NR_END;
}

void free_ivector(int *v, long nl, long nh)
/* free an int vector allocated with ivector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

void gaussj(double **a, int n, double **b, int m)
{
	int *indxc,*indxr,*ipiv;
	int i,icol,irow,j,k,l,ll;
	double big,dum,pivinv,swap;

	indxc=ivector(1,n);
	indxr=ivector(1,n);
	ipiv=ivector(1,n);
	for (j=1;j<=n;j++) ipiv[j]=0;
	for (i=1;i<=n;i++) {
		big=0.0;
		for (j=1;j<=n;j++)
			if (ipiv[j] != 1)
				for (k=1;k<=n;k++) {
					if (ipiv[k] == 0) {
						if (fabs(a[j][k]) >= big) {
							big=fabs(a[j][k]);
							irow=j;
							icol=k;
						}
					} else if (ipiv[k] > 1) nrerror("gaussj: Singular Matrix-1");
				}
		++(ipiv[icol]);
		if (irow != icol) {
			for (l=1;l<=n;l++) SWAP(a[irow][l],a[icol][l])
			for (l=1;l<=m;l++) SWAP(b[irow][l],b[icol][l])
		}
		indxr[i]=irow;
		indxc[i]=icol;
		if (a[icol][icol] == 0.0) nrerror("gaussj: Singular Matrix-2");
		if (a[icol][icol]!=0) pivinv=1.0/a[icol][icol]; else pivinv=1E10;
		a[icol][icol]=1.0;
		for (l=1;l<=n;l++) a[icol][l] *= pivinv;
		for (l=1;l<=m;l++) b[icol][l] *= pivinv;
		for (ll=1;ll<=n;ll++)
			if (ll != icol) {
				dum=a[ll][icol];
				a[ll][icol]=0.0;
				for (l=1;l<=n;l++) a[ll][l] -= a[icol][l]*dum;
				for (l=1;l<=m;l++) b[ll][l] -= b[icol][l]*dum;
			}
	}
	for (l=n;l>=1;l--) {
		if (indxr[l] != indxc[l])
			for (k=1;k<=n;k++)
				SWAP(a[k][indxr[l]],a[k][indxc[l]]);
	}
	free_ivector(ipiv,1,n);
	free_ivector(indxr,1,n);
	free_ivector(indxc,1,n);
}

void covsrt(double **covar, int ma, int ia[], int mfit)
{
	int i,j,k;
	double swap;

	for (i=mfit+1;i<=ma;i++)
		for (j=1;j<=i;j++) covar[i][j]=covar[j][i]=0.0;
	k=mfit;
	for (j=ma;j>=1;j--) {
		if (ia[j]) {
			for (i=1;i<=ma;i++) SWAP(covar[i][k],covar[i][j])
			for (i=1;i<=ma;i++) SWAP(covar[k][i],covar[j][i])
			k--;
		}
	}
}

void mrqcof(double x[], double y[], double sig[], int ndata, double a[], int ia[],
	int ma, double **alpha, double beta[], double *chisq,
	void (*funcs)(double, double [], double *, double [], int))
{
	int i,j,k,l,m,mfit=0;
	double ymod,wt,sig2i,dy,*dyda;

	dyda=vector(1,ma);
	for (j=1;j<=ma;j++)
		if (ia[j]) mfit++;
	for (j=1;j<=mfit;j++) {
		for (k=1;k<=j;k++) alpha[j][k]=0.0;
		beta[j]=0.0;
	}
	*chisq=0.0;
	for (i=1;i<=ndata;i++) {
		if (!matherror) (*funcs)(x[i],a,&ymod,dyda,ma);
      if (matherror) {
        free_vector(dyda,1,ma);
        return;
      }
		if (sig[i]!=0) sig2i=1.0/(sig[i]*sig[i]); else sig2i=1E10;
		dy=y[i]-ymod;
		for (j=0,l=1;l<=ma;l++) {
			if (ia[l]) {
				wt=dyda[l]*sig2i;
				for (j++,k=0,m=1;m<=l;m++)
					if (ia[m]) alpha[j][++k] += wt*dyda[m];
				beta[j] += dy*wt;
			}
		}
		*chisq += dy*dy*sig2i;
	}
	for (j=2;j<=mfit;j++)
		for (k=1;k<j;k++) alpha[k][j]=alpha[j][k];
	free_vector(dyda,1,ma);
}

void mrqmin(double x[], double y[], double sig[], int ndata, double a[], int ia[],
	int ma, double **covar, double **alpha, double *chisq,
	void (*funcs)(double, double [], double *, double [], int), double *alamda)
{
	void covsrt(double **covar, int ma, int ia[], int mfit);
	void gaussj(double **a, int n, double **b, int m);
	void mrqcof(double x[], double y[], double sig[], int ndata, double a[],
		int ia[], int ma, double **alpha, double beta[], double *chisq,
		void (*funcs)(double, double [], double *, double [], int));
	int j,k,l,m;
	static int mfit;
	static double ochisq,*atry,*beta,*da,**oneda;

	if (*alamda < 0.0) {
		atry=vector(1,ma);
		beta=vector(1,ma);
		da=vector(1,ma);
		for (mfit=0,j=1;j<=ma;j++)
			if (ia[j]) mfit++;
		oneda=matrix(1,mfit,1,1);
		*alamda=0.001;
		mrqcof(x,y,sig,ndata,a,ia,ma,alpha,beta,chisq,funcs);
		ochisq=(*chisq);
		for (j=1;j<=ma;j++) atry[j]=a[j];
	}
	for (j=0,l=1;l<=ma;l++) {
		if (ia[l]) {
			for (j++,k=0,m=1;m<=ma;m++) {
				if (ia[m]) {
					k++;
					covar[j][k]=alpha[j][k];
				}
			}
			covar[j][j]=alpha[j][j]*(1.0+(*alamda));
			oneda[j][1]=beta[j];
		}
	}
	if (!matherror) {
      gaussj(covar,mfit,oneda,1);
      for (j=1;j<=mfit;j++) da[j]=oneda[j][1];
      if (*alamda == 0.0) {
         if (!matherror) covsrt(covar,ma,ia,mfit);
         free_matrix(oneda,1,mfit,1,1);
         free_vector(da,1,ma);
         free_vector(beta,1,ma);
         free_vector(atry,1,ma);
         return;
      }
      for (j=0,l=1;l<=ma;l++)
         if (ia[l]) atry[l]=a[l]+da[++j];
      if (!matherror) mrqcof(x,y,sig,ndata,atry,ia,ma,covar,da,chisq,funcs);
      if (matherror) {
         free_matrix(oneda,1,mfit,1,1);
         free_vector(da,1,ma);
         free_vector(beta,1,ma);
         free_vector(atry,1,ma);
         return;
      }
      if (*chisq < ochisq) {
         *alamda *= 0.1;
         ochisq=(*chisq);
         for (j=0,l=1;l<=ma;l++) {
            if (ia[l]) {
               for (j++,k=0,m=1;m<=ma;m++) {
                  if (ia[m]) {
                     k++;
                     alpha[j][k]=covar[j][k];
                  }
               }
               beta[j]=da[j];
               a[l]=atry[l];
            }
         }
      } else {
         *alamda *= 10.0;
         *chisq=ochisq;
      }
   }
}

#define CON 1.4
#define CON2 (CON*CON)
#define BIG 1.0e30
#define NTAB 10
#define SAFE 2.0
static float maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ?\
        (maxarg1) : (maxarg2))

double dfridr(double (*func)(double), double x, double h, double *err)
{
	int i,j;
	double errt,fac,hh,**a,ans;
   ans=0;
	if (h == 0.0) nrerror("h must be nonzero in dfridr.");
	a=matrix(1,NTAB,1,NTAB);
	hh=h;
	a[1][1]=((*func)(x+hh)-(*func)(x-hh))/(2.0*hh);
   if (matherror) {
     free_matrix(a,1,NTAB,1,NTAB);
     return ans;
   }
	*err=BIG;
	for (i=2;i<=NTAB;i++) {
		hh /= CON;
		a[1][i]=((*func)(x+hh)-(*func)(x-hh))/(2.0*hh);
      if (matherror) {
        free_matrix(a,1,NTAB,1,NTAB);
        return ans;
      }
		fac=CON2;
		for (j=2;j<=i;j++) {
			a[j][i]=(a[j-1][i]*fac-a[j-1][i-1])/(fac-1.0);
			fac=CON2*fac;
			errt=FMAX(fabs(a[j][i]-a[j-1][i]),fabs(a[j][i]-a[j-1][i-1]));
			if (errt <= *err) {
				*err=errt;
				ans=a[j][i];
			}
		}
		if (fabs(a[i][i]-a[i-1][i-1]) >= SAFE*(*err)) break;
	}
	free_matrix(a,1,NTAB,1,NTAB);
	return ans;
}

#undef CON
#undef CON2
#undef BIG
#undef NTAB
#undef SAFE
#undef NRANSI

double (*FitFunc1D)(double, double []);  //set my fit function: func(x,a): x=f(a);

void SetFitFunc1D(double (*func)(double, double [])) {
  FitFunc1D=func;
}

double aktParams[100];
int aktDeriveParam;
double aktX;
double DFitFunc1D(double P) {	//just to make it compatible with dfridr
  aktParams[aktDeriveParam]=P;
  return FitFunc1D(aktX,aktParams);
}

void GeneralFitFunc1D(double x, double a[], double *y, double dyda[], int na)
{
  aktX=x;
  *y=FitFunc1D(x,a);
  if (matherror) return;
  for (int j=1;j<=na;j++) {
    double err;
    for (int i=1;i<=na;i++) aktParams[i]=a[i];
    aktDeriveParam=j;
    dyda[j]=dfridr(DFitFunc1D,a[j],1E-4,&err);
    if (matherror) return;
  }
}

bool fit(double x[],	//x and y datapoints from 0 to ndata-1   //returns matherror
		 double y[], 
		 int ndata,		//number of datapoints
		 double a[],	//coefficients 0 to ma-1
		 double v[],	//should coefficient be fitted v=1 or treated as constant v=0  from 0 to ma-1
  		 int ma,		//number of arguments
		 double *chisq, //chisqu
		 int maxn,		//maximum number of fitting rounds
		 int minn,		//number of fitting rounds after last that changed something in coefficients		 
		 double minchisq,	//stop if chisqu<minchisqu		 
		 int &nend,			//return value: number of fitting rounds
		 double &changeend,	//return value: value of change during last round
		 void (*funcs)(  //fitting function
			double,			//point at which to evaluate
			double [],		//coefficients
			double *,		//return value
			double [],		//return derivatives
			int)			//number of coefficients
	) {
	double ** covar=matrix(1,ma,1,ma);
	double **alpha=matrix(1,ma,1,ma);
	int *ia=ivector(1,ma);
	for (int i=1;i<=ma;i++) ia[i]=v[i-1];
	double *nx=vector(1,ndata);
	for (i=1;i<=ndata;i++) nx[i]=x[i-1];
	double *ny=vector(1,ndata);
	for (i=1;i<=ndata;i++) ny[i]=y[i-1];
	double *nsig=vector(1,ndata);
	for (i=1;i<=ndata;i++) nsig[i]=1;
	double *na=vector(1,ma);
	for (i=1;i<=ma;i++) na[i]=a[i-1];
	double lamda=-1;
	double *alamda=&lamda;
	int n=0;
   matherror=false;
   TrapMathError=true;
   double *alta=new double[ma+1];
   double change;
   for (i=1;i<=ma;i++) alta[i]=na[i];
   int lastchange=0;
	do {
	  mrqmin(nx,ny,nsig,ndata,na,ia,ma,covar,alpha,chisq,funcs,alamda);
     change=0;
     if (!matherror) {
       for (int j=1;j<=ma;j++) {
         if (na[j]!=0) change+=((alta[j]-na[j])*(alta[j]-na[j])/(na[j]*na[j]));
         alta[j]=na[j];
       }
       if (change>0) change=change/ma;
       if (change==0) lastchange++; else lastchange=0;
     }
	  n++;
	} while ((n<maxn) && (*chisq>minchisq) && (!matherror) && (lastchange<=minn));
   changeend=change;
   nend=n;
   delete alta;
	lamda=0;
	if (!matherror) {
     mrqmin(nx,ny,nsig,ndata,na,ia,ma,covar,alpha,chisq,funcs,alamda);
     for (int i=1;i<=ma;i++) a[i-1]=na[i];
   }
   TrapMathError=false;
	free_matrix(covar,1,ma,1,ma);
	free_matrix(alpha,1,ma,1,ma);
	free_ivector(ia,1,ma);
	free_vector(na,1,ma);
	free_vector(nx,1,ndata);
	free_vector(ny,1,ndata);
	free_vector(nsig,1,ndata);
   return (matherror);
}

#undef NRANSI
