/*

  Copyright (c) 2000, 2002 Finger Lakes Instrumentation (FLI), L.L.C.
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

        Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

        Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.

        Neither the name of Finger Lakes Instrumentation (FLI), LLC
        nor the names of its contributors may be used to endorse or
        promote products derived from this software without specific
        prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

  ======================================================================

  Finger Lakes Instrumentation, L.L.C. (FLI)
  web: http://www.fli-cam.com
  email: support@fli-cam.com

*/

#ifndef __fli__
#define __fli__

#ifndef LIBFLIAPI
#define LIBFLIAPI __declspec(dllimport)
#endif LIBFLIAPI

typedef long flidev_t;
typedef long fliframe_t;
typedef long flibitdepth_t;
typedef long flidomain_t;

#define FLIDOMAIN_NONE (0x00)
#define FLIDOMAIN_PARALLEL_PORT (0x01)
#define FLIDOMAIN_USB (0x02)
#define FLIDOMAIN_SERIAL (0x03)
#define FLIDOMAIN_INET (0x04)

#define FLIDEVICE_NONE (0x000)
#define FLIDEVICE_CAMERA (0x100)
#define FLIDEVICE_FILTERWHEEL (0x200)
#define FLIDEVICE_FOCUSER (0x300)

#define FLI_FRAME_TYPE_NORMAL (0)
#define FLI_FRAME_TYPE_DARK (1)

#define FLI_FILTERPOSITION_HOME (-1)

#define FLI_SHUTTER_CLOSE	(0)
#define FLI_SHUTTER_OPEN (1)
#define FLI_SHUTTER_EXTERNAL_TRIGGER (2)

/* I/O Bit definitions */

#define FLICCD_IO_P0	(0x01)
#define FLICCD_IO_P1	(0x02)
#define FLICCD_IO_P2	(0x04)
#define FLICCD_IO_P3	(0x08)

typedef enum {
  FLI_MODE_8BIT = 0,
  FLI_MODE_16BIT
} _flibitdepth_t;

LIBFLIAPI long __stdcall FLIOpen(flidev_t *dev, char *name, flidomain_t domain);
LIBFLIAPI long __stdcall FLIClose(flidev_t dev);
LIBFLIAPI long __stdcall FLISetDebugLevel(char *host, long level);
LIBFLIAPI long __stdcall FLIGetLibVersion(char* ver, size_t len);
LIBFLIAPI long __stdcall FLIGetModel(flidev_t dev, char* model, size_t len);
LIBFLIAPI long __stdcall FLIGetArrayArea(flidev_t dev, long* ul_x, long* ul_y, long* lr_x, long* lr_y);
LIBFLIAPI long __stdcall FLIGetVisibleArea(flidev_t dev, long* ul_x, long* ul_y, long* lr_x, long* lr_y);
LIBFLIAPI long __stdcall FLIExposeFrame(flidev_t dev);
LIBFLIAPI long __stdcall FLICancelExposure(flidev_t dev);
LIBFLIAPI long __stdcall FLIGetExposureStatus(flidev_t dev, long* timeleft);
LIBFLIAPI long __stdcall FLISetTemperature(flidev_t dev, double  temperature);
LIBFLIAPI long __stdcall FLIGetTemperature(flidev_t dev, double* temperature);
LIBFLIAPI long __stdcall FLIGrabRow(flidev_t dev, void* buff, size_t width);
LIBFLIAPI long __stdcall FLIGrabFrame(flidev_t dev, void* buff, size_t buffsize, size_t* bytesgrabbed);
LIBFLIAPI long __stdcall FLIFlushRow(flidev_t dev, long rows, long repeat);
LIBFLIAPI long __stdcall FLISetExposureTime(flidev_t dev, long exptime);
LIBFLIAPI long __stdcall FLISetFrameType (flidev_t dev, fliframe_t frametype);
LIBFLIAPI long __stdcall FLISetImageArea (flidev_t dev, long ul_x, long ul_y, long lr_x, long lr_y);
LIBFLIAPI long __stdcall FLISetHBin(flidev_t dev, long hbin);
LIBFLIAPI long __stdcall FLISetVBin(flidev_t dev, long hbin);
LIBFLIAPI long __stdcall FLISetNFlushes(flidev_t dev, long nflushes);
LIBFLIAPI long __stdcall FLISetBitDepth(flidev_t dev, flibitdepth_t bitdepth);
LIBFLIAPI long __stdcall FLIReadIOPort(flidev_t dev, long *ioportset);
LIBFLIAPI long __stdcall FLIWriteIOPort(flidev_t dev, long ioportset);
LIBFLIAPI long __stdcall FLIConfigureIOPort(flidev_t dev, long ioportset);
LIBFLIAPI long __stdcall FLIControlShutter(flidev_t dev, long shutter);
LIBFLIAPI long __stdcall FLILockDevice(flidev_t dev);
LIBFLIAPI long __stdcall FLIUnlockDevice(flidev_t dev);
LIBFLIAPI long __stdcall FLIList(flidomain_t domain, char ***names);
LIBFLIAPI long __stdcall FLIFreeList(char **names);
LIBFLIAPI long __stdcall FLISetFilterPos(flidev_t dev, long filter);
LIBFLIAPI long __stdcall FLIGetFilterPos(flidev_t dev, long* filter);
LIBFLIAPI long __stdcall FLIGetFilterCount(flidev_t dev, long* filter);
LIBFLIAPI long __stdcall FLIStepMotor(flidev_t dev, long steps);
LIBFLIAPI long __stdcall FLIGetStepperPosition(flidev_t, long *position);
LIBFLIAPI long __stdcall FLIHomeFocuser(flidev_t dev);
LIBFLIAPI long __stdcall FLIGetHWRevision(flidev_t dev, long *hwrev);
LIBFLIAPI long __stdcall FLIGetPixelSize(flidev_t dev, double *pixel_x, double *pixel_y);
LIBFLIAPI long __stdcall FLIGetFWRevision(flidev_t dev, long *fwrev);
#endif __fli__
